@@ -205,6 +205,10 @@
 #    define HAVE_GRANTPT
 #endif
 
+#if defined(HAVE_LIBCRYPT) || defined(HAVE_LIBCRYPT_I)
+#    define HAVE_CRYPT
+#endif
+
 #ifdef HAVE_XVIEW
 #    include <xvmain.h>
 #endif
