@@ -3,100 +3,108 @@
 
 #ifdef MIDNIGHT
 
-#include <stdio.h>
-#include <stdarg.h>
-#include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <string.h>
-#include "src/tty.h"
-#include <sys/stat.h>
-
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-
-#include <stdlib.h>
-#include <malloc.h>
+#    include <stdio.h>
+#    include <stdarg.h>
+#    include <sys/types.h>
+#    ifdef HAVE_UNISTD_H
+#    	 include <unistd.h>
+#    endif
+#    include <string.h>
+#    include "src/tty.h"
+#    include <sys/stat.h>
+#    include <errno.h>
+     
+#    ifdef HAVE_FCNTL_H
+#        include <fcntl.h>
+#    endif
+     
+#    include <stdlib.h>
+#    include <malloc.h>
 
 #else
 
-#include "global.h"
-#include <stdio.h>
-#include <stdarg.h>
-#include <sys/types.h>
+#    include "global.h"
+#    include <stdio.h>
+#    include <stdarg.h>
+#    include <sys/types.h>
+     
+#    	 ifdef HAVE_UNISTD_H
+#    	     include <unistd.h>
+#    	 endif
+     
+#    include <my_string.h>
+#    include <sys/stat.h>
+     
+#    ifdef HAVE_FCNTL_H
+#    	 include <fcntl.h>
+#    endif
+     
+#    include <stdlib.h>
+#    include <stdarg.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <my_string.h>
-#include <sys/stat.h>
-
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-
-#include <stdlib.h>
-#include <stdarg.h>
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
-#include "regex.h"
+#    if TIME_WITH_SYS_TIME
+#    	 include <sys/time.h>
+#    	 include <time.h>
+#    else
+#    	 if HAVE_SYS_TIME_H
+#    	     include <sys/time.h>
+#    	 else
+#    	     include <time.h>
+#    	 endif
+#    endif
+ 
+#    include "regex.h"
 
 #endif
 
 #ifndef MIDNIGHT
 
-#include <signal.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xresource.h>
-#include "lkeysym.h"
-#include "coolwidget.h"
-#include "app_glob.c"
-#include "coollocal.h"
-#include "dialog.h"
-#include "stringtools.h"
+#    include <signal.h>
+#    include <X11/Xlib.h>
+#    include <X11/Xutil.h>
+#    include <X11/Xresource.h>
+#    include "lkeysym.h"
+#    include "coolwidget.h"
+#    include "app_glob.c"
+#    include "coollocal.h"
+#    include "stringtools.h"
 
 #else
 
-#include "src/main.h"		/* for char *shell */
-#include "src/mad.h"
-#include "src/dlg.h"
-#include "src/widget.h"
-#include "src/color.h"
-#include "src/dialog.h"
-#include "src/mouse.h"
-#include "src/global.h"
-#include "src/help.h"
-#include "src/key.h"
-#include "src/wtools.h"		/* for QuickWidgets */
-#include "src/win.h"
-#include "vfs/vfs.h"
-#include "src/menu.h"
-#include "src/regex.h"
-#define WANT_WIDGETS
-
-#define WIDGET_COMMAND (WIDGET_USER + 10)
-#define N_menus 5
+#    include "src/main.h"		/* for char *shell */
+#    include "src/mad.h"
+#    include "src/dlg.h"
+#    include "src/widget.h"
+#    include "src/color.h"
+#    include "src/dialog.h"
+#    include "src/mouse.h"
+#    include "src/global.h"
+#    include "src/help.h"
+#    include "src/key.h"
+#    include "src/wtools.h"		/* for QuickWidgets */
+#    include "src/win.h"
+#    include "vfs/vfs.h"
+#    include "src/menu.h"
+#    include "src/regex.h"
+#    define WANT_WIDGETS
+     
+#    define WIDGET_COMMAND (WIDGET_USER + 10)
+#    define N_menus 5
 
 #endif
 
 #define SEARCH_DIALOG_OPTION_NO_SCANF	1
 #define SEARCH_DIALOG_OPTION_NO_REGEX	2
 #define SEARCH_DIALOG_OPTION_NO_CASE	4
+#define SEARCH_DIALOG_OPTION_BACKWARDS	8
 
-#define CLIP_FILE "/cedit/cooledit.clip"
-#define MACRO_FILE "/cedit/cooledit.macros"
-#define BLOCK_FILE "/cedit/cooledit.block"
-#define ERROR_FILE "/cedit/cooledit.error"
-#define TEMP_FILE "/cedit/cooledit.temp"
-#define SCRIPT_FILE "/cedit/cooledit.script"
-#define EDIT_DIR "/cedit"
+#define CLIP_FILE "/.cedit/cooledit.clip"
+#define MACRO_FILE "/.cedit/cooledit.macros"
+#define BLOCK_FILE "/.cedit/cooledit.block"
+#define ERROR_FILE "/.cedit/cooledit.error"
+#define TEMP_FILE "/.cedit/cooledit.temp"
+#define SCRIPT_FILE "/.cedit/cooledit.script"
+#define EDIT_DIR "/.cedit"
 
 #define EDIT_KEY_EMULATION_NORMAL 0
 #define EDIT_KEY_EMULATION_EMACS  1
@@ -111,23 +119,23 @@
 #define REDRAW_CHAR_ONLY     (1 << 7)
 #define REDRAW_COMPLETELY    (1 << 8)
 
-#define MOD_ABNORMAL 1
-#define MOD_UNDERLINED 2
-#define MOD_BOLD 4
-#define MOD_HIGHLIGHTED 8
-#define MOD_MARKED 16
-#define MOD_ITALIC 32
-#define MOD_CURSOR 64
+#define MOD_ABNORMAL		(1 << 0)
+#define MOD_UNDERLINED		(1 << 1)
+#define MOD_BOLD		(1 << 2)
+#define MOD_HIGHLIGHTED		(1 << 3)
+#define MOD_MARKED		(1 << 4)
+#define MOD_ITALIC		(1 << 5)
+#define MOD_CURSOR		(1 << 6)
+#define MOD_INVERSE		(1 << 7)
 
 #ifndef MIDNIGHT
-#define EDIT_TEXT_HORIZONTAL_OFFSET 4
-#define EDIT_TEXT_VERTICAL_OFFSET 3
+#    define EDIT_TEXT_HORIZONTAL_OFFSET 4
+#    define EDIT_TEXT_VERTICAL_OFFSET 3
 #else
-#define EDIT_TEXT_HORIZONTAL_OFFSET 0
-#define EDIT_TEXT_VERTICAL_OFFSET 1
-#define FONT_OFFSET_X 0
-#define FONT_OFFSET_Y 0
-
+#    define EDIT_TEXT_HORIZONTAL_OFFSET 0
+#    define EDIT_TEXT_VERTICAL_OFFSET 1
+#    define FONT_OFFSET_X 0
+#    define FONT_OFFSET_Y 0
 #endif
 
 #define EDIT_RIGHT_EXTREME option_edit_right_extreme
@@ -152,11 +160,10 @@
 #define SIZE_LIMIT (EDIT_BUF_SIZE * (MAXBUFF - 2))
 /* Note a 16k stack is 64k of data and enough to hold (usually) around 10
    pages of undo info. */
-#define STACK_BITS 14
-#define MAX_STACK (1<<(STACK_BITS-1))	/* the most that one command can occupy without giving a warning */
-#define STACK_SIZE (1<<STACK_BITS)
-#define M_STACK_SIZE (STACK_SIZE-1)
-/*.. used same as M_EDIT_BUF_SIZE */
+
+/* undo stack */
+#define START_STACK_SIZE 32
+
 
 /*some codes that may be pushed onto or returned from the undo stack: */
 #define CURS_LEFT 601
@@ -224,6 +231,11 @@ struct editor_widget {
     unsigned char overwrite;
     unsigned char modified;	/*has the file been changed?: 1 if char inserted or
 				   deleted at all since last load or save */
+#ifdef MIDNIGHT
+    int delete_file;			/* has the file been created in edit_load_file? Delete
+			           it at end of editing when it hasn't been modified 
+				   or saved */
+#endif				   
     unsigned char highlight;
     long prev_col;		/*recent column position of the curser - used when moving
 				   up or down past lines that are shorter than the current line */
@@ -234,11 +246,15 @@ struct editor_widget {
     long total_lines;		/*total lines in the file */
     long mark1;			/*position of highlight start */
     long mark2;			/*position of highlight end */
+    int column1;			/*position of column highlight start */
+    int column2;			/*position of column highlight end */
     long bracket;		/*position of a matching bracket */
 
 /* undo stack and pointers */
     unsigned long stack_pointer;
     long *undo_stack;
+    unsigned long stack_size;
+    unsigned long stack_size_mask;
     unsigned long stack_bottom;
     struct stat stat;
 
@@ -282,17 +298,18 @@ void edit_init_menu_normal (void);
 void edit_done_menu (void);
 int edit_raw_key_query (char *heading, char *query, int cancel);
 char *strcasechr (const unsigned char *s, int c);
-int edit (const char *_file);
+int edit (const char *_file, int line);
+int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch);
 
 #endif
 
 int edit_get_byte (WEdit * edit, long byte_index);
-int load_edit_file (WEdit * edit, const char *filename, const char *text);
+char *edit_get_buffer_as_text (WEdit * edit);
+int edit_load_file (WEdit * edit, const char *filename, const char *text, unsigned long text_size);
 int edit_count_lines (WEdit * edit, long current, int upto);
 long edit_move_forward (WEdit * edit, long current, int lines, long upto);
 long edit_move_forward3 (WEdit * edit, long current, int cols, long upto);
 long edit_move_backward (WEdit * edit, long current, int lines);
-int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch);
 void edit_scroll_screen_over_cursor (WEdit * edit);
 void edit_render_keypress (WEdit * edit);
 void edit_scroll_upward (WEdit * edit, unsigned long i);
@@ -302,8 +319,8 @@ void edit_scroll_left (WEdit * edit, int i);
 int edit_get_col (WEdit * edit);
 long edit_bol (WEdit * edit, long current);
 long edit_eol (WEdit * edit, long current);
-void update_curs_row (WEdit * edit);
-void update_curs_col (WEdit * edit);
+void edit_update_curs_row (WEdit * edit);
+void edit_update_curs_col (WEdit * edit);
 
 void edit_block_copy_cmd (WEdit * edit);
 void edit_block_move_cmd (WEdit * edit);
@@ -312,21 +329,21 @@ int edit_block_delete_cmd (WEdit * edit);
 int edit_delete (WEdit * edit);
 void edit_insert (WEdit * edit, int c);
 int edit_cursor_move (WEdit * edit, long increment);
-void push_action (WEdit * edit, long c,...);
-void push_key_press (WEdit * edit);
+void edit_push_action (WEdit * edit, long c,...);
+void edit_push_key_press (WEdit * edit);
 void edit_insert_ahead (WEdit * edit, int c);
 int edit_save_file (WEdit * edit, const char *filename);
 int edit_save_cmd (WEdit * edit);
 int edit_save_confirm_cmd (WEdit * edit);
 int edit_save_as_cmd (WEdit * edit);
-WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const char *text, const char *dir);
+WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, const char *text, const char *dir, unsigned long text_size);
 int edit_clean (WEdit * edit);
 int edit_renew (WEdit * edit);
 int edit_new_cmd (WEdit * edit);
-int edit_reload (WEdit * edit, const char *filename, const char *text, const char *dir);
+int edit_reload (WEdit * edit, const char *filename, const char *text, const char *dir, unsigned long text_size);
 int edit_load_cmd (WEdit * edit);
 void edit_mark_cmd (WEdit * edit, int unmark);
-void edit_set_markers (WEdit * edit, long m1, long m2);
+void edit_set_markers (WEdit * edit, long m1, long m2, int c1, int c2);
 void edit_push_markers (WEdit * edit);
 void edit_quit_cmd (WEdit * edit);
 void edit_replace_cmd (WEdit * edit, int again);
@@ -341,15 +358,15 @@ void edit_date_cmd (WEdit * edit);
 void edit_goto_cmd (WEdit * edit);
 int eval_marks (WEdit * edit, long *start_mark, long *end_mark);
 void edit_status (WEdit * edit);
-int Cedit_execute_command (WEdit * edit, int command, int char_for_insertion);
+int edit_execute_command (WEdit * edit, int command, int char_for_insertion);
 int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion);
-void Cedit_update_screen (WEdit * edit);
+void edit_update_screen (WEdit * edit);
 int edit_printf (WEdit * e, const char *fmt,...);
 int edit_print_string (WEdit * e, const char *s);
 void edit_move_to_line (WEdit * e, long line);
 void edit_move_display (WEdit * e, long line);
 void edit_word_wrap (WEdit * edit);
-unsigned char *edit_get_block (WEdit * edit, long start, long finish);
+unsigned char *edit_get_block (WEdit * edit, long start, long finish, int *l);
 int edit_sort_cmd (WEdit * edit);
 void edit_help_cmd (WEdit * edit);
 void edit_left_word_move (WEdit * edit);
@@ -362,57 +379,85 @@ void edit_delete_macro_cmd (WEdit * edit);
 
 int edit_copy_to_X_buf_cmd (WEdit * edit);
 int edit_cut_to_X_buf_cmd (WEdit * edit);
-void paste_from_X_buf_cmd (WEdit * edit);
+void edit_paste_from_X_buf_cmd (WEdit * edit);
 
 void edit_paste_from_history (WEdit *edit);
 
-void split_filename (WEdit * edit, char *name);
+void edit_split_filename (WEdit * edit, char *name);
 
 #ifdef MIDNIGHT
 
+#    ifndef _
+#        define _(s) s
+#    endif
+
+
+#    define gettext(s) (s)
+#    define N_(s) s
+#    define gettext_noop(s) (s)
+#    endif
+
 /* put OS2/NT/WIN95 defines here */
 
-#define TEXT_PIX_PER_LINE 1
-#define TEXT_M_WIDTH 1
+#    ifdef OS2_NT
+#    	 define MY_O_TEXT O_TEXT
+#    else
+#    	 define MY_O_TEXT 0
+#    endif
 
-#define get_sys_error(s) (s)
-#define open mc_open
-#define close(f) mc_close(f)
-#define read(f,b,c) mc_read(f,b,c)
-#define write(f,b,c) mc_write(f,b,c)
-#define stat(f,s) mc_stat(f,s)
-#if defined(OS2_NT)
-#  define mkdir(s,m) mc_mkdir(s)
-#else
-#  define mkdir(s,m) mc_mkdir(s,m)
-#endif
-#define itoa MY_itoa
+#    define FONT_PIX_PER_LINE 1
+#    define FONT_MEAN_WIDTH 1
+     
+#    define get_sys_error(s) (s)
+#    define open mc_open
+#    define close(f) mc_close(f)
+#    define read(f,b,c) mc_read(f,b,c)
+#    define write(f,b,c) mc_write(f,b,c)
+#    define stat(f,s) mc_stat(f,s)
+#    if defined(OS2_NT)
+#         define mkdir(s,m) mc_mkdir(s)
+#    else
+#         define mkdir(s,m) mc_mkdir(s,m)
+#    endif
+#    define itoa MY_itoa
 
-#define Cgetloadfile(w,x,y,d,f,h) input_dialog (h, " Enter file name: ", f)
-#define Cgetsavefile(w,x,y,d,f,h) input_dialog (h, " Enter file name: ", f)
-#define Cmalloc(x) malloc(x)
+#    define edit_get_load_file(d,f,h) input_dialog (h, " Enter file name: ", f)
+#    define edit_get_save_file(d,f,h) input_dialog (h, " Enter file name: ", f)
+#    define CMalloc(x) malloc(x)
+     
+#    define set_error_msg(s) edit_init_error_msg = strdup(s)
 
-#define set_error_msg(s) edit_init_error_msg = strdup(s)
+#    ifdef _EDIT_C
 
-#ifdef _EDIT_C
-
-#define Cerrordialogue(w,x,y,h,s) set_error_msg(s)
+#         define edit_error_dialog(h,s) set_error_msg(s)
 char *edit_init_error_msg = NULL;
-#else				/* ! _EDIT_C */
 
-#define Cerrordialogue(w,x,y,h,s) query_dialog (h, s, 0, 1, " Cancel ")
-#define Cmessagedialogue(w,x,y,h,s) query_dialog (h, s, 0, 1, " Ok ")
+#    else				/* ! _EDIT_C */
+
+#    define edit_error_dialog(h,s) query_dialog (h, s, 0, 1, "Cancel")
+#    define edit_message_dialog(h,s) query_dialog (h, s, 0, 1, "Ok")
 extern char *edit_init_error_msg;
 
-#endif				/* ! _EDIT_C */
+#    endif				/* ! _EDIT_C */
 
-#define get_error_msg(s) edit_init_error_msg
+
+#    define get_error_msg(s) edit_init_error_msg
+#    define edit_query_dialog2(h,t,a,b) query_dialog(h,t,0,2,a,b)
+#    define edit_query_dialog3(h,t,a,b,c) query_dialog(h,t,0,3,a,b,c)
+#    define edit_query_dialog4(h,t,a,b,c,d) query_dialog(h,t,0,4,a,b,c,d)
 
 #else				/* ! MIDNIGHT */
 
-#define Cerrordialogue(w,x,y,h,s) Cerrordialog(w,x,y,h,s)
-#define Cmessagedialogue(w,x,y,h,s) Cmessagedialog(w,x,y,0,h,s)
-#define Cquerydialogue Cquerydialog
+#    define MY_O_TEXT 0
+#    define WIN_MESSAGES edit->widget->mainid, 20, 20
+     
+#    define edit_get_load_file(d,f,h) CGetLoadFile(WIN_MESSAGES,d,f,h)
+#    define edit_get_save_file(d,f,h) CGetSaveFile(WIN_MESSAGES,d,f,h)
+#    define edit_error_dialog(h,t) CErrorDialog(WIN_MESSAGES,h,"%s",t)
+#    define edit_message_dialog(h,t) CMessageDialog(WIN_MESSAGES,0,h,"%s",t)
+#    define edit_query_dialog2(h,t,a,b) CQueryDialog(WIN_MESSAGES,h,t,a,b,0)
+#    define edit_query_dialog3(h,t,a,b,c) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,0)
+#    define edit_query_dialog4(h,t,a,b,c,d) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,d,0)
 
 #endif				/* ! MIDNIGHT */
 
@@ -448,7 +493,9 @@ struct selection selection_history[NUM_SELECTION_HISTORY] =
 int edit_key_emulation = EDIT_KEY_EMULATION_NORMAL;
 #endif	/* ! MIDNIGHT */
 
-int option_word_wrap_line_length = 0;
+int option_word_wrap_line_length = 72;
+int option_typewriter_wrap = 0;
+int option_auto_para_formatting = 0;
 int option_international_characters = 0;
 int option_tab_spacing = 8;
 int option_fill_tabs_with_spaces = 0;
@@ -458,6 +505,7 @@ int option_fake_half_tabs = 1;
 int option_save_mode = 0;
 int option_backup_ext_int = -1;
 int option_find_bracket = 1;
+int option_max_undo = 8192;
 
 int option_editor_fg_normal = 26;
 int option_editor_fg_bold = 8;
@@ -494,6 +542,8 @@ extern int edit_key_emulation;
 #endif	/* ! MIDNIGHT */
 
 extern int option_word_wrap_line_length;
+extern int option_typewriter_wrap;
+extern int option_auto_para_formatting;
 extern int option_international_characters;
 extern int option_tab_spacing;
 extern int option_fill_tabs_with_spaces;
@@ -503,6 +553,7 @@ extern int option_fake_half_tabs;
 extern int option_save_mode;
 extern int option_backup_ext_int;
 extern int option_find_bracket;
+extern int option_max_undo;
 
 extern int option_editor_fg_normal;
 extern int option_editor_fg_bold;
@@ -524,6 +575,7 @@ extern char *option_whole_chars_search;
 extern char *option_whole_chars_move;
 extern char *option_backup_ext;
 
+extern int edit_confirm_save;
+
 #endif				/* ! _EDIT_C */
 
-#endif
