@@ -18,6 +18,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+
 #include <config.h>
 #include "edit.h"
 
@@ -25,9 +26,9 @@
 #include <X11/Xmd.h>		/* CARD32 */
 #include <X11/Xatom.h>
 #include "app_glob.c"
-#include "dirtools.h"
 #include "coollocal.h"
 #include "editcmddef.h"
+#include "mousemark.h"
 #endif
 
 #ifdef __os2__
@@ -39,7 +40,7 @@
 extern int EditExposeRedraw;
 CWidget *wedit = 0;
 
-void Cedit_destroy (CWidget * w)
+void edit_destroy_callback (CWidget * w)
 {
     if (w) {
 	edit_clean (w->editor);
@@ -47,57 +48,62 @@ void Cedit_destroy (CWidget * w)
 	    free (w->editor);
 	w->editor = NULL;
     } else
-	Cerror ("Trying to destroy non-existing editor widget.\n");
+/* NLS ? */
+	CError ("Trying to destroy non-existing editor widget.\n");
 }
 
 extern int option_editor_bg_normal;
+void edit_tri_cursor (Window win);
 
 /* starting_directory is for the filebrowser */
-CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
+CWidget *CDrawEditor (const char *identifier, Window parent, int x, int y,
 	   int width, int height, const char *text, const char *filename,
-		    const char *starting_directory, unsigned int options)
+		const char *starting_directory, unsigned int options, unsigned long text_size)
 {
     static made_directory = 0;
     CWidget *w;
     WEdit *e;
-    wedit = w = Csetupwidget (identifier, parent, x, y,
-			      width + 7, height + 6, CEDITOR_WIDGET,
+    wedit = w = CSetupWidget (identifier, parent, x, y,
+			      width + 7, height + 6, C_EDITOR_WIDGET,
 		   ExposureMask | ButtonPressMask | ButtonReleaseMask | \
 		     KeyPressMask | KeyReleaseMask | ButtonMotionMask | \
-	   FocusChangeMask | StructureNotifyMask | PropertyChangeMask | \
-		   EnterWindowMask, Ccolor (option_editor_bg_normal), 1);
-    w->options = options;
+			      PropertyChangeMask | StructureNotifyMask | \
+			      EnterWindowMask | LeaveWindowMask, color_palette (option_editor_bg_normal), 1);
+    edit_tri_cursor (w->winid);
+    w->options = options | WIDGET_TAKES_SELECTION;
 
-    w->destroy = Cedit_destroy;
+    w->destroy = edit_destroy_callback;
     if (filename)
 	w->label = strdup (filename);
     else
 	w->label = strdup ("");
 
     if (!made_directory) {
-	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0700);
 	made_directory = 1;
     }
-    e = w->editor = Cmalloc (sizeof (WEdit));
+    e = w->editor = CMalloc (sizeof (WEdit));
     if (!w->editor) {
-	Cerror ("Error initialising editor.\n");
+/* Not essential to translate */
+	CError (_ ("Error initialising editor.\n"));
 	return 0;
     }
     w->editor->widget = w;
-    e = w->editor = edit_init (e, height / TEXT_PIX_PER_LINE, width / TEXT_M_WIDTH, filename, text, starting_directory);
+    e = w->editor = edit_init (e, height / FONT_PIX_PER_LINE, width / FONT_MEAN_WIDTH, filename, text, starting_directory, text_size);
     if (!e) {
-	Cundrawwidget (w->ident);
+	CDestroyWidget (w->ident);
 	return 0;
     }
     e->macro_i = -1;
     e->widget = w;
 
+    set_hint_pos (x + width + 7 + WIDGET_SPACING, y + height + 6 + WIDGET_SPACING);
     if (!(options & EDITOR_NO_TEXT))
-	Cdrawtext (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING, "%s", e->filename);
+	CDrawText (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING, "%s", e->filename);
     if (!(options & EDITOR_NO_SCROLL)) {
-	w->scrollbar = Cdrawvertscrollbar (catstrs (identifier, ".vsc", 0), parent,
+	w->vert_scrollbar = CDrawVerticalScrollbar (catstrs (identifier, ".vsc", 0), parent,
 		x + width + 7 + WIDGET_SPACING, y, height + 6, 20, 0, 0);
-	w->scrollbar->editor = (void *) w;
+	CSetScrollbarCallback (w->vert_scrollbar->ident, w->ident, link_scrollbar_to_editor);
     }
     return w;
 }
@@ -106,7 +112,7 @@ void update_scroll_bar (WEdit * e)
 {
     int i, x1, x2;
     CWidget *scroll;
-    scroll = e->widget->scrollbar;
+    scroll = e->widget->vert_scrollbar;
     if (!scroll)
 	return;
 
@@ -124,7 +130,7 @@ void update_scroll_bar (WEdit * e)
 	scroll->firstline = x1;
 	scroll->numlines = x2;
 	EditExposeRedraw = 1;
-	Crenderscrollbar (scroll);
+	render_scrollbar (scroll);
 	EditExposeRedraw = 0;
     }
 }
@@ -150,120 +156,151 @@ long edit_get_click_pos (WEdit * edit, int x, int y)
 void edit_translate_xy (int xs, int ys, int *x, int *y)
 {
     *x = xs - EDIT_TEXT_HORIZONTAL_OFFSET;
-    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET - option_text_line_spacing / 2 - 1) / TEXT_PIX_PER_LINE + 1;
+    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET - option_text_line_spacing / 2 - 1) / FONT_PIX_PER_LINE + 1;
 }
 
-static int just_dropped_something = 0;
+extern int just_dropped_something;
 
 static void mouse_redraw (WEdit * edit, long click)
 {
-    edit->force |= REDRAW_PAGE;
-    update_curs_row (edit);
-    update_curs_col (edit);
+    edit->force |= REDRAW_PAGE | REDRAW_LINE;
+    edit_update_curs_row (edit);
+    edit_update_curs_col (edit);
     edit->prev_col = edit_get_col (edit);
-    Cedit_update_screen (edit);
+    edit_update_screen (edit);
     edit->search_start = click;
 }
 
+static void xy (int x, int y, int *x_return, int *y_return)
+{
+    edit_translate_xy (x, y, x_return, y_return);
+}
+
+static long cp (WEdit *edit, int x, int y)
+{
+    return edit_get_click_pos (edit, x, y);
+}
+
+/* return 1 if not marked */
+static int marks (WEdit * edit, long *start, long *end)
+{
+    return eval_marks (edit, start, end);
+}
+
+int column_highlighting = 0;
+
+static int erange (WEdit * edit, long start, long end, int click)
+{
+    if (column_highlighting) {
+	int x;
+	x = edit_move_forward3 (edit, edit_bol (edit, click), 0, click);
+	if ((x >= edit->column1 && x < edit->column2)
+	    || (x > edit->column2 && x <= edit->column1))
+	    return (start <= click && click < end);
+	else
+	    return 0;
+    }
+    return (start <= click && click < end);
+}
+
+static void fin_mark (WEdit *edit)
+{
+    if (edit->mark2 < 0)
+	edit_mark_cmd (edit, 0);
+}
+
+static void move_mark (WEdit *edit)
+{
+    edit_mark_cmd (edit, 1);
+    edit_mark_cmd (edit, 0);
+}
+
+static void release_mark (WEdit *edit, XEvent *event)
+{
+    if (edit->mark2 < 0)
+	edit_mark_cmd (edit, 0);
+    else
+	edit_mark_cmd (edit, 1);
+    if (edit->mark1 != edit->mark2) {
+	edit_get_selection (edit);
+	XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
+    }
+}
+
+static char *get_block (WEdit * edit, long start_mark, long end_mark, int *type, int *l)
+{
+    char *t;
+    t = (char *) edit_get_block (edit, start_mark, end_mark, l);
+    if (strlen (t) < *l)
+	*type = DndRawData;	/* if there are nulls in the data, send as raw */
+    else
+	*type = DndText;	/* else send as text */
+    return t;
+}
+
+static void move (WEdit *edit, long click, int y)
+{
+    edit_cursor_move (edit, click - edit->curs1);
+}
+
+static void dclick (WEdit *edit, XEvent *event)
+{
+    edit_mark_cmd (edit, 1);
+    edit_right_word_move (edit);
+    edit_mark_cmd (edit, 0);
+    edit_left_word_move (edit);
+    release_mark (edit, event);
+}
+
+static void redraw (WEdit *edit, long click)
+{
+    mouse_redraw (edit, click);
+}
+
 static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
 {
-    static Window win_press = 0;
-    static int state = 0;	/* 0 = button up, 1 = button pressed, 2 = button pressed and dragging */
-    static int x_last, y_last;
-    long click;
-
-    update_curs_row (edit);
-    update_curs_col (edit);
-
+    edit_update_curs_row (edit);
+    edit_update_curs_col (edit);
     if (event->type != MotionNotify) {
-	push_action (edit, KEY_PRESS + edit->start_display);
+	edit_push_action (edit, KEY_PRESS + edit->start_display);
 	if (edit->mark2 == -1)
-	    push_action (edit, MARK_1 + edit->mark1);	/* mark1 must be following the cursor */
+	    edit_push_action (edit, MARK_1 + edit->mark1);	/* mark1 must be following the cursor */
     }
     if (event->type == ButtonPress) {
-	long start_mark, end_mark;
-	state = 1;
-	win_press = edit->widget->winid;
 	edit->highlight = 0;
-	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x_last, &y_last);
-	click = edit_get_click_pos (edit, x_last, y_last);
-	if (!eval_marks (edit, &start_mark, &end_mark)) {
-	    if (start_mark <= click && click < end_mark) {	/* if clicked on highlighted text */
-		unsigned char *t;
-		if (edit->mark2 < 0)
-		    edit_mark_cmd (edit, 0);
-		t = edit_get_block (edit, start_mark, end_mark);
-		just_dropped_something = 1;
-		CDrag (event->xbutton.window, DndText, t, end_mark - start_mark, event->xbutton.button == Button1 ? Button1Mask : 0);
-		free (t);
-		return;
-	    }
-	}
-	just_dropped_something = 0;
-	if (edit->mark2 < 0)
-	    edit_mark_cmd (edit, 0);
-	edit_cursor_move (edit, click - edit->curs1);
-	if (ce->double_click) {
-	    edit_mark_cmd (edit, 1);
-	    edit_right_word_move (edit);
-	    edit_mark_cmd (edit, 0);
-	    edit_left_word_move (edit);
-	    edit_mark_cmd (edit, 0);
-	    state = 0;
-	}
-	mouse_redraw (edit, click);
-    } else if (event->type == ButtonRelease && state > 0 && win_press == edit->widget->winid) {
-	int x, y;
-	long start_mark, end_mark;
-	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x, &y);
-	click = edit_get_click_pos (edit, x, y);
-	edit_cursor_move (edit, click - edit->curs1);
-	if (state == 2)
-	    goto unhighlight;
-	if (!eval_marks (edit, &start_mark, &end_mark)) {
-	    if (start_mark <= click && click < end_mark) {	/* if clicked on highlighted text */
-	      unhighlight:
-		state = 0;
-		if (edit->mark2 < 0)
-		    edit_mark_cmd (edit, 0);
-		else
-		    edit_mark_cmd (edit, 1);
-		if (edit->mark2 != edit->mark1) {
-		    edit_get_selection (edit);
-		    XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
-		}
-	    }
-	}
-	mouse_redraw (edit, click);
-    } else if (event->type == MotionNotify && state > 0 && win_press == edit->widget->winid) {
-	int x, y;
-	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x, &y);
-	if (x == x_last && y == y_last && state == 1)
-	    return;
-	click = edit_get_click_pos (edit, x, y);
-	if (state == 1) {
-	    state = 2;
-	    edit_mark_cmd (edit, 1);
-	    edit_mark_cmd (edit, 0);
-	}
-	edit_cursor_move (edit, click - edit->curs1);
-	mouse_redraw (edit, click);
+	edit->found_len = 0;
     }
+    mouse_mark (
+	(void  *) edit,
+	event,
+	ce,
+	(void (*) (int, int, int *, int *)) xy,
+	(long (*) (void *, int, int)) cp,
+	(int (*) (void *, long *, long *)) marks,
+	(int (*) (void *, long, long, long)) erange,
+	(void (*) (void *)) fin_mark,
+	(void (*) (void *)) move_mark,
+	(void (*) (void *, XEvent *)) release_mark,
+	(char * (*) (void *, long, long, int *, int *)) get_block,
+	(void (*) (void *, long, int)) move,
+	0,
+	(void (*) (void *, XEvent *)) dclick,
+	(void (*) (void *, long)) redraw
+    );
 }
 
-void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
+void link_scrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
 {
     int i, start_line;
     WEdit *e;
     e = editor->editor;
     if (!e)
 	return;
-    if (!e->widget->scrollbar)
+    if (!e->widget->vert_scrollbar)
 	return;
     start_line = e->start_line;
-    w->firstline = min (65535, w->firstline);
     if ((xevent->type == ButtonRelease || xevent->type == MotionNotify) && whichscrbutton == 3) {
-	edit_move_display (e, (double) w->firstline * e->total_lines / 65535.0 + 1);
+	edit_move_display (e, (double) scrollbar->firstline * e->total_lines / 65535.0 + 1);
     } else if (xevent->type == ButtonPress && (cwevent->button == Button1 || cwevent->button == Button2)) {
 	switch (whichscrbutton) {
 	case 1:
@@ -281,19 +318,19 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
 	}
     }
     if (e->total_lines)
-	w->firstline = (double) 65535.0 *e->start_line / (e->total_lines + 1);
+	scrollbar->firstline = (double) 65535.0 *e->start_line / (e->total_lines + 1);
     else
-	w->firstline = 0;
+	scrollbar->firstline = 0;
     i = e->total_lines - e->start_line + 1;
     if (i > e->num_widget_lines)
 	i = e->num_widget_lines;
     if (e->total_lines)
-	w->numlines = (double) 65535.0 *i / (e->total_lines + 1);
+	scrollbar->numlines = (double) 65535.0 *i / (e->total_lines + 1);
     else
-	w->numlines = 65535;
+	scrollbar->numlines = 65535;
     if (start_line != e->start_line) {
-	e->force |= REDRAW_PAGE;
-	Csetcursor (0, 0, 0, 0, 0, 0, 0, 0, 0);
+	e->force |= REDRAW_PAGE | REDRAW_LINE;
+	set_cursor_position (0, 0, 0, 0, 0, 0, 0, 0, 0);
 	if (CCheckWindowEvent (xevent->xany.window, ButtonReleaseMask | ButtonMotionMask, 0))
 	    return;
     }
@@ -354,12 +391,10 @@ void selection_send (XSelectionRequestEvent * rq)
 
 /*{{{ paste selection */
 
-void paste_text (WEdit *edit, unsigned char *data, unsigned int nitems);
-
 /*
  * Respond to a notification that a primary selection has been sent
  */
-void selection_paste (WEdit * edit, Window win, unsigned prop, int delete)
+void paste_prop (void *data, void (*insert) (void *, int), Window win, unsigned prop, int delete)
 {
     long nread;
     unsigned long bytes_after;
@@ -369,25 +404,37 @@ void selection_paste (WEdit * edit, Window win, unsigned prop, int delete)
 
     nread = 0;
     do {
-	unsigned char *data;
+	unsigned char *s;
 	Atom actual_type;
-	int actual_fmt;
+	int actual_fmt, i;
 	unsigned long nitems;
 
 	if (XGetWindowProperty (CDisplay, win, prop,
 				nread / 4, 65536, delete,
 			      AnyPropertyType, &actual_type, &actual_fmt,
 				&nitems, &bytes_after,
-				&data) != Success) {
-	    XFree (data);
+				&s) != Success) {
+	    XFree (s);
 	    return;
 	}
 	nread += nitems;
-	paste_text (edit, data, nitems);
-
-	XFree (data);
+	for (i = 0; i < nitems; i++)
+	    (*insert) (data, s[i]);
+	XFree (s);
     } while (bytes_after);
 }
+
+void selection_paste (WEdit * edit, Window win, unsigned prop, int delete)
+{
+    long c;
+    c = edit->curs1;
+    paste_prop ((void *) edit,
+		(void (*)(void *, int)) edit_insert,
+		win, prop, delete);
+    edit_cursor_move (edit, c - edit->curs1);
+    edit->force |= REDRAW_COMPLETELY | REDRAW_LINE;
+}
+
 /*}}} */
 
 void selection_clear (void)
@@ -396,7 +443,7 @@ void selection_clear (void)
     selection.len = 0;
 }
 
-void Cedit_update_screen (WEdit *e)
+void edit_update_screen (WEdit * e)
 {
     if (!e)
 	return;
@@ -404,8 +451,8 @@ void Cedit_update_screen (WEdit *e)
 	return;
 
     edit_scroll_screen_over_cursor (e);
-    update_curs_row (e);
-    update_curs_col (e);
+    edit_update_curs_row (e);
+    edit_update_curs_col (e);
     update_scroll_bar (e);
     edit_status (e);
 
@@ -415,7 +462,8 @@ void Cedit_update_screen (WEdit *e)
 /* pop all events for this window for internal handling */
     if (e->force & (REDRAW_CHAR_ONLY | REDRAW_COMPLETELY)) {
 	edit_render_keypress (e);
-    } else if (CCheckWindowEvent (e->widget->winid, KeyPressMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, 0)) {
+    } else if (CCheckWindowEvent (e->widget->winid, ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, 0)
+	       || CKeyPending ()) {
 	e->force |= REDRAW_PAGE;
 	return;
     } else {
@@ -423,9 +471,48 @@ void Cedit_update_screen (WEdit *e)
     }
 }
 
-#define dnd_null_term_type(d) \
-	((d) == DndFile || (d) == DndText || (d) == DndDir || \
-	(d) == DndLink || (d) == DndExe || (d) == DndURL)
+extern int space_width;
+
+static void edit_insert_column_of_text (WEdit * edit, unsigned char *data, int size, int width)
+{
+    long cursor;
+    int i, col;
+    cursor = edit->curs1;
+    col = edit_get_col (edit);
+    for (i = 0; i < size; i++) {
+	if (data[i] == '\n') {	/* fill in and move to next line */
+	    int l;
+	    long p;
+	    if (edit_get_byte (edit, edit->curs1) != '\n') {
+		l = width - (edit_get_col (edit) - col);
+		while (l > 0) {
+		    edit_insert (edit, ' ');
+		    l -= space_width;
+		}
+	    }
+	    for (p = edit->curs1;; p++) {
+		if (p == edit->last_byte)
+		    goto insert;
+		if (edit_get_byte (edit, p) == '\n') {
+		    p++;
+		    break;
+		}
+	    }
+	    edit_cursor_move (edit, edit_move_forward3 (edit, p, col, 0) - edit->curs1);
+	    l = col - edit_get_col (edit);
+	    while (l >= space_width) {
+		edit_insert (edit, ' ');
+		l -= space_width;
+	    }
+	    continue;
+	}
+      insert:
+	edit_insert (edit, data[i]);
+    }
+    edit_cursor_move (edit, cursor - edit->curs1);
+}
+
+#define free_data if (data) {free(data);data=0;}
 
 /* handles drag and drop */
 void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
@@ -434,7 +521,7 @@ void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
     unsigned char *data;
     unsigned long size;
     int xs, ys;
-    int x, y, r;
+    int x, y, r, deleted = 0;
     long click;
     unsigned int state;
     long start_mark = 0, end_mark = 0;
@@ -443,36 +530,69 @@ void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
 /* see just below for a comment on what this is for: */
     if (CIsDropAcknowledge (xevent, &state) != DndNotDnd) {
 	if (!(state & Button1Mask) && just_dropped_something) {
-	    push_action (e, KEY_PRESS + e->start_display);
+	    edit_push_action (e, KEY_PRESS + e->start_display);
 	    edit_block_delete_cmd (e);
 	}
 	return;
     }
     data_type = CGetDrop (xevent, &data, &size, &xs, &ys);
-    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height)
+
+    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height) {
+	free_data;
 	return;
+    }
     edit_translate_xy (xs, ys, &x, &y);
     click = edit_get_click_pos (e, x, y);
 
     r = eval_marks (e, &start_mark, &end_mark);
-    if (r || click < start_mark || click >= end_mark) {		/* musn't be able to drop into a block,
-								   otherwise a single click will copy a block: */
-	push_action (e, KEY_PRESS + e->start_display);
-	edit_cursor_move (e, click - e->curs1);
-	if (data_type == DndFile) {
-	    edit_insert_file (e, (char *) data);
-	} else if (data_type != DndFiles) {
-	    if (dnd_null_term_type (data_type)) {
-		int len;
-		len = strlen ((char *) data);
-		size = min (len, size);
-	    }
-	    while (size--)
-		edit_insert_ahead (e, data[size]);
+/* musn't be able to drop into a block, otherwise a single click will copy a block: */
+    if (r)
+	goto fine;
+    if (start_mark > click || click >= end_mark)
+	goto fine;
+    if (column_highlighting) {
+	if (!((x >= e->column1 && x < e->column2)
+	      || (x > e->column2 && x <= e->column1)))
+	    goto fine;
+    }
+    free_data;
+    return;
+  fine:
+    edit_push_action (e, KEY_PRESS + e->start_display);
+
+/* drops to the same window moving to the left: */
+    if (xevent->xclient.data.l[2] == xevent->xclient.window && !(xevent->xclient.data.l[1] & Button1Mask))
+	if ((column_highlighting && x < max (e->column1, e->column2)) || !column_highlighting) {
+	    edit_block_delete_cmd (e);
+	    deleted = 1;
+	}
+    edit_update_curs_row (e);
+    click = edit_get_click_pos (e, x, y);	/* click pos changes with edit_block_delete_cmd() */
+    edit_cursor_move (e, click - e->curs1);
+    if (data_type == DndFile) {
+	edit_insert_file (e, (char *) data);
+    } else if (data_type != DndFiles) {
+	if (dnd_null_term_type (data_type)) {
+	    int len;
+	    len = strlen ((char *) data);
+	    size = min (len, size);
+	}
+	if (column_highlighting) {
+	    edit_insert_column_of_text (e, data, size, abs (e->column2 - e->column1));
 	} else {
 	    while (size--)
-		edit_insert_ahead (e, data[size] ? data[size] : '\n');
+		edit_insert_ahead (e, data[size]);
 	}
+    } else {
+	while (size--)
+	    edit_insert_ahead (e, data[size] ? data[size] : '\n');
+    }
+
+/* drops to the same window moving to the right: */
+    if (xevent->xclient.data.l[2] == xevent->xclient.window && !(xevent->xclient.data.l[1] & Button1Mask))
+	if (column_highlighting && !deleted)
+	    edit_block_delete_cmd (e);
+
 /* The drop has now been successfully recieved. We can now send an acknowledge
    event back to the window that send the data. When this window recieves
    the acknowledge event, the app can decide whether or not to delete the data.
@@ -482,116 +602,104 @@ void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
    application must delete its selection after recieving an acknowledge
    event). We must not, however, send an acknowledge signal if a filelist
    (for example) was passed to us, since the sender might take this to
-   mean that all those file can be deleted! The two types we can acknowledge
+   mean that all those files can be deleted! The two types we can acknowledge
    are: */
+    if (xevent->xclient.data.l[2] != xevent->xclient.window)	/* drops to the same window */
 	if (data_type == DndText || data_type == DndRawData)
 	    CDropAcknowledge (xevent);
-	e->force |= REDRAW_PAGE;
-	return;
-    }
+    e->force |= REDRAW_COMPLETELY | REDRAW_LINE;
+    free_data;
 }
 
 int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 {
     WEdit *e = w->editor;
-    char xlat;
-    KeySym key;
     int r = 0;
-    int cmd, ch;
     static int old_tab_spacing = -1;
 
     if (!e)
 	return 0;
 
     if (old_tab_spacing != option_tab_spacing)
-	e->force |= REDRAW_COMPLETELY;
+	e->force |= REDRAW_COMPLETELY + REDRAW_LINE;
     old_tab_spacing = option_tab_spacing;
 
     if (xevent->type == KeyPress) {
 	if (xevent->xkey.keycode == 0x31 && xevent->xkey.state == 0xD) {
-	    Csetcolor (Ccolor(18));
-	    Crect (w->winid, 0, 0, w->width, w->height);
+	    CSetColor (color_palette (18));
+	    CRectangle (w->winid, 0, 0, w->width, w->height);
 	}
     }
-
     switch (xevent->type) {
     case SelectionNotify:
 	selection_paste (e, xevent->xselection.requestor, xevent->xselection.property, True);
 	r = 1;
 	break;
     case SelectionRequest:
-	r = 1;
 	selection_send (&(xevent->xselectionrequest));
-	break;
+	return 1;
 /*  case SelectionClear:   ---> This is handled by coolnext.c: CNextEvent() */
     case ClientMessage:
 	handle_client_message (w, xevent, cwevent);
 	r = 1;
 	break;
-    case ResizeNotify:
-	if (w->options & EDITOR_NO_RESIZE)
-	    break;
-	Csetwidgetsize (w->ident, w->width + xevent->xconfigure.x, w->height + xevent->xconfigure.y);
-	e->num_widget_lines = (w->height - 6) / TEXT_PIX_PER_LINE;
-	e->num_widget_columns = (w->width - 7) / TEXT_M_WIDTH;
-	if (!(w->options & EDITOR_NO_SCROLL)) {
-	    Csetwidgetsize (catstrs (w->ident, ".vsc", 0), 20, w->height);
-	    Csetwidgetposition (catstrs (w->ident, ".vsc", 0), (Cwidget (catstrs (w->ident, ".vsc", 0)))->x + xevent->xconfigure.x, w->y);
-	}
-	if (!(w->options & EDITOR_NO_TEXT))
-	    Csetwidgetposition (catstrs (w->ident, ".text", 0), w->x, (Cwidget (catstrs (w->ident, ".text", 0)))->y + xevent->xconfigure.y);
-	edit_status (e);
-	break;
     case ButtonPress:
-	CFocusWindow (w->winid);
+	CFocus (w);
 	edit_render_tidbits (w);
     case ButtonRelease:
+	if (xevent->xbutton.state & ControlMask)
+	    column_highlighting = 1;
+	else
+	    column_highlighting = 0;
     case MotionNotify:
-	Cresolvebutton (xevent, cwevent);
+	if (!xevent->xmotion.state && xevent->type == MotionNotify)
+	    return 0;
+	resolve_button (xevent, cwevent);
 	edit_mouse_mark (e, xevent, cwevent);
 	break;
     case Expose:
-	EditExposeRedraw = 1;
 	edit_render_expose (e, &(xevent->xexpose));
-	EditExposeRedraw = 0;
 	return 1;
     case FocusIn:
+	CSetCursorColor (e->overwrite ? color_palette (24) : color_palette (19));
     case FocusOut:
 	edit_render_tidbits (w);
-	e->force |= REDRAW_CHAR_ONLY;
+	e->force |= REDRAW_CHAR_ONLY | REDRAW_LINE;
 	break;
     case KeyRelease:
+	if (column_highlighting) {
+	    column_highlighting = 0;
+	    e->force = REDRAW_COMPLETELY | REDRAW_LINE;
+	    edit_mark_cmd (e, 1);
+	}
 	break;
     case KeyPress:
-	xlat = 0;
-	key = CKeySym (xevent);
-	if (!key)
-	    break;
 	cwevent->ident = w->ident;
-	cwevent->key = key;
-	cwevent->xlat = xlat;
-	cwevent->state = xevent->xkey.state;
-	if (!edit_translate_key (e, xevent->xkey.keycode, key, xevent->xkey.state, &cmd, &ch)) {
-	    cmd = CKeySymMod (xevent);
-	    if (cmd > 0)
-		cmd = CK_Macro (cmd);
-	    else
-		break;
+	if (!cwevent->command && cwevent->insert < 0) {		/* no translation */
+	    if ((cwevent->key == XK_r || cwevent->key == XK_R) && (cwevent->state & ControlMask)) {
+		cwevent->command = e->macro_i < 0 ? CK_Begin_Record_Macro : CK_End_Record_Macro;
+	    } else {
+		cwevent->command = CKeySymMod (xevent);
+		if (cwevent->command > 0)
+		    cwevent->command = CK_Macro (cwevent->command);
+		else
+		    break;
+	    }
 	}
-	cwevent->command = cmd;
-	r = edit_execute_key_command (e, cmd, ch);
+	r = edit_execute_key_command (e, cwevent->command, cwevent->insert);
 	break;
     case EditorCommand:
-	r = edit_execute_key_command (e, xevent->xkey.keycode, -1);
+	cwevent->ident = w->ident;
+	cwevent->command = xevent->xkey.keycode;
+	r = cwevent->handled = edit_execute_key_command (e, xevent->xkey.keycode, -1);
 	break;
     default:
 	return 0;
     }
-    Cedit_update_screen (e);
+    edit_update_screen (e);
     return r;
 }
 
-
 #else
 
 WEdit *wedit;
@@ -599,8 +707,9 @@ WButtonBar *edit_bar;
 Dlg_head *edit_dlg;
 WMenu *edit_menubar;
 
+int column_highlighting = 0;
 
-static int edit_callback (Dlg_head * h, void * e, int msg, int par);
+static int edit_callback (Dlg_head * h, WEdit * edit, int msg, int par);
 
 static int edit_mode_callback (struct Dlg_head *h, int id, int msg)
 {
@@ -610,8 +719,8 @@ static int edit_mode_callback (struct Dlg_head *h, int id, int msg)
 int edit_event (WEdit * edit, Gpm_Event * event, int *result)
 {
     *result = MOU_NORMAL;
-    update_curs_row (edit);
-    update_curs_col (edit);
+    edit_update_curs_row (edit);
+    edit_update_curs_col (edit);
     if (event->type & (GPM_DOWN | GPM_DRAG | GPM_UP)) {
 	if (event->y > 1 && event->x > 0
 	    && event->x <= edit->num_widget_columns
@@ -619,7 +728,7 @@ int edit_event (WEdit * edit, Gpm_Event * event, int *result)
 	    if (edit->mark2 != -1 && event->type & (GPM_UP | GPM_DRAG))
 		return 1;	/* a lone up mustn't do anything */
 	    if (event->type & (GPM_DOWN | GPM_UP))
-		push_key_press (edit);
+		edit_push_key_press (edit);
 	    edit_cursor_move (edit, edit_bol (edit, edit->curs1) - edit->curs1);
 	    if (--event->y > (edit->curs_row + 1))
 		edit_cursor_move (edit,
@@ -639,9 +748,9 @@ int edit_event (WEdit * edit, Gpm_Event * event, int *result)
 	    if (!(event->type & GPM_DRAG))
 		edit_mark_cmd (edit, 0);
 	    edit->force |= REDRAW_COMPLETELY;
-	    update_curs_row (edit);
-	    update_curs_col (edit);
-	    Cedit_update_screen (edit);
+	    edit_update_curs_row (edit);
+	    edit_update_curs_col (edit);
+	    edit_update_screen (edit);
 	    return 1;
 	}
     }
@@ -663,7 +772,7 @@ int edit_mouse_event (Gpm_Event * event, void *x)
 
 extern Menu EditMenuBar[5];
 
-int edit (const char *_file)
+int edit (const char *_file, int line)
 {
     static int made_directory = 0;
     int framed = 0;
@@ -677,7 +786,7 @@ int edit (const char *_file)
     }
 
     if (!made_directory) {
-	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0700);
 	made_directory = 1;
     }
     if (_file) {
@@ -688,8 +797,8 @@ int edit (const char *_file)
     } else
 	text = "";
 
-    if (!(wedit = edit_init (NULL, LINES - 2, COLS, _file, text, ""))) {
-	message (1, " Error ", get_error_msg (""));
+    if (!(wedit = edit_init (NULL, LINES - 2, COLS, _file, text, "", 0))) {
+	message (1, _(" Error "), get_error_msg (""));
 	return 0;
     }
     wedit->macro_i = -1;
@@ -728,6 +837,8 @@ int edit (const char *_file)
 	add_widget (edit_dlg, edit_menubar);
 
     add_widget (edit_dlg, edit_bar);
+    edit_move_display (wedit, line - 1);
+    edit_move_to_line (wedit, line - 1);
 
     run_dlg (edit_dlg);
 
@@ -800,17 +911,17 @@ void edit_labels (WEdit * edit)
 {
     Dlg_head *h = edit->widget.parent;
 
-    edit_my_define (h, 1, "Help", cmd_F1, edit);
-    edit_my_define (h, 2, "Save", cmd_F2, edit);
-    edit_my_define (h, 3, "Mark", cmd_F3, edit);
-    edit_my_define (h, 4, "Replac", cmd_F4, edit);
-    edit_my_define (h, 5, "Copy", cmd_F5, edit);
-    edit_my_define (h, 6, "Move", cmd_F6, edit);
-    edit_my_define (h, 7, "Search", cmd_F7, edit);
-    edit_my_define (h, 8, "Delete", cmd_F8, edit);
+    edit_my_define (h, 1, _("Help"), cmd_F1, edit);
+    edit_my_define (h, 2, _("Save"), cmd_F2, edit);
+    edit_my_define (h, 3, _("Mark"), cmd_F3, edit);
+    edit_my_define (h, 4, _("Replac"), cmd_F4, edit);
+    edit_my_define (h, 5, _("Copy"), cmd_F5, edit);
+    edit_my_define (h, 6, _("Move"), cmd_F6, edit);
+    edit_my_define (h, 7, _("Search"), cmd_F7, edit);
+    edit_my_define (h, 8, _("Delete"), cmd_F8, edit);
     if (!edit->have_frame)
-	edit_my_define (h, 9, "PullDn", edit_menu_cmd, edit);
-    edit_my_define (h, 10, "Quit", cmd_F10, edit);
+	edit_my_define (h, 9, _("PullDn"), edit_menu_cmd, edit);
+    edit_my_define (h, 10, _("Quit"), cmd_F10, edit);
 
     redraw_labels (h, (Widget *) edit);
 }
@@ -832,11 +943,11 @@ void edit_adjust_size (Dlg_head * h)
     widget_set_size (&edit_bar->widget, LINES - 1, 0, 1, COLS);
 }
 
-void Cedit_update_screen (WEdit * e)
+void edit_update_screen (WEdit * e)
 {
     edit_scroll_screen_over_cursor (e);
 
-    update_curs_col (e);
+    edit_update_curs_col (e);
     edit_status (e);
 
 /* pop all events for this window for internal handling */
@@ -850,10 +961,8 @@ void Cedit_update_screen (WEdit * e)
     edit_render_keypress (e);
 }
 
-static int edit_callback (Dlg_head * h, void * ve, int msg, int par)
+static int edit_callback (Dlg_head * h, WEdit * e, int msg, int par)
 {
-    WEdit *e = (WEdit *) ve;
-
     switch (msg) {
     case WIDGET_INIT:
 	e->force |= REDRAW_COMPLETELY;
@@ -864,7 +973,7 @@ static int edit_callback (Dlg_head * h, void * ve, int msg, int par)
 	e->num_widget_lines = LINES - 2;
 	e->num_widget_columns = COLS;
     case WIDGET_FOCUS:
-	Cedit_update_screen (e);
+	edit_update_screen (e);
 	return 1;
     case WIDGET_KEY:{
 	    int cmd, ch;
@@ -873,12 +982,12 @@ static int edit_callback (Dlg_head * h, void * ve, int msg, int par)
 	    if (!edit_translate_key (e, 0, par, get_key_state (), &cmd, &ch))
 		return 0;
 	    edit_execute_key_command (e, cmd, ch);
-	    Cedit_update_screen (e);
+	    edit_update_screen (e);
 	}
 	return 1;
     case WIDGET_COMMAND:
 	edit_execute_key_command (e, par, -1);
-	Cedit_update_screen (e);
+	edit_update_screen (e);
 	return 1;
     case WIDGET_CURSOR:
 	widget_move (&e->widget, e->curs_row + EDIT_TEXT_VERTICAL_OFFSET, e->curs_col + e->start_col);
