@@ -122,22 +122,22 @@ struct {
 } chmod_but[BUTTONS] = {
 
     {
-	B_CANCEL, 2, 33, "[ Cancel ]", 'c', 2,
+	B_CANCEL, 2, 33, "Cancel", 'c', 0,
     },
     {
-	B_ENTER, 2, 17, "[ Set ]", 's', 2,
+	B_ENTER, 2, 17, "Set", 's', 0,
     },
     {
-	B_CLRMRK, 0, 42, "[ Clear marked ]", 'l', 3,
+	B_CLRMRK, 0, 42, "Clear marked", 'l', 1,
     },
     {
-	B_SETMRK, 0, 27, "[ Set marked ]", 'e', 3,
+	B_SETMRK, 0, 27, "Set marked", 'e', 1,
     },
     {
-	B_MARKED, 0, 12, "[ Marked all ]", 'm', 2,
+	B_MARKED, 0, 12, "Marked all", 'm', 0,
     },
     {
-	B_ALL, 0, 0, "[ Set all ]", 'a', 6,
+	B_ALL, 0, 0, "Set all", 'a', 4,
     },
 };
 
@@ -248,7 +248,7 @@ static void init_chmod (void)
     x_set_dialog_title (ch_dlg, "Chmod command");
 
 #define XTRACT(i) BY+chmod_but[i].y-single_set, BX+chmod_but[i].x, \
-                  chmod_but[i].ret_cmd, chmod_but[i].text, chmod_but[i].hkey,                    chmod_but[i].hpos, 0, 0, NULL
+                  chmod_but[i].ret_cmd, NORMAL_BUTTON, chmod_but[i].text, chmod_but[i].hkey,                    chmod_but[i].hpos, 0, 0, NULL
 
     tk_new_frame (ch_dlg, "b.");
     for (i = 0; i < BUTTONS; i++) {
