@@ -216,6 +216,7 @@ char *canonicalize_pathname (char *path)
 
     /* FIXME: buffer large enough? */
     strcpy (path, pCanonical);
+    return path;
 }
 /* Do 'manual' canonicalizing, needed for some VFS functions
 
@@ -536,7 +537,7 @@ int gettimeofday (struct timeval* tvp, void *p)
 	   on the file pointed to and lstat on the symlink itself.
 	   We have no such a difference/trouble.
  */
-int lstat (const char* pathname, struct _stat *buffer)
+int lstat (const char* pathname, struct stat *buffer)
 {
 	return stat (pathname, buffer);
 }
@@ -652,13 +653,37 @@ errno_dir_not_empty (int err)
     return 0;
 }
 
+/* The MC library directory is by default the directory where mc.exe
+   is situated. It is possible to specify this directory via MCHOME
+   environment variable */
 char *
 get_mc_lib_dir ()
 {
+    char *cur;
     char *mchome = getenv("MCHOME");
 
-    if (!mchome)
-        return "D:\\mc";
-    else
-        return mchome;
+    if (mchome && *mchome)
+	return mchome;
+    mchome = malloc(MC_MAXPATHLEN);
+    GetModuleFileName(NULL, mchome, MC_MAXPATHLEN);
+    for (cur = mchome + strlen(mchome); \
+	(cur > mchome) && (*cur != PATH_SEP); cur--);
+    *cur = 0;
+    cur = strdup(mchome);
+    free(mchome);
+    if (!cur || !*cur) {
+	free(cur);
+        return "C:\\MC";
+    }
+    return cur;
+}
+int get_user_rights (struct stat *buf)
+{
+    return 2;
+}
+void init_groups (void)
+{
+}
+void delete_groups (void)
+{
 }
