@@ -104,10 +104,37 @@ typedef struct termios TTY_Termio_Type;
 static TTY_Termio_Type Old_TTY;
 
 #ifdef HAVE_TERMIOS_H
-static int Baud_Rates[20] = 
+static struct 
 {
-   0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800, 
-   9600, 19200, 38400, 0, 0, 0, 0
+    speed_t key;
+    int value;
+} Baud_Rates[] = 
+{
+   {B0, 0},
+   {B50, 50}, 
+   {B75, 75}, 
+   {B110, 110}, 
+   {B134, 134}, 
+   {B150, 150}, 
+   {B200, 200}, 
+   {B300, 300}, 
+   {B600, 600}, 
+   {B1200, 1200}, 
+   {B1800, 1800}, 
+   {B2400, 2400}, 
+   {B4800, 4800}, 
+   {B9600, 9600}, 
+   {B19200, 19200}, 
+   {B38400, 38400}
+#ifdef B57600
+   , {B57600, 57600}
+#endif
+#ifdef B115200
+   , {B115200, 115200}
+#endif
+#ifdef B230400
+   , {B230400, 230400}
+#endif
 };
 #endif
 
@@ -144,6 +171,16 @@ static int TTY_Open = 0;
 # endif
 #endif
 
+static int
+speed_t2baud_rate (speed_t s)
+{
+    int i;
+    
+    for (i = 0; i < sizeof (Baud_Rates)/sizeof (Baud_Rates[0]); i++)
+	if (Baud_Rates[i].key == s)
+	    return (Baud_Rates[i].value);
+    return 0;
+}
 
 int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 {
@@ -162,6 +199,7 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
    if ((SLang_TT_Read_FD == -1)
        || (1 != isatty (SLang_TT_Read_FD)))
      {
+#if 0
 #ifdef O_RDWR
 # ifndef __BEOS__  /* I have been told that BEOS will HANG if passed /dev/tty */
 	if ((SLang_TT_Read_FD = open("/dev/tty", O_RDWR)) >= 0)
@@ -170,10 +208,21 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 	  }
 # endif
 #endif
+#endif /* 0 */
 	if (TTY_Open == 0)
 	  {
+
+#if 0
+/* In the Midnight Commander we bind stderr sometimes to a pipe. If we
+   use stderr for terminal input and call SLang_getkey while stderr is
+   bound to a pipe MC will hang completly in SLsys_input_pending. 
+   NOTE: There's an independent fix for this problem in src/slint.c for
+   the case that the Midnight Commander is linked against a shared slang 
+   library compiled from different sources.
+ */
 	     SLang_TT_Read_FD = fileno (stderr);
 	     if (1 != isatty (SLang_TT_Read_FD))
+#endif	     
 	       {
 		  SLang_TT_Read_FD = fileno (stdin);
 		  if (1 != isatty (SLang_TT_Read_FD))
@@ -237,11 +286,7 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 #ifdef HAVE_CFGETOSPEED
 	SLang_TT_Baud_Rate = cfgetospeed (&newtty);
 #endif	
-#ifndef __bsdi__	
-	SLang_TT_Baud_Rate = ((SLang_TT_Baud_Rate > 0) && (SLang_TT_Baud_Rate < 19)
-			      ? Baud_Rates[SLang_TT_Baud_Rate]
-			      : 0);
-#endif
+	SLang_TT_Baud_Rate = speed_t2baud_rate (SLang_TT_Baud_Rate);
      }
    if (no_flow_control) newtty.c_iflag &= ~IXON; else newtty.c_iflag |= IXON;
 
