@@ -66,19 +66,18 @@
 #define B_OGROUP        B_USER + 7
 
 struct {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
 } chown_advanced_but [BUTTONS] = {
-    { B_CANCEL, 4, 55, "[ Cancel ]", 'c', 2 },
-    { B_ENTER, 4, 46, "[ Set ]", 's', 2 },
-    { B_SKIP, 4, 36, "[ Skip ]", 'k', 3 },
-    { B_SETALL, 4, 23, "[ Set all ]", 'a', 6 },
-    { B_ENTER, 0, 48, "[               ]", 0, -1 },
-    { B_ENTER, 0, 30, "[               ]", 0, -1 },
-    { B_ENTER, 0, 19, "[   ]", 0, -1 },
-    { B_ENTER, 0, 11, "[   ]", 0, -1 },
-    { B_ENTER, 0, 3, "[   ]", 0, -1 },
+    { B_CANCEL, NORMAL_BUTTON, 4, 55, "&Cancel" },
+    { B_ENTER,  DEFPUSH_BUTTON,4, 45, "&Set" },
+    { B_SKIP,   NORMAL_BUTTON, 4, 36, "S&kip" },
+    { B_SETALL, NORMAL_BUTTON, 4, 24, "Set &all"},
+    { B_ENTER,  NARROW_BUTTON, 0, 47, "               "},
+    { B_ENTER,  NARROW_BUTTON, 0, 29, "               "},
+    { B_ENTER,  NARROW_BUTTON, 0, 19, "   "},
+    { B_ENTER,  NARROW_BUTTON, 0, 11, "   "},
+    { B_ENTER,  NARROW_BUTTON, 0, 3, "   "},
 };
 
 WButton *b_att[3];	/* permission */
@@ -102,11 +101,11 @@ static void get_ownership ()
     char *name_t;
 
     name_t = name_trunc (get_owner (sf_stat->st_uid), 15);
-    memset (b_user->text + 1, ' ', 15);
-    strncpy (b_user->text + 1, name_t, strlen (name_t));
+    memset (b_user->text, ' ', 15);
+    strncpy (b_user->text, name_t, strlen (name_t));
     name_t = name_trunc (get_group (sf_stat->st_gid), 15);
-    memset (b_group->text + 1, ' ', 15);
-    strncpy (b_group->text + 1, name_t, strlen (name_t));
+    memset (b_group->text, ' ', 15);
+    strncpy (b_group->text, name_t, strlen (name_t));
 }
 
 
@@ -204,16 +203,16 @@ static void print_flags (void)
 	addch (ch_flags [i+6]);
     }
 
-    set_perm_by_flags (b_att[0]->text + 1, 0);
-    set_perm_by_flags (b_att[1]->text + 1, 3);
-    set_perm_by_flags (b_att[2]->text + 1, 6);
+    set_perm_by_flags (b_att[0]->text, 0);
+    set_perm_by_flags (b_att[1]->text, 3);
+    set_perm_by_flags (b_att[2]->text, 6);
 
     for (i = 0; i < 15; i++){
-	dlg_move (ch_dlg, BY+1, 36+i);
+	dlg_move (ch_dlg, BY+1, 35+i);
 	addch (ch_flags[9]);
     }
     for (i = 0; i < 15; i++){
-	dlg_move (ch_dlg, BY + 1, 54 + i);
+	dlg_move (ch_dlg, BY + 1, 53 + i);
 	addch (ch_flags[10]);
     }
 }
@@ -391,16 +390,16 @@ static void chown_info_update ()
     printw ("%12o", get_mode ());
     
     /* permissions */
-    set_perm (b_att[0]->text + 1, sf_stat->st_mode >> 6);
-    set_perm (b_att[1]->text + 1, sf_stat->st_mode >> 3);
-    set_perm (b_att[2]->text + 1, sf_stat->st_mode);
+    set_perm (b_att[0]->text, sf_stat->st_mode >> 6);
+    set_perm (b_att[1]->text, sf_stat->st_mode >> 3);
+    set_perm (b_att[2]->text, sf_stat->st_mode);
 }
 
 static void b_setpos (int f_pos) {
 	b_att[0]->hotpos=-1;
 	b_att[1]->hotpos=-1;
 	b_att[2]->hotpos=-1;
-	b_att[f_pos]->hotpos = (flag_pos % 3) + 1;
+	b_att[f_pos]->hotpos = (flag_pos % 3);
 }
 
 static int advanced_chown_callback (Dlg_head * h, int Par, int Msg)
@@ -496,7 +495,7 @@ static int advanced_chown_callback (Dlg_head * h, int Par, int Msg)
 	    if (f_pos > 2)
 		break;
 	    flag_pos = f_pos * 3 + i;	/* (strchr(ch_perm,Par)-ch_perm); */
-	    if (((WButton *) h->current->widget)->text[(flag_pos % 3) + 1] == '-')
+	    if (((WButton *) h->current->widget)->text[(flag_pos % 3)] == '-')
 		ch_flags[flag_pos] = '+';
 	    else
 		ch_flags[flag_pos] = '-';
@@ -557,8 +556,8 @@ static void init_chown_advanced (void)
 			 "[Chown-advanced]", "achown", DLG_CENTER);
 
 #define XTRACT(i) BY+chown_advanced_but[i].y, BX+chown_advanced_but[i].x, \
-                  chown_advanced_but[i].ret_cmd, chown_advanced_but[i].text, \
-                  chown_advanced_but[i].hkey, chown_advanced_but[i].hpos, 0, 0, NULL
+	chown_advanced_but[i].ret_cmd, chown_advanced_but[i].flags, chown_advanced_but[i].text, \
+        0, 0, NULL
 
     for (i = 0; i < BUTTONS - 5; i++)
 	if (!single_set || i < 2)
