@@ -53,21 +53,21 @@
 #define DISPLAY_Y  15
 
 static Dlg_head *dd;
+static WRadio *my_radio;
 static WInput *user;
 static WInput *status;
 static WCheck *check_status;
 static int current_mode;
 extern int ftpfs_always_use_proxy;
 
-#define VIEW_TYPES 4
+static char **displays_status;
 
-static char *displays [VIEW_TYPES] = {
+static char *displays [LIST_TYPES] = {
     "Full file list", "Brief file list", "Long file list",
     "User:"
 };
 
 static char *formats_section = "Formats";
-static WRadio *my_radio;
 
 static char *select_format (WInput *i)
 {
@@ -121,12 +121,12 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
     
     switch (Msg){
     case DLG_DRAW:
-	attrset (REVERSE_COLOR);
+	attrset (COLOR_NORMAL);
 	dlg_erase (h);
 	draw_box (h, 1, 1, DISPLAY_Y-2, DISPLAY_X-2);
 
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 2);
+	dlg_move (h, 1, (DISPLAY_X-17)/2);
 	addstr (" Listing mode ");
 	attrset (COLOR_NORMAL);
 	dlg_move (h, 11, 3);
@@ -134,9 +134,22 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
 	dlg_move (h, 12, 3);
 	addstr ("with other formats");
 	break;
+
+    case DLG_UNFOCUS:
+	if((WRadio *) h->current->widget == my_radio){
+	    assign_text (status, displays_status [my_radio->sel]);
+	    input_set_point (status, 0);
+	}
+	break;
 	
     case DLG_KEY:
 	if (id == '\n'){
+	    if((WRadio *) h->current->widget == my_radio){
+		h->running = 0;
+		assign_text (status, displays_status [my_radio->sel]);
+		break;
+	    }
+	    
 	    if ((WInput *) h->current->widget == user){
 		h->running = 0;
 		h->ret_value = B_USER + 6;
@@ -160,7 +173,7 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
 	    text = select_format (input);
 	    if (text){
 		assign_text (input, text);
-		update_input (input);
+		input_set_point (input, 0);
 	    }
 	    return MSG_HANDLED;
 	}
@@ -169,7 +182,7 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
 	    && h->current->widget != (Widget *) status){
 	    my_radio->sel = 3;
 	    dlg_select_widget (h, my_radio); /* force redraw */
-    dlg_select_widget (h, user);
+	    dlg_select_widget (h, user);
 	    return MSG_HANDLED;
 	}
     }
@@ -178,33 +191,35 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
 }
 
 static void display_init (int radio_sel, char *init_text,
-			  int _check_status, char *_status)
+			  int _check_status, char ** _status)
 {
+    displays_status = _status;
+    
     dd = create_dlg (0, 0, DISPLAY_Y, DISPLAY_X, dialog_colors,
 		     display_callback, "[Left and Right Menus]", "display",
 		     DLG_CENTER | DLG_GRID);
 
     x_set_dialog_title (dd, "Listing mode");
     add_widgetl (dd,
-        button_new (3, 32, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "cancel-button"),
+        button_new (4, 32, B_CANCEL, NORMAL_BUTTON, "&Cancel", 0, 0, "cancel-button"),
 	XV_WLAY_RIGHTOF);
 
     add_widgetl (dd,
-	button_new (5, 32, B_ENTER,  "[[ Ok ]]", 'o', 3, 0, 0, "ok-button"),
+	button_new (3, 32, B_ENTER, DEFPUSH_BUTTON, "&Ok", 0, 0, "ok-button"),
 	 XV_WLAY_CENTERROW);
 
-    status = input_new (9, 8, NORMAL_COLOR, 34, _status, "mini-input");
+    status = input_new (9, 8, INPUT_COLOR, 34, _status [radio_sel], "mini-input");
     add_widgetl (dd, status, XV_WLAY_RIGHTDOWN);
     input_set_point (status, 0);
 
-    check_status = check_new (8, 4, _check_status, "user Mini status",'m', 5, "mini-status");
+    check_status = check_new (8, 4, _check_status, "user &Mini status", "mini-status");
     add_widgetl (dd, check_status, XV_WLAY_NEXTROW);
     
-    user = input_new  (6, 14, NORMAL_COLOR, 29, init_text, "user-fmt-input");
+    user = input_new  (6, 14, INPUT_COLOR, 29, init_text, "user-fmt-input");
     add_widgetl (dd, user, XV_WLAY_RIGHTDOWN);
     input_set_point (user, 0);
 
-    my_radio = radio_new (3, 4, VIEW_TYPES, displays, 1, "radio");
+    my_radio = radio_new (3, 4, LIST_TYPES, displays, 1, "radio");
     my_radio->sel = my_radio->pos = current_mode;
     add_widgetl (dd, my_radio, XV_WLAY_BELOWCLOSE);
 }
@@ -212,7 +227,7 @@ static void display_init (int radio_sel, char *init_text,
 int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
     int num)
 {
-    int result;
+    int result, i;
     char *section = NULL;
     char *p;
 
@@ -222,7 +237,8 @@ int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
         panel->list_type = list_full;
         panel->user_format = strdup (DEFAULT_USER_FORMAT);
         panel->user_mini_status = 0;
-        panel->mini_status_format = strdup (DEFAULT_USER_FORMAT);
+	for (i = 0; i < LIST_TYPES; i++)
+    	    panel->user_status_format[i] = strdup (DEFAULT_USER_FORMAT);
         section = copy_strings ("Temporal:", p, 0);
         if (!profile_has_section (section, profile_name)) {
             free (section);
@@ -233,8 +249,8 @@ int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
     }
 
     current_mode = panel->list_type;
-    display_init (panel->list_type, panel->user_format,
-		  panel->user_mini_status, panel->mini_status_format);
+    display_init (current_mode, panel->user_format, 
+	panel->user_mini_status, panel->user_status_format);
 		  
     run_dlg (dd);
 
@@ -242,7 +258,8 @@ int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
     
     if (section) {
         free (panel->user_format);
-        free (panel->mini_status_format);
+	for (i = 0; i < LIST_TYPES; i++)
+	    free (panel->user_status_format [i]);
         free (panel);
     }
     
@@ -281,15 +298,15 @@ sortfn *sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
 		     
     x_set_dialog_title (dd, "Sort order");
 
-    add_widgetl (dd, button_new (6, 25, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "cancel-button"),
+    add_widgetl (dd, button_new (8, 23, B_CANCEL, NORMAL_BUTTON, "&Cancel", 0, 0, "cancel-button"),
         XV_WLAY_CENTERROW);
 
-    add_widgetl (dd, button_new (8, 25, B_ENTER, "[ Ok ]", 'o', 2, 0, 0, "ok-button"),
+    add_widgetl (dd, button_new (7, 23, B_ENTER, DEFPUSH_BUTTON, "&Ok", 0, 0, "ok-button"),
 	XV_WLAY_RIGHTDOWN);
 
-    case_sense = check_new (4, 19, *case_sensitive, "case sensitive", 't', 10, "case-check");
+    case_sense = check_new (4, 19, *case_sensitive, "case sensi&tive", "case-check");
     add_widgetl (dd, case_sense, XV_WLAY_RIGHTDOWN);
-    c = check_new (3, 19, *reverse, "Reverse", 'r', 0, "reverse-check");
+    c = check_new (3, 19, *reverse, "&Reverse", "reverse-check");
     add_widgetl (dd, c, XV_WLAY_RIGHTDOWN);
 
     for (i = SORT_TYPES-1; i >= 0; i--){
@@ -322,20 +339,20 @@ static int my_execute;
 static int my_exit;
 
 static QuickWidget conf_widgets [] = {
-{ quick_button,   4, 6, 4, CONFY, " [ Cancel ]",
-      'c', 3, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "c" },
-{ quick_button,   4, 6, 3, CONFY, " [ oK ] ",
-      'k', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
+{ quick_button,   4, 6, 4, CONFY, "&Cancel",
+      0, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "c" },
+{ quick_button,   4, 6, 3, CONFY, "&Ok",
+      0, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
 
-{ quick_checkbox, 1, 13, 6, CONFY, " confirm Exit ",
-      'e', 9, 0, &my_exit, 0, XV_WLAY_BELOWCLOSE, "e" },
-{ quick_checkbox, 1, 13, 5, CONFY, " confirm eXecute ",
-      'x', 10, 0, &my_execute, 0, XV_WLAY_BELOWCLOSE, "x" },
-{ quick_checkbox, 1, 13, 4, CONFY, " confirm Overwrite ",
-      'o', 9, 0, &my_overwrite, 0, XV_WLAY_BELOWCLOSE, "ov" },
-{ quick_checkbox, 1, 13, 3, CONFY, " confirm Delete ",
-      'd', 9, 0, &my_delete, 0, XV_WLAY_BELOWCLOSE, "de" },
-{ 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
+{ quick_checkbox, 1, 13, 6, CONFY, " confirm &Exit ",
+      9, 0, &my_exit, 0, XV_WLAY_BELOWCLOSE, "e" },
+{ quick_checkbox, 1, 13, 5, CONFY, " confirm e&Xecute ",
+      10, 0, &my_execute, 0, XV_WLAY_BELOWCLOSE, "x" },
+{ quick_checkbox, 1, 13, 4, CONFY, " confirm o&Verwrite ",
+      10, 0, &my_overwrite, 0, XV_WLAY_BELOWCLOSE, "ov" },
+{ quick_checkbox, 1, 13, 3, CONFY, " confirm &Delete ",
+      9, 0, &my_delete, 0, XV_WLAY_BELOWCLOSE, "de" },
+{ 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
 };
 
 static QuickDialog confirmation =
@@ -366,15 +383,15 @@ char *display_bits_str [] =
 { "Full 8 bits output", "ISO 8859-1", "7 bits" };
 
 static QuickWidget display_widgets [] = {
-{ quick_button,   4,  6,    4, DISPY, " [ Cancel ]",
-      'c', 3, B_CANCEL, 0, 0, XV_WLAY_CENTERROW, "c" },
-{ quick_button,   4,  6,    3, DISPY, " [ Ok ]",
-      'o', 3, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
-{ quick_checkbox, 4, DISPX, 7, DISPY, "Full 8 bits input",
-      'u', 0, 0, &new_meta, 0, XV_WLAY_BELOWCLOSE, "u" },
-{ quick_radio,    4, DISPX, 3, DISPY, "", 3, 0, 0,
+{ quick_button,   4,  6,    4, DISPY, "&Cancel",
+      0, B_CANCEL, 0, 0, XV_WLAY_CENTERROW, "c" },
+{ quick_button,   4,  6,    3, DISPY, "&Ok",
+      0, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
+{ quick_checkbox, 4, DISPX, 7, DISPY, "F&ull 8 bits input",
+      0, 0, &new_meta, 0, XV_WLAY_BELOWCLOSE, "u" },
+{ quick_radio,    4, DISPX, 3, DISPY, "", 3, 0,
       &new_mode, display_bits_str, XV_WLAY_BELOWCLOSE, "r" },
-{ 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
+{ 0,              0, 0, 0, 0, 0,  0, 0, 0, 0, XV_WLAY_DONTCARE }
 };
 
 static QuickDialog display_bits =
@@ -404,7 +421,7 @@ void display_bits_box ()
 #else
     SLsmg_Display_Eight_Bit = full_eight_bits ? 128 : 160;
 #endif
-    use_8th_bit_as_meta=!new_meta;
+    use_8th_bit_as_meta = !new_meta;
 }
 
 #define TREE_Y 20
@@ -508,38 +525,38 @@ char *confvfs_str [] =
 { "Always to memory", "If size less than:" };
 
 static QuickWidget confvfs_widgets [] = {
-{ quick_button,   VFSX - 10 - 4,  VFSX,    VFSY - 3, VFSY, "[ Cancel ]",
-      'c', 2, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "button-cancel" },
-{ quick_button,   4, VFSX,    VFSY - 3, VFSY, "[ Ok ]",
-      'o', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "button-ok" },
+{ quick_button,   30,  VFSX,    VFSY - 3, VFSY, "&Cancel",
+      0, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "button-cancel" },
+{ quick_button,   12, VFSX,    VFSY - 3, VFSY, "&Ok",
+      0, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "button-ok" },
 #if defined(USE_NETCODE)
-{ quick_input,    30, VFSX, 10, VFSY, 0, 0, 10, 0, 0, &ret_ftp_proxy,
+{ quick_input,    30, VFSX, 10, VFSY, "", 10, 0, 0, &ret_ftp_proxy,
       XV_WLAY_RIGHTDOWN, "input-ftp-proxy" },
-{ quick_checkbox,    4, VFSX, 10, VFSY, "always use ftp proxy", 0, 0, 0,
+{ quick_checkbox,    4, VFSX, 10, VFSY, "&Always use ftp proxy", 0, 0,
       &ftpfs_always_use_proxy, 0, XV_WLAY_RIGHTDOWN, "check-ftp-proxy" },
 { quick_label,    46, VFSX, 9, VFSY, "sec",
-      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec" },
-{ quick_input,    35, VFSX, 9, VFSY, 0, 0, 10, 0, 0, &ret_directory_timeout,
+      0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec" },
+{ quick_input,    35, VFSX, 9, VFSY, "", 10, 0, 0, &ret_directory_timeout,
       XV_WLAY_RIGHTDOWN, "input-timeout" },
 { quick_label,     4, VFSX, 9, VFSY, "ftpfs directory cache timeout:",
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-cache"},
-{ quick_input,    28, VFSX, 8, VFSY, 0, 0, 21, 0, 0, &ret_passwd,
+      0, 0, 0, 0, XV_WLAY_NEXTROW, "label-cache"},
+{ quick_input,    28, VFSX, 8, VFSY, "", 21, 0, 0, &ret_passwd,
       XV_WLAY_RIGHTDOWN, "input-passwd" },
 { quick_label,     4, VFSX, 8, VFSY, "ftp anonymous password:",
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-pass"},
+      0, 0, 0, 0, XV_WLAY_NEXTROW, "label-pass"},
 #endif
-{ quick_input,    26, VFSX, 6, VFSY, 0, 0, 10, 0, 0, &ret_limit, 
+{ quick_input,    26, VFSX, 6, VFSY, "", 10, 0, 0, &ret_limit, 
       XV_WLAY_RIGHTDOWN, "input-limit" },
-{ quick_radio,    4, VFSX, 5, VFSY, "", 2, 0, 0,
+{ quick_radio,    4, VFSX, 5, VFSY, "", 2, 0,
       &vfs_use_limit, confvfs_str, XV_WLAY_BELOWCLOSE, "radio" },
 { quick_label,    4,  VFSX, 4, VFSY, "Gzipped tar archive extract:", 
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-tar" },
+      0, 0, 0, 0, XV_WLAY_NEXTROW, "label-tar" },
 { quick_label,    46, VFSX, 3, VFSY, "sec",
-      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec2" },
-{ quick_input,    35, VFSX, 3, VFSY, 0, 0, 10, 0, 0, &ret_timeout, 
+      0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec2" },
+{ quick_input,    35, VFSX, 3, VFSY, "", 10, 0, 0, &ret_timeout, 
       XV_WLAY_RIGHTOF, "input-timo-vfs" },
 { quick_label,    4,  VFSX, 3, VFSY, "Timeout for freeing VFSs:", 
-      0, 0, 0, 0, 0, XV_WLAY_BELOWCLOSE, "label-vfs" },
+      0, 0, 0, 0, XV_WLAY_BELOWCLOSE, "label-vfs" },
 { 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, 0 }
 };
 
@@ -629,15 +646,13 @@ char *cd_dialog (void)
     QuickWidget quick_widgets [] = {
 #ifdef HAVE_TK
 #define INPUT_INDEX 2
-    { quick_button, 0, 1, 0, 1, "Cancel", 'c', 0, B_CANCEL, 0, 0,
-	  XV_WLAY_DONTCARE, "cancel" },
-    { quick_button, 0, 1, 0, 1, "Ok", 'o', 0, B_ENTER, 0, 0,
-	  XV_WLAY_DONTCARE, "ok" },
+    { quick_button, 0, 1, 0, 1, "&Cancel", 0, B_CANCEL, 0, 0, XV_WLAY_DONTCARE, "cancel" },
+    { quick_button, 0, 1, 0, 1, "&Ok",     0, B_ENTER,  0, 0, XV_WLAY_DONTCARE, "ok" },
 #else
 #define INPUT_INDEX 0
 #endif
-    { quick_input,  8, 80, 5, 0, 0, 0, 50, 0, 0, 0, XV_WLAY_RIGHTOF, "input" },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
+    { quick_input,  8, 80, 5, 0, "", 50, 0, 0, 0, XV_WLAY_RIGHTOF, "input" },
+    { quick_label,  3, 80, 2, 0, "",  0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
     char *my_str;
@@ -671,17 +686,17 @@ void symlink_dialog (char *existing, char *new, char **ret_existing,
     QuickWidget quick_widgets [] = {
 #ifdef HAVE_TK
 #define INPUT_INDEX 2
-    { quick_button, 0, 1, 0, 1, "Cancel", 'c', 0, B_CANCEL, 0, 0,
+    { quick_button, 0, 1, 0, 1, "&Cancel", 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "cancel" },
-    { quick_button, 0, 1, 0, 1, "Ok", 'o', 0, B_ENTER, 0, 0,
+    { quick_button, 0, 1, 0, 1, "&Ok", 0, B_ENTER, 0, 0,
 	  XV_WLAY_DONTCARE, "ok" },
 #else
 #define INPUT_INDEX 0
 #endif
-    { quick_input,  4, 80, 5, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-1" },
-    { quick_label,  4, 80, 4, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_BELOWOF, "label-1" },
-    { quick_input,  4, 80, 3, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-2" },
-    { quick_label,  4, 80, 2, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-2" },
+    { quick_input,  4, 80, 5, 8, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-1" },
+    { quick_label,  4, 80, 4, 8, "", 0, 0, 0, 0, XV_WLAY_BELOWOF, "label-1" },
+    { quick_input,  4, 80, 3, 8, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-2" },
+    { quick_label,  4, 80, 2, 8, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-2" },
     { 0 } };
     
     Quick_input.xlen  = 64;
@@ -776,13 +791,13 @@ jobs_cmd (void)
     bg_list = listbox_new (2, 3, JOBS_X-7, JOBS_Y-9, listbox_nothing, 0, "listbox");
     add_widget (jobs_dlg, bg_list);
     add_widget (jobs_dlg,
-		button_new (JOBS_Y-4, 35, B_CANCEL,"[ Ok ]", 'o', 2,0,0, "button-kill"));
+		button_new (JOBS_Y-4, 35, B_CANCEL, NORMAL_BUTTON, "&Ok", 0, 0, "button-kill"));
     add_widget (jobs_dlg,
-		button_new (JOBS_Y-4, 23, B_KILL,  "[ Kill ]",   'k', 2, task_cb, 0, "button-kill"));
+		button_new (JOBS_Y-4, 23, B_KILL, NORMAL_BUTTON, "&Kill", task_cb, 0, "button-kill"));
     add_widget (jobs_dlg,
-		button_new (JOBS_Y-4, 12, B_RESUME,"[ Resume ]", 'r', 2, task_cb, 0, "button-cont"));
+		button_new (JOBS_Y-4, 12, B_RESUME, NORMAL_BUTTON, "&Resume", task_cb, 0, "button-cont"));
     add_widget (jobs_dlg,
-		button_new (JOBS_Y-4, 3,  B_STOP,  "[ Stop ]",   's', 2, task_cb, 0, "button-stop"));
+		button_new (JOBS_Y-4, 3, B_STOP, NORMAL_BUTTON, "&Stop", task_cb, 0, "button-stop"));
 
     
     /* Insert all of task information in the list */
