@@ -75,15 +75,14 @@ static int single_set;
 static WListbox *l_user, *l_group;
 
 static struct {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
 } chown_but[BUTTONS] = {
-     { B_CANCEL, 0, 54, "[ Cancel ]", 'c', 2, },
-     { B_ENTER, 0, 45, "[ Set ]", 's', 2, },
-     { B_SETUSR, 0, 29, "[ Set users ]", 'u', 6, },
-     { B_SETGRP, 0, 13, "[ Set groups ]", 'g', 6, },
-     { B_SETALL, 0, 0, "[ Set all ]", 'a', 6, },
+    { B_CANCEL, NORMAL_BUTTON,  0, 54, "&Cancel" },
+    { B_ENTER,  DEFPUSH_BUTTON, 0, 44, "&Set" },
+    { B_SETUSR, NORMAL_BUTTON,  0, 30, "Set &users" },
+    { B_SETGRP, NORMAL_BUTTON,  0, 15, "Set &groups" },
+    { B_SETALL, NORMAL_BUTTON,  0, 3,  "Set &all" },
 };
 
 #define LABELS 5 
@@ -101,7 +100,7 @@ static struct {
 #ifndef HAVE_X
 static void chown_refresh (void)
 {
-    attrset (REVERSE_COLOR);
+    attrset (COLOR_NORMAL);
     dlg_erase (ch_dlg);
 
     draw_box (ch_dlg, 1, 2, 16, 70);
@@ -171,7 +170,7 @@ static void init_chown (void)
     ch_dlg = create_dlg (0, 0, 18, 74, dialog_colors, chown_callback,
 			 "[Chown]", "chown", DLG_CENTER);
 
-#define XTRACT(i) BY+chown_but[i].y, BX+chown_but[i].x, chown_but[i].ret_cmd, chown_but[i].text, chown_but[i].hkey, chown_but[i].hpos, 0, 0, NULL
+#define XTRACT(i) BY+chown_but[i].y, BX+chown_but[i].x, chown_but[i].ret_cmd, chown_but[i].flags, chown_but[i].text, 0, 0, NULL
 
     tk_new_frame (ch_dlg, "b.");
     for (i = 0; i < BUTTONS-single_set; i++)
@@ -323,8 +322,16 @@ void chown_cmd (void)
 	}
 	case B_SETALL:
 	case B_ENTER:
-	    new_group = (getgrnam (l_group->current->text))->gr_gid;
-	    new_user = (getpwnam (l_user->current->text))->pw_uid;
+	{
+	    struct group *grp;
+	    struct passwd *user;
+	    
+	    grp = getgrnam (l_group->current->text);
+	    if (grp)
+		new_group = grp->gr_gid;
+	    user = getpwnam (l_user->current->text);
+	    if (user)
+		new_user = user->pw_uid;
 	    if (ch_dlg->ret_value==B_ENTER) {
 		need_update = 1;
 		if (mc_chown (fname, new_user, new_group) == -1)
@@ -334,6 +341,7 @@ void chown_cmd (void)
 		apply_chowns (new_user, new_group);
 	    break;
 	}
+	}
 	
 	if (cpanel->marked && ch_dlg->ret_value != B_CANCEL){
 	    do_file_mark (cpanel, current_file, 0);
