@@ -25,7 +25,7 @@
 #  define INCL_DOSERROR
 #endif
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #include "tty.h"
 #include <stdio.h>
@@ -74,9 +74,12 @@
 #include "fs.h"
 #include "boxes.h"		/* cd_dialog */
 #include "color.h"
+#include "user.h"
+#include "setup.h"
+
 #define MIDNIGHT
 #ifdef  USE_INTERNAL_EDIT
-    extern int edit (const char *file);
+    extern int edit (const char *file, int line);
 #endif
 #include "../vfs/vfs.h"
 #define WANT_WIDGETS
@@ -98,6 +101,11 @@ int source_route = 0;
 /* If set, use the builtin editor */
 int use_internal_edit = 0;
 
+/* Ugly hack in order to distinguish between left and right panel in menubar */
+int is_right;
+#define MENU_PANEL_IDX  (is_right ? 1 : 0)
+
+
 #ifndef HAVE_TK
 #    define x_filter_changed(p)
 #endif
@@ -111,9 +119,9 @@ WPanel *get_a_panel (WPanel *panel)
     if (panel)
 	return panel;
     if (get_current_type () == view_listing){
-	return (WPanel *) get_panel_widget (get_current_index ());
+	return get_current_panel ();
     } else
-	return (WPanel *) get_panel_widget (get_other_index ());
+	return other_panel;
 }
 
 /* view_file (filename, normal, internal)
@@ -225,7 +233,7 @@ static void do_view_cmd (WPanel *panel, int normal)
 	link_isdir (selection (panel))){
 	if (confirm_view_dir && (panel->marked || panel->dirs_marked)){
 	    if (query_dialog (" CD ", "Files tagged, want to cd?",
-			      0, 2, " Yes ", " No ") == 1){
+			      0, 2, "&Yes", "&No") == 1){
 		return;
 	    }
 	}
@@ -262,7 +270,7 @@ void filtered_view_cmd (WPanel *panel)
     char *command;
 
     panel = get_a_panel (panel);
-    command = input_dialog (" Filtered view ", "Filter command and arguments:",
+    command = input_dialog (" Filtered view ", " Filter command and arguments:",
 			    selection (panel)->fname);
     if (!command)
 	return;
@@ -277,13 +285,13 @@ void filtered_view_cmd_cpanel (void)
     filtered_view_cmd (cpanel);
 }
 
-void do_edit (const char *what)
+void do_edit_at_line (const char *what, int start_line)
 {
     static char *editor = 0;
 
 #ifdef USE_INTERNAL_EDIT
     if (use_internal_edit){
-	edit (what);
+	edit (what, start_line);
 	reread_cmd ();
 	return;
     }
@@ -297,6 +305,12 @@ void do_edit (const char *what)
     reread_cmd ();
 }
 
+void
+do_edit (const char *what)
+{
+    do_edit_at_line (what, 1);
+}
+
 void edit_cmd (WPanel *panel)
 {
     panel = get_a_panel(panel);
@@ -342,7 +356,7 @@ void mkdir_cmd (WPanel *panel)
     char *dir;
     
     panel = get_a_panel (panel);
-    dir = input_expand_dialog (" Mkdir ", "Enter directory name:" , "");
+    dir = input_expand_dialog (" Mkdir ", " Enter directory name:" , "");
     
     if (!dir)
 	return;
@@ -382,7 +396,7 @@ void set_panel_filter (WPanel *p)
     
     x = p->filter ? p->filter : easy_patterns ? "*" : ".";
 	
-    reg_exp = input_dialog (" Filter ", "Set expression for filtering filenames", x);
+    reg_exp = input_dialog (" Filter ", " Set expression for filtering filenames", x);
     if (!reg_exp)
 	return;
     if (p->filter){
@@ -564,7 +578,8 @@ static int check_for_default(char *default_file, char *file)
 	    return -1;
 	}
 	create_op_win (OP_COPY, 0);
-	copy_file_file (default_file, file, 0, 1);
+        file_mask_defaults ();
+	copy_file_file (default_file, file, 1);
 	destroy_op_win ();
     }
     return 0;
@@ -580,12 +595,12 @@ void ext_cmd (void)
     if (geteuid () == 0){
 	dir = query_dialog ("Extension file edit",
 			    " Which extension file you want to edit? ", 0, 2,
-			    " User ", " System Wide ");
+			    "&User", "&System Wide");
     }
-    extdir = concat_dir_and_file (mc_home, "mc.ext");
+    extdir = concat_dir_and_file (mc_home, MC_LIB_EXT);
 
     if (dir == 0){
-	buffer = concat_dir_and_file (home_dir, ".mc.ext");
+	buffer = concat_dir_and_file (home_dir, MC_USER_EXT);
 	check_for_default (extdir, buffer);
 	do_edit (buffer);
 	free (buffer);
@@ -598,38 +613,43 @@ void ext_cmd (void)
 
 void menu_edit_cmd (void)
 {
-#define MC_MENU "mc.menu"
-
-    char *buffer, *home_file;
+    char *buffer;
     char *menufile;
     int dir = 0;
     
-    if (geteuid () == 0)
-	dir = query_dialog ("Menu file edit",
-			    " Which menu file will you edit? ", 0, 2,
-			      " User ", " System Wide ");
+    dir = query_dialog (
+	"Menu file edit",
+	" Which menu file will you edit? ", 
+	0, geteuid() ? 2 : 3,
+	"&Local", "&Home", "&System Wide"
+    );
 
-    home_file = copy_strings (home_dir, "/.mc.menu", 0);
+    menufile = concat_dir_and_file(mc_home, MC_GLOBAL_MENU);
 
     switch (dir){
-    case 0:
-	buffer = strdup (home_file);
-        menufile = concat_dir_and_file(mc_home, MC_MENU);
-	check_for_default (menufile, buffer);
-        free (menufile);
-	break;
+	case 0:
+	    buffer = strdup (MC_LOCAL_MENU);
+	    check_for_default (menufile, buffer);
+	    break;
 
-    case 1:
-	buffer = copy_strings (mc_home, MC_MENU, 0);
-	break;
+	case 1:
+	    buffer = concat_dir_and_file (home_dir, MC_HOME_MENU);
+	    check_for_default (menufile, buffer);
+	    break;
+	
+	case 2:
+	    buffer = concat_dir_and_file (mc_home, MC_GLOBAL_MENU);
+	    break;
 
-    default:
-	free (home_file);
-	return;
+	default:
+	    free (menufile);
+	    return;
     }
     do_edit (buffer);
+	if (dir == 0)
+		chmod(buffer, 0600);
     free (buffer);
-    free (home_file);
+    free (menufile);
 }
 
 void quick_chdir_cmd (void)
@@ -667,12 +687,12 @@ static int compare_files (char *name1, char *name2, long size)
     char *data1, *data2;
     int result = -1;		/* Different by default */
 
-#ifdef HAVE_MMAP
-    /* Ugly if jungle */
     file1 = open (name1, O_RDONLY);
     if (file1 >= 0){
 	file2 = open (name2, O_RDONLY);
 	if (file2 >= 0){
+#ifdef HAVE_MMAP
+	    /* Ugly if jungle */
 	    data1 = mmap (0, size, PROT_READ, MAP_FILE | MAP_PRIVATE, file1, 0);
 	    if (data1 != (char*) -1){
 		data2 = mmap (0, size, PROT_READ, MAP_FILE | MAP_PRIVATE, file2, 0);
@@ -683,11 +703,22 @@ static int compare_files (char *name1, char *name2, long size)
 		}
 		munmap (data1, size);
 	    }
+#else
+	    /* Don't have mmap() :( Even more ugly :) */
+	    char buf1[BUFSIZ], buf2[BUFSIZ];
+	    int n1, n2;
+	    rotate_dash ();
+	    do
+	    {
+		while((n1 = read(file1,buf1,BUFSIZ)) == -1 && errno == EINTR);
+		while((n2 = read(file2,buf2,BUFSIZ)) == -1 && errno == EINTR);
+	    } while (n1 == n2 && n1 == BUFSIZ && !memcmp(buf1,buf2,BUFSIZ));
+	    result = (n1 != n2) || memcmp(buf1,buf2,n1);
+#endif
 	    close (file2);
 	}
 	close (file1);
     }
-#endif
     return result;
 }
 
@@ -771,12 +802,10 @@ void compare_dirs_cmd (void)
 {
     enum CompareMode thorough_flag = compare_quick;
 
-#ifdef HAVE_MMAP
     thorough_flag = query_dialog (" Compare directories ", " Select compare method: ",
-				  0, 3, " Quick ", " Size only ", " Thorough ", " Cancel ");
+				  0, 3, "&Quick", "&Size only", "&Thorough", "&Cancel");
     if (thorough_flag < 0 || thorough_flag > 2)
 	return;
-#endif
     if (get_current_type () == view_listing &&
 	get_other_type () == view_listing){
 	compare_dir (cpanel, opanel, thorough_flag);
@@ -826,7 +855,7 @@ void history_cmd (void)
     update_input (input_w (cmdline));
 }
 
-#ifndef HAVE_XVIEW
+#if !defined(HAVE_XVIEW) && !defined(HAVE_GNOME)
 void swap_cmd (void)
 {
     swap_panels ();
@@ -835,7 +864,8 @@ void swap_cmd (void)
 }
 #endif
 
-void view_other_cmd (void)
+void
+view_other_cmd (void)
 {
     static int message_flag = TRUE;
 #ifdef HAVE_SUBSHELL_SUPPORT
@@ -854,12 +884,13 @@ void view_other_cmd (void)
 	    shut_mouse ();
 #endif	    
 
-	if (clear_before_exec) {
+#ifndef HAVE_GNOME
+	if (clear_before_exec)
 	    clr_scr ();
-	}
-
+#endif
+	
 	if (alternate_plus_minus && (console_flag || xterm_flag)) {
-	    fprintf (stderr, "\033>"); fflush (stderr);
+	    fprintf (stdout, "\033>"); fflush (stdout);
 	}
 #ifndef HAVE_SLANG
 	/* With slang we don't want any of this, since there
@@ -898,7 +929,7 @@ void view_other_cmd (void)
 	    do_enter_ca_mode ();
 	    
 	if (alternate_plus_minus && (console_flag || xterm_flag)) {
-	    fprintf (stderr, "\033="); fflush (stderr);
+	    fprintf (stdout, "\033="); fflush (stdout);
 	}
 	reset_prog_mode ();
 	keypad(stdscr, TRUE);
@@ -1340,3 +1371,151 @@ void dirsizes_cmd (void)
 }
 #endif
 
+void
+save_setup_cmd (void)
+{
+    save_setup ();
+    sync_profiles ();
+    message (0, " Setup ", " Setup saved to ~/" PROFILE_NAME);
+}
+
+#ifndef HAVE_GNOME
+void
+info_cmd_no_menu (void)
+{
+    set_display_type (cpanel == left_panel ? 1 : 0, view_info);
+}
+
+void
+quick_cmd_no_menu (void)
+{
+    set_display_type (cpanel == left_panel ? 1 : 0, view_quick);
+}
+
+void
+switch_to_listing (int panel_index)
+{
+    if (get_display_type (panel_index) != view_listing)
+	set_display_type (panel_index, view_listing);
+}
+
+void
+listing_cmd (void)
+{
+    int   view_type, use_msformat;
+    char  *user, *status;
+    WPanel *p;
+    int   display_type, err;
+
+    display_type = get_display_type (MENU_PANEL_IDX);
+    if (display_type == view_listing)
+	p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
+    else
+	p = 0;
+
+    view_type = display_box (p, &user, &status, &use_msformat, MENU_PANEL_IDX);
+
+    if (view_type == -1)
+	return;
+
+    switch_to_listing (MENU_PANEL_IDX);
+
+    p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
+
+    p->user_mini_status = use_msformat; 
+    p->list_type = view_type;
+    
+    if (view_type == list_user || use_msformat){
+    
+	free (p->user_format);
+	p->user_format = user;
+    
+	free (p->user_status_format [view_type]);
+	p->user_status_format [view_type] = status;
+    
+	err = set_panel_formats (p);
+	
+	if (err){
+	    if (err & 0x01){
+	    	free (p->user_format);
+		p->user_format  = strdup (DEFAULT_USER_FORMAT);
+	    }
+		
+	    if (err & 0x02){
+		free (p->user_status_format [view_type]);
+		p->user_status_format [view_type]  = strdup (DEFAULT_USER_FORMAT);
+	    }
+	}
+    }
+    else {
+        free (user);
+        free (status);
+    }
+
+    set_panel_formats (p);
+    paint_panel (p);
+    
+    do_refresh ();
+}
+
+void
+tree_cmd (void)
+{
+    set_display_type (MENU_PANEL_IDX, view_tree);
+}
+
+void
+info_cmd (void)
+{
+    set_display_type (MENU_PANEL_IDX, view_info);
+}
+
+void
+quick_view_cmd (void)
+{
+    set_display_type (MENU_PANEL_IDX, view_quick);
+}
+#endif
+
+/* Handle the tree internal listing modes switching */
+static int
+set_basic_panel_listing_to (int panel_index, int listing_mode)
+{
+    WPanel *p = (WPanel *) get_panel_widget (panel_index);
+
+#ifndef HAVE_GNOME
+    switch_to_listing (panel_index);
+#endif
+    p->list_type = listing_mode;
+    if (set_panel_formats (p))
+	return 0;
+	
+    paint_panel (p);
+    do_refresh ();
+    return 1;
+}
+
+void
+toggle_listing_cmd (void)
+{
+    int current = get_current_index ();
+    WPanel *p = (WPanel *) get_panel_widget (current);
+    int list_mode = p->list_type;
+    int m;
+    
+    switch (list_mode){
+    case list_full:
+    case list_brief:
+	m = list_long;
+	break;
+    case list_long:
+	m = list_user;
+	break;
+    default:
+	m = list_full;
+    }
+    if (set_basic_panel_listing_to (current, m))
+	return;
+    set_basic_panel_listing_to (current, list_full);
+}
+
