@@ -23,6 +23,7 @@
 #include "mad.h"
 #include "setup.h"		/* For the externs */
 #include "color.h"
+#include "x.h"
 
 /* "$Id: color.c,v 1.6 1995/02/21 19:05:18 miguel Exp $" */
 
@@ -35,15 +36,19 @@ int   disable_colors = 0;
 /* Set if we are actually using colors */
 int use_colors = 0;
 
-int sel_mark_color [4];
 int dialog_colors [4];
 
 #define ELEMENTS(arr) ( sizeof(arr) / sizeof((arr)[0]) )
 
-#ifdef HAVE_SLANG
-#   define CTYPE char *
+#ifdef HAVE_GNOME
+#   define CTYPE GdkColor
+void init_pair (int, CTYPE, CTYPE);
 #else
-#   define CTYPE int
+#    ifdef HAVE_SLANG
+#        define CTYPE char *
+#    else
+#        define CTYPE int
+#    endif
 #endif
 
 struct colorpair {
@@ -97,7 +102,7 @@ struct color_table_s {
 
 struct color_table_s color_table [] = {
     { "black",         COLOR_BLACK   },
-    { "gray",         COLOR_BLACK   | A_BOLD },
+    { "gray",          COLOR_BLACK   | A_BOLD },
     { "red",           COLOR_RED     },
     { "brightred",     COLOR_RED     | A_BOLD },
     { "green",         COLOR_GREEN   },
@@ -113,59 +118,85 @@ struct color_table_s color_table [] = {
     { "lightgray",     COLOR_WHITE },
     { "white",         COLOR_WHITE   | A_BOLD }
 };
-#ifdef HAVE_SLANG
-#   define color_value(i) color_table [i].name
-#   define color_name(i)  color_table [i].name
-#else
-#   define color_value(i) color_table [i].value
-#   define color_name(i)  color_table [i].name
-#endif
 
-static void get_color (char **cpp, CTYPE *colp)
+#ifdef HAVE_GNOME
+void get_color (char *cpp, CTYPE *colp);
+#else
+#   ifdef HAVE_SLANG
+#    	define color_value(i) color_table [i].name
+#    	define color_name(i)  color_table [i].name
+#    else
+#    	define color_value(i) color_table [i].value
+#    	define color_name(i)  color_table [i].name
+#    endif
+
+static void get_color (char *cpp, CTYPE *colp)
 {
     int i;
     
     for (i = 0; i < ELEMENTS(color_table); i++){
-	int klen = strlen (color_name (i));
-	
-	if (strncmp (*cpp, color_name (i), klen) == 0){
-	    *cpp += klen;
+	if (strcmp (cpp, color_name (i)) == 0){
 	    *colp = color_value (i);
             return;
 	}
     }
 }
+#endif /* HAVE_GNOME */
 
 static void get_two_colors (char **cpp, struct colorpair *colorpairp)
 {
-    get_color (cpp, &colorpairp->fg);
-    for(;;){
-	if (!**cpp || **cpp == ':')
+    char *p = *cpp;
+    int state;
+
+    state = 0;
+    
+    for (; *p; p++){
+	if (*p == ':'){
+	    *p = 0;
+	    get_color (*cpp, state ? &colorpairp->bg : &colorpairp->fg);
+	    *p = ':';
+	    *cpp = p + 1;
 	    return;
-	if (*(*cpp)++ == ',')
-	    break;
+	}
+
+	if (*p == ','){
+	    state = 1;
+	    *p = 0;
+	    get_color (*cpp, &colorpairp->fg);
+	    *p = ',';
+	    *cpp = p + 1;
+	}
     }
-    get_color (cpp, &colorpairp->bg);
+    get_color (*cpp, state ? &colorpairp->bg : &colorpairp->fg);
 }
 
-void configure_colors_string (char *color_string)
+void configure_colors_string (char *the_color_string)
 {
-    int  i;
-    
+    char *color_string;
+    int  i, found;
+
+    if (!the_color_string)
+	return;
+
+    color_string = strdup (the_color_string);
     while (color_string && *color_string){
 	while (*color_string == ' ' || *color_string == '\t')
 	    color_string++;
 
+	found = 0;
 	for (i = 0; i < ELEMENTS(color_map); i++){
 	    int klen = strlen (color_map [i].name);
 
 	    if (strncmp (color_string, color_map [i].name, klen) == 0){
 		color_string += klen;
 		get_two_colors (&color_string, &color_map [i]);
+		found = 1;
 	    }
 	}
-	while(*color_string && *color_string++ != ':')
-	    ;
+	if (!found){
+		while (*color_string && *color_string != ':')
+			color_string++;
+	}
     }
 }
 
@@ -265,11 +296,6 @@ void init_colors (void)
 	}
     }
 
-    sel_mark_color [0] = NORMAL_COLOR;
-    sel_mark_color [1] = SELECTED_COLOR;
-    sel_mark_color [2] = MARKED_COLOR;
-    sel_mark_color [3] = MARKED_SELECTED_COLOR;
-
     dialog_colors [0] = COLOR_NORMAL;
     dialog_colors [1] = COLOR_FOCUS;
     dialog_colors [2] = COLOR_HOT_NORMAL;
