@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include "x.h"
 #include "mad.h"
 #include "global.h"
 #define DIR_H_INCLUDE_HANDLE_DIRENT
@@ -641,7 +642,7 @@ char *sort_type_to_name (sortfn *sort_fn)
 	if ((sortfn *) (sort_orders [i].sort_fn) == sort_fn)
 	    return sort_orders [i].sort_name;
 
-    return "Unknown";
+    return _("Unknown");
 }
 
 sortfn *sort_name_to_type (char *sname)
