@@ -36,23 +36,13 @@
 #include "dialog.h"	/* For push_refresh() and pop_refresh() */
 #include "layout.h"
 #include "main.h"	
-#ifdef HAVE_XVIEW
-#   include "xvmain.h"
-#endif
-#ifdef HAVE_TK
-#    include "tkmain.h"
+#include "x.h"
 
 /* This is the current frame, used to group Tk packings */
 char *the_frame = "";
 
-#endif
-
 #define waddc(w,y1,x1,c) move (w->y+y1, w->x+x1); addch (c)
 
-#ifdef HAVE_X
-extern Dlg_head *midnight_dlg;
-#endif
-
 /* Primitive way to check if the the current dialog is our dialog */
 /* This is needed by async routines like load_prompt */
 Dlg_head *current_dlg = 0;
@@ -172,11 +162,8 @@ int default_proc (Dlg_head *h, int Msg, int Par)
 	return 0;		/* Didn't use the key */
 	
     case WIDGET_FOCUS:		/* We accept FOCUSes */
-#ifdef HAVE_TK
-	if (h->current){
-	    tk_evalf ("focus %s", ((char *)(h->current->widget->wdata))+1);
-	}
-#endif
+	if (h->current)
+	    x_focus_widget (h->current);
 	return 1;
 	
     case WIDGET_UNFOCUS:	/* We accept loose FOCUSes */
@@ -242,6 +229,7 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
     new_d->has_menubar = 0;
     new_d->name = name;
     new_d->raw = 0;
+    new_d->grided = 0;
     new_d->initfocus = NULL;
 #ifdef HAVE_X
     if (callback != midnight_callback)
@@ -263,9 +251,8 @@ int add_widgetl (Dlg_head *where, void *what, WLay layout)
     Widget_Item *back;
     Widget      *widget = (Widget *) what;
 
-#ifdef HAVE_TK
+    /* Only used by Tk */
     widget->frame = the_frame;
-#endif
     
     widget->layout = layout;
     /* Don't accept 0 widgets, this could be from widgets that could not */
@@ -482,9 +469,7 @@ void update_cursor (Dlg_head *h)
 	do {
 	    if (p->widget->options & W_WANT_CURSOR)
 		if ((*p->widget->callback)(h, p->widget, WIDGET_CURSOR, 0)){
-#ifdef HAVE_TK
-		    tk_focus_widget (p);
-#endif
+		    x_focus_widget (p);
 		    break;
 		}
 	    p = p->next;
@@ -506,8 +491,6 @@ void dlg_redraw (Dlg_head *h)
 
 void dlg_refresh (void *parameter)
 {
-    extern void clr_scr ();
-
     dlg_redraw ((Dlg_head *) parameter);
 }
 
@@ -696,12 +679,6 @@ static INLINE int dlg_mouse_event (Dlg_head *h, Gpm_Event *event)
 
 /* Run dialog routines */
 
-/* Amazing, Solaris linker found this problem */
-#ifdef HAVE_X
-extern widget_data containers [];
-extern int containers_no;
-#endif
-
 /* Init the process */
 void init_dlg (Dlg_head *h)
 {
@@ -740,18 +717,7 @@ void init_dlg (Dlg_head *h)
 	h->current = h->current->next;
 
     h->running = 1;
-#ifdef HAVE_XVIEW
-    if (h != midnight_dlg)
-        xvdlg_show(h, h->wdata);
-    else {
-        int i;
-        for (i = 0; i < containers_no; i++)
-            x_panel_container_show (containers [i]);
-    }
-#endif
-#ifdef HAVE_TK
-    tk_init_dlg (h);
-#endif
+    x_init_dlg (h);
 }
 
 /* Shutdown the run_dlg */
@@ -759,12 +725,8 @@ void dlg_run_done (Dlg_head *h)
 {
     (*h->callback) (h, h->current->dlg_id, DLG_END);
     current_dlg = (Dlg_head *) h->previous_dialog;
-#ifdef HAVE_TK
-    if (current_dlg){
-	char *wdata = (char *) current_dlg->current->widget->wdata;
-	tk_evalf ("focus %s", wdata+1);
-    }
-#endif
+    if (current_dlg)
+	    x_focus_widget (current_dlg->current);
 }
 
 void dlg_process_event (Dlg_head *h, int key, Gpm_Event *event)
@@ -782,27 +744,13 @@ void dlg_process_event (Dlg_head *h, int key, Gpm_Event *event)
 	dlg_key_event (h, key);
 }
 
-
-/* Standard run dialog routine
- * We have to keep this routine small so that we can duplicate it's
- * behavior on complex routines like the file routines, this way,
- * they can call the dlg_process_event without rewriting all the code
- */
-void run_dlg (Dlg_head *h)
+#ifndef PORT_HAS_FRONTEND_RUN_DLG
+static inline void
+frontend_run_dlg (Dlg_head *h)
 {
-#ifndef HAVE_X
     int d_key;
     Gpm_Event event;
-#endif
-    
-    init_dlg (h);
-#ifdef HAVE_X
-#ifdef HAVE_XVIEW
-    xvrundlg_event (h);
-#else
-    tkrundlg_event (h);
-#endif
-#else
+
     event.x = -1;
     while (h->running) {
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
@@ -824,15 +772,27 @@ void run_dlg (Dlg_head *h)
 
 	/* Clear interrupt flag */
 	got_interrupt ();
-	d_key = get_event (&event, h->mouse_status == MOU_REPEAT);
+	d_key = get_event (&event, h->mouse_status == MOU_REPEAT, 1);
 
 	dlg_process_event (h, d_key, &event);
     }
-#endif
+}
+#endif /* PORT_HAS_FRONTEND_RUN_DLG */
+
+/* Standard run dialog routine
+ * We have to keep this routine small so that we can duplicate it's
+ * behavior on complex routines like the file routines, this way,
+ * they can call the dlg_process_event without rewriting all the code
+ */
+void run_dlg (Dlg_head *h)
+{
+    init_dlg (h);
+    frontend_run_dlg (h);
     dlg_run_done (h);
 }
 
-void destroy_dlg (Dlg_head *h)
+void
+destroy_dlg (Dlg_head *h)
 {
     int i;
     Widget_Item *c;
@@ -852,12 +812,7 @@ void destroy_dlg (Dlg_head *h)
     }
     if (h->title)
 	free (h->title);
-#ifdef HAVE_TK
-    tk_evalf ("destroy %s", (char *) h->wdata);
-    tk_dispatch_all ();
-    free ((char *) h->wdata);
-    tk_end_frame ();		/* Cleanup, in case I forget */
-#endif
+    x_destroy_dlg (h);
     free (h);
     if (refresh_list)
 	do_refresh ();
@@ -948,14 +903,7 @@ int dlg_select_nth_widget (Dlg_head *h, int n)
     return dlg_select_widget (h, w->widget);
 }
 
-#ifndef HAVE_XVIEW
-void xv_layout (void *first, ...)
-{
-}
-#endif
-
 #ifdef HAVE_TK
-
 /* Frames must include a trailing dot */
 static void tk_frame_proc (Dlg_head *h, char *frame, int new_frame)
 {
@@ -990,10 +938,6 @@ void tk_end_frame ()
     the_frame = "";
 }
 #else
-void tk_layout (void *first, ...)
-{
-}
-
 void tk_new_frame (Dlg_head *h, char *x)
 {
 }
@@ -1005,8 +949,9 @@ void tk_frame (Dlg_head *h, char *x)
 void tk_end_frame (void)
 {
 }
+#endif
 
-#ifndef HAVE_XVIEW
+#ifndef PORT_HAS_DIALOG_TITLE
 void
 x_set_dialog_title (Dlg_head *h, char *title)
 {
@@ -1014,5 +959,3 @@ x_set_dialog_title (Dlg_head *h, char *title)
 }
 #endif
 
-#endif
-
