@@ -106,7 +106,7 @@ typedef struct Dlg_head {
     int  raw;		/* Should the tab key be sent to the dialog? */
     
     widget_data wdata;
-    
+    int  grided;	/* Does it use the automatic layout? */
 } Dlg_head;
 
 /* XView widget layout */
@@ -291,4 +291,15 @@ void x_set_dialog_title (Dlg_head *h, char *title);
 void dlg_key_event (Dlg_head *h, int d_key);
 void update_cursor (Dlg_head *h);
 
+#ifdef HAVE_X
+extern Dlg_head *midnight_dlg;
+void x_focus_widget (Widget_Item *p);
+void x_init_dlg (Dlg_head *h);
+void x_destroy_dlg (Dlg_head *h);
+#else
+#    define x_focus_widget(x) {}
+#    define x_init_dlg(x)     {}
+#    define x_destroy_dlg(x)  {}
+#endif
+
 #endif /* MC_DLG_H */
