@@ -20,13 +20,15 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <ctype.h>
+#include <stdlib.h>
 #ifdef __os2__
 # include <io.h>
 #endif
 
 #include "tty.h"
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #include <fcntl.h>
 #include <malloc.h>
@@ -90,6 +92,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
     time_t localmtime = 0;
     struct stat mystat;
     int    do_local_copy;
+    char * (*quote_func)(const char *name, int i) = name_quote;
 
     /* Avoid making a local copy if we are doing a cd */
     if (!vfs_file_is_local(filename))
@@ -148,6 +151,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
 	    	    run_view = 1;
 	    	} else if ((i = check_format_cd (data)) > 0) {
 	    	    is_cd = 1;
+		    quote_func = fake_name_quote;
 		    do_local_copy = 0;
 	    	    p = buffer;
 	    	    data += i - 1;
@@ -158,21 +162,25 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
 	        } else {
 		    char *text;
 
-		    if (*data == 'f' && do_local_copy){
-			localcopy = mc_getlocalcopy (filename);
-			if (localcopy == NULL) {
-			    fclose(cmd_file);
-			    unlink(file_name);
-			    free(file_name);
-			    return;
+		    if (*data == 'f'){
+			if (do_local_copy){
+			    localcopy = mc_getlocalcopy (filename);
+			    if (localcopy == NULL) {
+				fclose(cmd_file);
+				unlink(file_name);
+				free(file_name);
+				return;
+			    }
+			    mc_stat (localcopy, &mystat);
+			    localmtime = mystat.st_mtime;
+			    text = (*quote_func) (localcopy, 0);
+			} else {
+			    text = (*quote_func) (filename, 0);
 			}
-			mc_stat (localcopy, &mystat);
-			localmtime = mystat.st_mtime;
-		        text = strdup (localcopy);
 		    } else if (*data == 'q') {
-		        text = strdup (drops);
+		        text = (*quote_func) (drops, 0);
 		    } else
-		        text = expand_format (*data);
+		        text = expand_format (*data, !is_cd);
 		    if (!is_cd)
 		        fputs (text, cmd_file);
 		    else {
@@ -187,7 +195,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
 	    if (*data == '%')
 		expand_prefix_found = 1;
 	    else {
-	        if (*data != ' ' && *data != '\t')
+	        if (!isspace(*data))
 	            written_nonspace = 1;
 	        if (is_cd)
 	            *(p++) = *data;
@@ -223,10 +231,10 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
     	char *q;
     	*p = 0;
     	p = buffer;
-    	while (*p == ' ' || *p == '\t')
+    	while (isspace(*p))
     	    p++;
     	q = p;
-    	while (*q && *q != ' ' && *q != '\t')
+    	while (*q && !isspace(*q))
     	    q++;
     	*q = 0;
     	do_cd (p, cd_parse_command);
@@ -323,12 +331,12 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir)
     if (data == NULL) {
         int home_error = 0;
 
-        buffer = concat_dir_and_file (home_dir, MC_EXT);
+        buffer = concat_dir_and_file (home_dir, MC_USER_EXT);
     	if (exist_file (buffer))
 	    extension_file = buffer;
         else
 check_stock_mc_ext:
-	    extension_file = concat_dir_and_file (mc_home, "mc.ext");
+	    extension_file = concat_dir_and_file (mc_home, MC_LIB_EXT);
         if ((data = load_file (extension_file)) == NULL) {
 	    free (buffer);
 	    return 0;
@@ -344,7 +352,7 @@ check_stock_mc_ext:
 	        } else {
                     char *msg;
                     char *msg2;
-                    msg = copy_strings(" ", mc_home, "mc.ext file error");
+                    msg = copy_strings(" ", mc_home, MC_LIB_EXT " file error");
                     msg2 = copy_strings("Format of the ", 
                                          mc_home, 
 "mc.ext file has changed\n\
@@ -363,8 +371,8 @@ have any, get it from ftp://ftp.nuclecu.unam.mx.", 0);
 	if (home_error) {
             char *msg;
             char *msg2;
-            msg = copy_strings(" ~/" MC_EXT " file error ");
-            msg2 = copy_strings("Format of the ~/" MC_EXT " file has changed\n\
+            msg = copy_strings(" ~/" MC_USER_EXT " file error ");
+            msg2 = copy_strings("Format of the ~/" MC_USER_EXT " file has changed\n\
 with version 3.0. You may want either to\n\
 copy it from ", mc_home, "mc.ext or use that\n\
 file as an example of how to write it.\n\
@@ -382,7 +390,7 @@ file as an example of how to write it.\n\
     easy_patterns = 0; /* Real regular expressions are needed :) */
     include_target = NULL;
     for (p = data; *p; p++) {
-    	for (q = p; *q == ' ' || *q == '\t'; q++);
+    	for (q = p; isspace(*q); q++);
     	if (*q == '\n' || !*q)
     	    p = q; /* empty line */
     	if (*p == '#') /* comment */
@@ -442,10 +450,12 @@ file as an example of how to write it.\n\
 
 		    hasread = 0;
 	    	    if (islocal) {
+			char *tmp = name_quote (filename, 0);
 	    	        char *command =
-			    copy_strings (FILE_CMD, "'", filename, "'", NULL);
+			    copy_strings (FILE_CMD, tmp, NULL);
 	    	        FILE *f = popen (command, "r");
 	    	    
+			free (tmp);
 	    	        free (command);
 	    	        if (f != NULL) {
 	    	            hasread = (fgets (content_string, 2047, f) 
@@ -589,7 +599,7 @@ match_file_output:
     	                    break;
     	            } else if (!strcmp (action, p)) {
     	                *r = c;
-    	                for (p = r + 1; *p == ' ' || *p == '\t'; p++)
+    	                for (p = r + 1; isspace(*p); p++)
 			    ;
 
 			/* Empty commands just stop searching
