@@ -56,6 +56,7 @@
 
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <stdlib.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include "global.h"
@@ -175,31 +176,28 @@ find_par_start:
 			   DLG_CENTER | DLG_GRID);
     x_set_dialog_title (find_dlg, "Find File");
 
-    in_start = input_new (3, 14, INPUT_COLOR, 30, in_start_dir, "start");
-    in_name  = input_new (5, 14, INPUT_COLOR, 30, in_start_name, "name");
+    add_widgetl (find_dlg, button_new (9, 36, B_CANCEL, NORMAL_BUTTON,
+	"&Cancel", 0 ,0, "cancel"), XV_WLAY_RIGHTOF);
+    add_widgetl (find_dlg, button_new (9, 16, B_TREE, NORMAL_BUTTON,
+	"&Tree", 0, 0, "tree"), XV_WLAY_RIGHTOF);
+    add_widgetl (find_dlg, button_new (9, 4, B_ENTER, DEFPUSH_BUTTON,
+	"&Ok", 0, 0, "ok"), XV_WLAY_CENTERROW);
+
     in_with  = input_new (7, 14, INPUT_COLOR, 30, in_contents, "content");
-    
-#ifndef HAVE_XVIEW
-    add_widget (find_dlg, in_start);
-#endif
-    add_widgetl (find_dlg, button_new (9, 36, B_CANCEL,"[ Cancel ]",'c',2,0,0, "cancel"),
-        XV_WLAY_RIGHTOF);
-    
-    add_widgetl (find_dlg, button_new (9, 12, B_TREE, "[ Tree ]", 't', 2,0,0, "tree"),
-        XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, button_new (9, 4, B_ENTER,"[ Ok ]",'o', 2, 0, 0, "ok"),
-        XV_WLAY_CENTERROW);
+    add_widgetl (find_dlg, in_with, XV_WLAY_BELOWOF);
 
-    add_widget (find_dlg, in_with);
-    add_widgetl (find_dlg, in_name, XV_WLAY_RIGHTOF);
+    in_name  = input_new (5, 14, INPUT_COLOR, 30, in_start_name, "name");
+    add_widgetl (find_dlg, in_name, XV_WLAY_BELOWOF);
 
-    add_widgetl (find_dlg, label_new (7, 3, "Content: ", "label-cont"), XV_WLAY_NEXTROW);
-    add_widgetl (find_dlg, label_new (5, 3, "Filename:", "label-file"), XV_WLAY_NEXTROW);
+    in_start = input_new (3, 14, INPUT_COLOR, 30, in_start_dir, "start");
+    add_widgetl (find_dlg, in_start, XV_WLAY_NEXTCOLUMN);
 
-#ifdef HAVE_XVIEW
-    add_widgetl (find_dlg, in_start, XV_WLAY_RIGHTOF);
-#endif    
-    add_widget (find_dlg, label_new (3, 3, "Start at:", "label-start"));
+    add_widgetl (find_dlg, label_new (7, 3, "Content: ", "label-cont"),
+	XV_WLAY_BELOWOF);
+    add_widgetl (find_dlg, label_new (5, 3, "Filename:", "label-file"),
+	XV_WLAY_BELOWOF);
+    add_widgetl (find_dlg, label_new (3, 3, "Start at:", "label-start"),
+	XV_WLAY_NEXTCOLUMN);
 
     run_dlg (find_dlg);
     if (find_dlg->ret_value == B_CANCEL)
@@ -224,10 +222,13 @@ find_par_start:
 	*start_dir = strdup (in_start->buffer);
 	*pattern   = strdup (in_name->buffer);
 
-	if (in_with->buffer [0])
-	    *content   = strdup (in_with->buffer);
-	else
-	    *content   = 0;
+	free (in_contents);
+	if (in_with->buffer [0]){
+	    *content    = strdup (in_with->buffer);
+	    in_contents = strdup (*content);
+	} else 
+	    *content = in_contents = NULL;
+
 	free (in_start_dir);
 	in_start_dir = strdup (*start_dir);
 	free (in_start_name);
@@ -556,7 +557,7 @@ do_search (struct Dlg_head *h)
 }
 
 static int
-view_currently_selected_file (int unparsed_view)
+view_edit_currently_selected_file (int unparsed_view, int edit)
 {
     WLEntry *entry = find_list->current;
     char *dir, *fullname, *filename;
@@ -584,7 +585,10 @@ view_currently_selected_file (int unparsed_view)
     else
 	 fullname = get_full_name (dir, filename);
 
-    view_file_at_line (fullname, unparsed_view, use_internal_view, line);
+    if (edit)
+	do_edit_at_line (fullname, line);
+    else
+        view_file_at_line (fullname, unparsed_view, use_internal_view, line);
     free (fullname);
     return MSG_HANDLED;
 }
@@ -602,7 +606,10 @@ find_callback (struct Dlg_head *h, int id, int Msg)
     case DLG_KEY:
 	if (id == KEY_F(3) || id == KEY_F(13)){
 	    int unparsed_view = (id == KEY_F(13));
-	    return view_currently_selected_file (unparsed_view);
+	    return view_edit_currently_selected_file (unparsed_view, 0);
+	}
+	if (id == KEY_F(4)){
+	    return view_edit_currently_selected_file (0, 1);
 	 }
 	 return MSG_NOT_HANDLED;
 
@@ -613,20 +620,11 @@ find_callback (struct Dlg_head *h, int id, int Msg)
      return 0;
 }
 
-#define BUTTON_NEW(i,msg,str,name) \
-   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, 0, 0,name)
-
-#define BUTTON_NEW_CBACK(i,msg,str,a,b,name) \
-   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, a, b, name)
-
-#define BUTTON_NEW_CBACKY(i,j,msg,str,a,b,name) \
-   button_new (FIND2_Y-4+j, 3+i*12, msg, str, str [2], 2, a, b, name)
-
 /* Handles the Stop/Start button in the find window */
 static int
 start_stop (int button, void *extra)
 {
-    char *button_labels [2] = { "[ Stop ] ", "[ Start ]" };
+    char *button_labels [2] = { "Stop", "Start" };
     running = is_start;
     set_idle_proc (find_dlg, running);
     is_start = !is_start;
@@ -641,7 +639,15 @@ start_stop (int button, void *extra)
 static int
 find_do_view_file (int button, void *extra)
 {
-    view_currently_selected_file (0);
+    view_edit_currently_selected_file (0, 0);
+    return 0;
+}
+
+/* Handle edit command, when invoked as a button */
+static int
+find_do_edit_file (int button, void *extra)
+{
+    view_edit_currently_selected_file (0, 1);
     return 0;
 }
 
@@ -674,34 +680,27 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
     
     x_set_dialog_title (find_dlg, "Find file");
 
-    find_list = listbox_new (2, 2, FIND2_X-4, FIND2_Y-9, listbox_finish, 0, "listbox");
-
-#ifndef HAVE_X
-    add_widget (find_dlg, find_list);
-#endif
-
     add_widgetl (find_dlg,
-		button_new (FIND2_Y-3, 3, B_PANELIZE, "[ Panelize ]", 'p', 2, 0, 0, "button-panelize"),
-		XV_WLAY_CENTERROW);
+    button_new (FIND2_Y-3, 32, B_VIEW, NORMAL_BUTTON, "&Edit - F4", find_do_edit_file, find_dlg, "button-edit"), 0);
     add_widgetl (find_dlg,
-		 BUTTON_NEW_CBACKY(2, 1,B_VIEW, "[ View - F3 ]", find_do_view_file, find_dlg, "button-view"),
-		 0);
-    add_widgetl (find_dlg, BUTTON_NEW (3, B_CANCEL, "[ Quit ]", "button-quit"),
+    button_new (FIND2_Y-3, 17, B_VIEW, NORMAL_BUTTON, "&View - F3", find_do_view_file, find_dlg, "button-view"), 0);
+    add_widgetl (find_dlg,
+	button_new (FIND2_Y-3, 3, B_PANELIZE, NORMAL_BUTTON, "&Panelize", 0, 0, "button-panelize"),
+	XV_WLAY_CENTERROW);
+    add_widgetl (find_dlg, button_new (FIND2_Y-4, 37, B_CANCEL, NORMAL_BUTTON, "&Quit", 0, 0, "button-quit"), 
         XV_WLAY_RIGHTOF);
-    stop_button = BUTTON_NEW_CBACK (2, B_STOP,   "[ Stop  ]", start_stop,
-				    find_dlg, "start-stop");
-    add_widgetl (find_dlg, stop_button,
+    stop_button = button_new (FIND2_Y-4, 27, B_STOP, NORMAL_BUTTON, "&Stop", start_stop, find_dlg, "start-stop");
+    add_widgetl (find_dlg, stop_button, XV_WLAY_RIGHTOF);
+    add_widgetl (find_dlg, button_new (FIND2_Y-4, 16, B_AGAIN, NORMAL_BUTTON, "&Again", 0, 0, "button-again"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, BUTTON_NEW (1, B_AGAIN,  "[ Again ]", "button-again"),
-        XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, BUTTON_NEW (0, B_ENTER,  "[ Chdir ]", "button-chdir"), 
+    add_widgetl (find_dlg, button_new (FIND2_Y-4, 3, B_ENTER, DEFPUSH_BUTTON, "&Chdir", 0, 0, "button-chdir"),
         XV_WLAY_CENTERROW);
 
     status_label = label_new (FIND2_Y-6, 4, "Searching", "label-search");
     add_widgetl (find_dlg, status_label, XV_WLAY_BELOWOF);
-#ifdef HAVE_X
+
+    find_list = listbox_new (2, 2, FIND2_X-4, FIND2_Y-9, listbox_finish, 0, "listbox");
     add_widgetl (find_dlg, find_list, XV_WLAY_EXTENDWIDTH);
-#endif
 
     /* FIXME: Need to cleanup this, this ought to be passed non-globaly */
     find_pattern    = pattern;
@@ -793,6 +792,8 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
 	    cpanel->has_dir_sizes = 0;
 	    cpanel->marked = 0;
 	    cpanel->total = 0;
+	    cpanel->top_file = 0;
+	    cpanel->selected = 0;
 
 	    if (start_dir [0] == PATH_SEP){
 		strcpy (cpanel->cwd, PATH_SEP_STR);
@@ -801,9 +802,6 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
 	}
     }
 
-    /* cpanel->top_file and cpanel->selected must be set to a valid
-     * value before call of destroy_dlg which calls paint_panel  */
-    try_to_select (cpanel, NULL);
     destroy_dlg (find_dlg);
     do_search (0); /* force do_search to release resources */
     if (old_dir){
