@@ -41,7 +41,7 @@
 #	include <sys/timeb.h>	/* alex: for struct timeb, used in time.h */
 #endif /* SCO_FLAVOR */
 #include <time.h>
-#ifndef __os2__
+#ifndef OS2_NT
 #    include <grp.h>
 #    include <pwd.h>
 #else
@@ -127,22 +127,21 @@ static struct {
 }   hotlist_state;
 
 struct _hotlist_but {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
     char *tkname;
     int   type;
 } hotlist_but[] = {
-    { B_MOVE,         1,   42, "[ Move ]",        'm',  2, "move",	LIST_HOTLIST},
-    { B_REMOVE,       1,   30, "[ Remove ]",      'r',  2, "r",		LIST_HOTLIST},
-    { B_APPEND,       1,   15, "[[ Append ]]",    'a',  3, "e",		LIST_MOVELIST},
-    { B_INSERT,       1,    0, "[ Insert ]",      'i',  2, "g",		LIST_MOVELIST},
-    { B_NEW_ENTRY,    1,   15, "[ New Entry ]",   'e',  6, "e",		LIST_HOTLIST},
-    { B_NEW_GROUP,    1,    0, "[ New Group ]",   'g',  6, "g",		LIST_HOTLIST},
-    { B_CANCEL,       0,   53, "[ Cancel ]",      'c',  2, "cc",	LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
-    { B_UP_GROUP,     0,   35, "[ Up ]",          'u',  2, "up",	LIST_HOTLIST|LIST_MOVELIST},
-    { B_ADD_CURRENT,  0,   18, "[ Add current ]", 'a',  2, "ad",	LIST_HOTLIST},
-    { B_ENTER,        0,    0, "[[ Change To ]]", 't', 10, "ct",	LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
+    { B_MOVE, NORMAL_BUTTON,         1,   42, "&Move",       "move", LIST_HOTLIST},
+    { B_REMOVE, NORMAL_BUTTON,       1,   30, "&Remove",     "r",    LIST_HOTLIST},
+    { B_APPEND, NORMAL_BUTTON,       1,   15, "&Append",     "e",    LIST_MOVELIST},
+    { B_INSERT, NORMAL_BUTTON,       1,    0, "&Insert",     "g",    LIST_MOVELIST},
+    { B_NEW_ENTRY, NORMAL_BUTTON,    1,   15, "New &Entry",  "e",    LIST_HOTLIST},
+    { B_NEW_GROUP, NORMAL_BUTTON,    1,    0, "New &Group",  "g",    LIST_HOTLIST},
+    { B_CANCEL, NORMAL_BUTTON,       0,   53, "&Cancel",     "cc",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
+    { B_UP_GROUP, NORMAL_BUTTON,     0,   42, "&Up",         "up",   LIST_HOTLIST|LIST_MOVELIST},
+    { B_ADD_CURRENT, NORMAL_BUTTON,  0,   20, "&Add current","ad",   LIST_HOTLIST},
+    { B_ENTER, DEFPUSH_BUTTON,       0,    0, "Change &To",  "ct",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
 };
 
 /* Directory hotlist */
@@ -178,8 +177,8 @@ static struct hotlist *new_hotlist (void)
 #ifndef HAVE_X
 static void hotlist_refresh (Dlg_head *dlg)
 {
-    dialog_repaint (dlg, REVERSE_COLOR, COLOR_HOT_NORMAL);
-    attrset (REVERSE_COLOR);
+    dialog_repaint (dlg, COLOR_NORMAL, COLOR_HOT_NORMAL);
+    attrset (COLOR_NORMAL);
     draw_box (dlg, UY, UX, UY+(LINES-14), COLS-2*UX-6);
     if (!hotlist_state.moving)
 	draw_box (dlg, UY+(LINES-12), UX, 3, COLS-2*UX-6);
@@ -351,6 +350,7 @@ static int hotlist_button_callback (int action, void *data)
 	    x_listbox_select_nth (l_hotlist, 0);
 #endif
 	    repaint_screen ();
+	    hotlist_state.modified = 1;
 	    return 0;
 	    break;
 	}
@@ -535,7 +535,7 @@ static void init_hotlist (int list_type)
     x_set_dialog_title (hotlist_dlg,
         list_type == LIST_VFSLIST ? "Active VFS directories" : "Directory hotlist");
 
-#define XTRACT(i) BY+hotlist_but[i].y, BX+hotlist_but[i].x, hotlist_but[i].ret_cmd, hotlist_but[i].text, hotlist_but[i].hkey, hotlist_but[i].hpos, hotlist_button_callback, 0, hotlist_but[i].tkname
+#define XTRACT(i) BY+hotlist_but[i].y, BX+hotlist_but[i].x, hotlist_but[i].ret_cmd, hotlist_but[i].flags, hotlist_but[i].text, hotlist_button_callback, 0, hotlist_but[i].tkname
 
     for (i = 0; i < BUTTONS; i++){
 	if (hotlist_but[i].type & list_type)
@@ -587,7 +587,7 @@ static void init_movelist (int list_type, struct hotlist *item)
     x_set_dialog_title (movelist_dlg, hdr);
     free (hdr);
 
-#define XTRACT(i) BY-4+hotlist_but[i].y, BX+hotlist_but[i].x, hotlist_but[i].ret_cmd, hotlist_but[i].text, hotlist_but[i].hkey, hotlist_but[i].hpos, hotlist_button_callback, 0, hotlist_but[i].tkname
+#define XTRACT(i) BY-4+hotlist_but[i].y, BX+hotlist_but[i].x, hotlist_but[i].ret_cmd, hotlist_but[i].flags, hotlist_but[i].text, hotlist_button_callback, 0, hotlist_but[i].tkname
 
     for (i = 0; i < BUTTONS; i++){
 	if (hotlist_but[i].type & list_type)
@@ -694,16 +694,16 @@ static int add_new_entry_input (char *header, char *text1, char *text2, char *he
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 55, 80, 4, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
+    { quick_button, 55, 80, 4, 0, "&Cancel", 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 30, 80, 4, 0, "[ Insert ]", 'i', 2, B_INSERT, 0, 0,
+    { quick_button, 30, 80, 4, 0, "&Insert", 0, B_INSERT, 0, 0,
 	  XV_WLAY_DONTCARE, "button-insert" },
-    { quick_button, 10, 80, 4, 0, "[[ Append ]]", 'a', 3, B_APPEND, 0, 0,
+    { quick_button, 10, 80, 4, 0, "&Append", 0, B_APPEND, 0, 0,
 	  XV_WLAY_DONTCARE, "button-append" },
-    { quick_input,  4, 80, 4, 0, "", 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
-    { quick_label,  3, 80, 3, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
-    { quick_input,  4, 80, 3, 0, "", 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-lbl" },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-lbl" },
+    { quick_input,  4, 80, 4, 0, "",58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
+    { quick_label,  3, 80, 3, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
+    { quick_input,  4, 80, 3, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-lbl" },
+    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-lbl" },
     { 0 } };
     
     int len;
@@ -769,13 +769,13 @@ static int add_new_group_input (char *header, char *label, char **result)
     int		ret;
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 55, 80, 1, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
+    { quick_button, 55, 80, 1, 0, "&Cancel", 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 30, 80, 1, 0, "[ Insert ]", 'i', 2, B_INSERT, 0, 0,
+    { quick_button, 30, 80, 1, 0, "&Insert", 0, B_INSERT, 0, 0,
 	  XV_WLAY_DONTCARE, "button-insert" },
-    { quick_button, 10, 80, 1, 0, "[[ Append ]]", 'a', 3, B_APPEND, 0, 0,
+    { quick_button, 10, 80, 1, 0, "&Append", 0, B_APPEND, 0, 0,
 	  XV_WLAY_DONTCARE, "button-append" },
-    { quick_input,  4, 80, 0, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input" },
+    { quick_input,  4, 80,  0, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input" },
     { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
@@ -877,8 +877,8 @@ static void remove_from_hotlist (struct hotlist *entry)
 				   " ",
 				   0);
 	    result = query_dialog (header, "\n Group not empty.\n Remove it?",
-				   D_ERROR | WITH_HOTKEYS, 2,
-				   "n No ", "y Yes ");
+				   D_ERROR, 2,
+				   "&No", "&Yes");
 	    free (header);
 
 	    if (!result)
@@ -1272,7 +1272,7 @@ void load_hotlist (void)
 	   "  Keep   - keep your old entries; you will be asked\n" /**/
 	   "           the same question next time\n" /**/
 	   "  Merge  - add old entries to hotlist as group \"Entries from ~/" PROFILE_NAME "\"\n\n",
-				   D_ERROR, 3, " Remove ", " Keep ", " Merge "
+				   D_ERROR, 3, "&Remove", "&Keep", "&Merge"
 		      );
 	    if (result == 0)
 		remove_old_list = 1;
