@@ -5,6 +5,7 @@
    Written by: 1994, 1995 Miguel de Icaza.
                1994, 1995 Janne Kukonlehto.
 	       1995  Jakub Jelinek.
+	       1997  Norbert Warmuth
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -54,11 +56,10 @@
 #    else
 #        include <termios.h>
 #    endif
+#    include <sys/ioctl.h>
 #endif
 
-#ifdef HAVE_TK
-#   include "tkmain.h"
-#endif
+#include "x.h"
 
 /* "$Id: key.c,v 1.18 1995/02/21 19:06:13 miguel Exp $" */
 
@@ -76,11 +77,7 @@ int old_esc_mode = 0;
 
 int use_8th_bit_as_meta = 1;
 
-#ifdef HAVE_XVIEW
-int xmouse_flag = 0;
-#endif
-
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 typedef struct key_def {
     char ch;			/* Holds the matching char code */
     int code;			/* The code returned, valid if child == NULL */
@@ -98,8 +95,9 @@ static key_def *keys = 0;
 static int input_fd;
 static fd_set select_set;
 static int disabled_channels = 0; /* Disable channels checking */
+int xgetch_second (void);
 
-#ifndef HAVE_TK
+#ifndef PORT_HAS_FILE_HANDLERS
 /* File descriptor monitoring add/remove routines */
 typedef struct SelectList {
     int fd;
@@ -109,28 +107,7 @@ typedef struct SelectList {
 } SelectList;
 
 SelectList *select_list = 0;
-#endif
 
-int xgetch_second (void);
-
-#ifdef HAVE_TK
-void add_select_channel (int fd, select_fn callback, void *info)
-{
-    Tcl_File handle;
-
-    handle = Tcl_GetFile ((ClientData) fd, TCL_UNIX_FD);
-    Tcl_CreateFileHandler (handle, TCL_READABLE, (Tcl_FileProc *) callback, 0);
-}
-
-void delete_select_channel (int fd)
-{
-    Tcl_File handle;
-
-    handle = Tcl_GetFile ((ClientData) fd, TCL_UNIX_FD);
-    Tcl_DeleteFileHandler (handle);
-    Tcl_FreeFile (handle);
-}
-#else
 void add_select_channel (int fd, select_fn callback, void *info)
 {
     SelectList *new;
@@ -209,7 +186,7 @@ typedef struct {
     int action;
 } key_define_t;
 
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 key_define_t mc_bindings [] = {
     { KEY_END,    ESC_STR ">", MCKEY_NOACTION },
     { KEY_HOME,   ESC_STR "<", MCKEY_NOACTION },
@@ -249,7 +226,7 @@ key_define_t mc_default_keys [] = {
 
 void define_sequences (key_define_t *kd)
 {
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
     int i;
     
     for (i = 0; kd [i].code; i++)
@@ -257,9 +234,11 @@ void define_sequences (key_define_t *kd)
 #endif	
 }
 
+/* This has to be called before slang_init or whatever routine
+   calls any define_sequence */
 void init_key (void)
 {
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
     char *term = (char *) getenv ("TERM");
     
     /* This has to be the first define_sequence */
@@ -271,14 +250,21 @@ void init_key (void)
 	define_sequences (xterm_key_defines);
     
     define_sequences (mc_bindings);
-    
-#ifdef HAVE_SLANG
-    input_fd = fileno (stdin);
-#endif
-#endif /* !HAVE_XVIEW */
+#endif /* !HAVE_X */
 }
 
-#ifndef HAVE_XVIEW
+/* This has to be called after SLang_init_tty/slint_init */
+void init_key_input_fd (void)
+{
+#ifndef HAVE_X
+#ifdef HAVE_SLANG
+    input_fd = SLang_TT_Read_FD;
+#endif
+#endif /* !HAVE_X */
+}
+
+
+#ifndef HAVE_X
 void xmouse_get_event (Gpm_Event *ev)
 {
     int btn;
@@ -311,13 +297,13 @@ void xmouse_get_event (Gpm_Event *ev)
 	
         switch (btn) {
 	case 0:
-            ev->buttons |= GPM_B_LEFT;
+            ev->buttons = GPM_B_LEFT;
             break;
 	case 1:
-            ev->buttons |= GPM_B_MIDDLE;
+            ev->buttons = GPM_B_MIDDLE;
             break;
 	case 2:
-            ev->buttons |= GPM_B_RIGHT;
+            ev->buttons = GPM_B_RIGHT;
             break;
 	default:
             /* Nothing */
@@ -366,11 +352,11 @@ static int push_char (int c)
     *seq_append = 0;
     return 1;
 }
-#endif /* !HAVE_XVIEW */
+#endif /* !HAVE_X */
 
 void define_sequence (int code, char *seq, int action)
 {
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
     key_def *base;
 
     if (strlen (seq) > sizeof (seq_buffer)-1)
@@ -403,7 +389,7 @@ void define_sequence (int code, char *seq, int action)
 #endif    
 }
 
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 static int *pending_keys;
 #endif
 
@@ -433,7 +419,7 @@ int correct_key_code (int c)
 
 int get_key_code (int no_delay)
 {
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
     int c;
     static key_def *this = NULL, *parent;
     static struct timeval esctime = { -1, -1 };
@@ -600,10 +586,10 @@ int get_key_code (int no_delay)
     return correct_key_code (c);
 #else
     return ERR;
-#endif /* HAVE_XVIEW */
+#endif /* HAVE_X */
 }
 
-#ifndef HAVE_TK
+#ifndef PORT_HAS_FILE_HANDLERS
 /* If set timeout is set, then we wait 0.1 seconds, else, we block */
 void try_channels (int set_timeout)
 {
@@ -633,7 +619,7 @@ void try_channels (int set_timeout)
     }
 }
 
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 /* Workaround for System V Curses vt100 bug */
 static int getch_with_delay (void)
 {
@@ -655,12 +641,12 @@ static int getch_with_delay (void)
     /* Success -> return the character */
     return c;
 }
-#endif
+#endif /* !HAVE_X */
 
 #ifndef HAVE_LIBGPM
 #define gpm_flag 0
 #endif
-#endif /* !HAVE_XVIEW */
+#endif /* !HAVE_FILE_HANDLERS */
 
 extern int max_dirt_limit;
 
@@ -668,11 +654,9 @@ extern int max_dirt_limit;
 /* Also takes care of generated mouse events */
 /* Returns EV_MOUSE if it is a mouse event */
 /* Returns EV_NONE  if non-blocking or interrupt set and nothing was done */
-/* The current behavior is to block allways */
-int get_event (Gpm_Event *event, int redo_event)
+int get_event (Gpm_Event *event, int redo_event, int block)
 {
 #ifndef HAVE_X
-#define block 1
     int c;
     static int flag;			/* Return value from select */
 #ifdef HAVE_LIBGPM
@@ -749,7 +733,6 @@ int get_event (Gpm_Event *event, int redo_event)
 	    /* select timed out: it could be for any of the following reasons:
 	     * redo_event -> it was because of the MOU_REPEAT handler
 	     * !block     -> we did not block in the select call
-	     *               (this is currently unused).
 	     * else       -> 10 second timeout to check the vfs status.
 	     */
 	    if (flag == 0){
@@ -783,7 +766,7 @@ int get_event (Gpm_Event *event, int redo_event)
     untouchwin (stdscr);
 #   endif
 
-    c = getch_with_delay ();
+    c = block ? getch_with_delay () : get_key_code(1);
     
 #   ifndef HAVE_SLANG
     if (flag)
@@ -798,27 +781,21 @@ int get_event (Gpm_Event *event, int redo_event)
     return c;
 #else
     return EV_NONE;
-#endif /* HAVE_XVIEW */
+#endif /* HAVE_X */
 }
 
+#ifndef PORT_HAS_GETCH
 /* Returns a key press, mouse events are discarded */
 int mi_getch ()
 {
-#ifdef HAVE_X
-#ifdef HAVE_TK
-    return tk_getch ();
-#else
-    /* FIXME: XView requires this routine to work with quote */
-#endif /* HAVE_TK */
-#else
     Gpm_Event ev;
     int       key;
     
-    while ((key = get_event (&ev, 0)) == 0)
+    while ((key = get_event (&ev, 0, 1)) == 0)
 	;
     return key;
-#endif /* HAVE_X */
 }
+#endif
 
 int xgetch_second (void)
 {
