@@ -26,9 +26,8 @@
 #include <string.h>
 #include <sys/param.h>		/* Required by tree.h */
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/stat.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
 #    include <termios.h>
 #endif
 #ifdef HAVE_UNISTD_H
@@ -67,30 +66,22 @@
 #define WANT_DEFAULTS
 #include "setup.h"		/* For save_setup() */
 
-#ifdef HAVE_TK
-#   include "tkmain.h"
-#   include "tkscreen.h"
-#endif
+#include "x.h"
 
 /* "$Id: layout.c,v 1.2 1995/02/21 19:06:17 miguel Exp $" */
 
-/* The maximum number of views managed by the set_display_type routine */
-/* Must be at least two (for current and other).  Please note that until */
-/* Janne gets around this, we will only manage two of them :-) */
-#define MAX_VIEWS 2
-
-/* If set, then we have to call the layout_change routine from main */
-int layout_do_change = 0;
-
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
 
-/* Set if the window has changed it's size */
-int winch_flag = 0;
+/* If set, then we have to call the layout_change routine from main */
+int layout_do_change = 0;
 
 /* Set if the panels are split horizontally */
 int horizontal_split = 0;
 
+/* Set if the window has changed it's size */
+int winch_flag = 0;
+
 /* Set if the split is the same */
 int equal_split = 1;
 
@@ -115,7 +106,12 @@ int xterm_hintbar = 0;
 /* The starting line for the output of the subprogram */
 int output_start_y = 0;
 
-static struct {
+/* The maximum number of views managed by the set_display_type routine */
+/* Must be at least two (for current and other).  Please note that until */
+/* Janne gets around this, we will only manage two of them :-) */
+#define MAX_VIEWS 2
+
+struct {
     int    type;
     Widget *widget;
 } panels [MAX_VIEWS];
@@ -136,6 +132,8 @@ static int _command_prompt;
 static int _keybar_visible;
 static int _message_visible;
 static int _xterm_hintbar;
+static int _permission_mode;
+static int _filetype_mode;
 
 static int height;
 
@@ -143,7 +141,7 @@ static int height;
 #define MINHEIGHT 5
 
 #define BX      12
-#define BY      11
+#define BY      12
 
 #define B_2LEFT B_USER
 #define B_2RIGHT B_USER + 1
@@ -159,18 +157,18 @@ static struct {
     char   *text;
     int    *variable;
     WCheck *widget;
-    int    hkey;
-    int    hpos;
     char   *tkname;
 } check_options [] = {
-    { "Xterm hintbar",	  &xterm_hintbar,0,    'X', 0, "h" },
-    { "hIntbar visible",  &message_visible,0,  'I', 1, "v" },
-    { "Keybar visible",   &keybar_visible, 0,  'K', 0, "k" },
-    { "command Prompt",   &command_prompt, 0,  'P', 8, "p" },
-    { "show Mini status", &show_mini_info, 0,  'M', 5, "m" },
-    { "menuBar visible",  &menubar_visible, 0, 'B', 4, "me" },
-    { "Equal split",      &equal_split, 0,     'E', 0, "eq" },
-    { 0, 0, 0, 0, 0 }
+    { "&Xterm hintbar",	   &xterm_hintbar,   0, "h" },
+    { "h&Intbar visible",  &message_visible, 0, "v" },
+    { "&Keybar visible",   &keybar_visible,  0, "k" },
+    { "command &Prompt",   &command_prompt,  0, "p" },
+    { "show &Mini status", &show_mini_info,  0, "m" },
+    { "menu&Bar visible",  &menubar_visible, 0, "me" },
+    { "&Equal split",      &equal_split,     0, "eq" },
+    { "pe&Rmissions",      &permission_mode, 0, "pr" },
+    { "&File types",	   &filetype_mode,   0, "ft" },
+    { 0, 0, 0, 0 }
 };
 
 static WButton *bleft_widget, *bright_widget;
@@ -209,10 +207,10 @@ static void update_split (void)
     old_first_panel_size = _first_panel_size;
     old_horizontal_split = _horizontal_split; 
    
-    attrset (REVERSE_COLOR);
-    dlg_move (layout_dlg, 6, 11);
+    attrset (COLOR_NORMAL);
+    dlg_move (layout_dlg, 6, 6);
     printw ("%03d", _first_panel_size);
-    dlg_move (layout_dlg, 6, 20);
+    dlg_move (layout_dlg, 6, 18);
     if (_horizontal_split)
 	printw ("%03d", height - _first_panel_size);
     else
@@ -247,7 +245,8 @@ static int b2right_cback (int action, void *data)
 
 static int bplus_cback (int action, void *data)
 {
-    _output_lines++;
+    if (_output_lines < 99)
+	_output_lines++;
     return 0;
 }
 
@@ -268,35 +267,41 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
    	old_first_panel_size = -1;
     	old_horizontal_split = -1;
     	old_output_lines     = -1;
-	attrset (REVERSE_COLOR);
+	attrset (COLOR_NORMAL);
 	dlg_erase (h);
-	draw_box (h, 1, 2, 12, 56);
-	draw_box (h, 2, 5, 6, 24);
-	draw_box (h, 2, 31, 9, 24);
+	draw_box (h, 1, 1, 13, 56);
+	draw_box (h, 2, 4, 6, 23);
+	draw_box (h, 2, 29, 10, 25);
+	draw_box (h, 8, 4, 4, 23);
 
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 24);
+	dlg_move (h, 1, 25);
 	addstr (" Layout ");
-	dlg_move (h, 2, 6);
+	dlg_move (h, 2, 5);
 	addstr (" Panel split ");
-	dlg_move (h, 2, 32);
+	dlg_move (h, 8, 5);
+	addstr (" Highlight... ");
+	dlg_move (h, 2, 30);
 	addstr (" Other options ");
 	update_split ();
-	dlg_move (h, 6, 16);
-	addstr ("--");
+	dlg_move (h, 6, 13);
+	addch ('=');
 	if (console_flag){
 	    if (old_output_lines != _output_lines){
 		old_output_lines = _output_lines;
+		attrset (COLOR_NORMAL);
+		dlg_move (h, 9, 40);
+		addstr ("output lines");
 		dlg_move (h, 9, 34);
 		printw ("%02d", _output_lines);
-		dlg_move (h, 9, 38);
-		addstr ("lines of output");
 	    }
 	}
 #endif
 	break;
 
     case DLG_POST_KEY:
+	_filetype_mode = check_options [8].widget->state & C_BOOL;
+	_permission_mode = check_options [7].widget->state & C_BOOL;
 #ifndef HAVE_X
 	_equal_split = check_options [6].widget->state & C_BOOL;
 #endif
@@ -333,6 +338,7 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 	if (console_flag){
 	    if (old_output_lines != _output_lines){
 		old_output_lines = _output_lines;
+		attrset (COLOR_NORMAL);
 		dlg_move (h, 9, 34);
 		printw ("%02d", _output_lines);
 	    }
@@ -349,41 +355,50 @@ static void init_layout (void)
 {
     int i;
 
-    layout_dlg = create_dlg (0, 0, 14, 60, dialog_colors, layout_callback,
+    layout_dlg = create_dlg (0, 0, 15, 58, dialog_colors, layout_callback,
 			     "[Layout]", "layout", DLG_CENTER | DLG_GRID);
 			     
     x_set_dialog_title (layout_dlg, "Layout");
 
     add_widgetl (layout_dlg,
-		button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0, "c"), 
+		button_new (BY, BX+25, B_CANCEL, NORMAL_BUTTON, "&Cancel", 0, 0, "c"), 
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0, "s"),
+		button_new (BY, BX+12, B_EXIT, NORMAL_BUTTON, "&Save", 0, 0, "s"),
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0, "o"),
+		button_new (BY, BX, B_ENTER, DEFPUSH_BUTTON, "&Ok", 0, 0, "o"),
 		XV_WLAY_CENTERROW);
 #ifndef HAVE_X
     if (console_flag){
 	add_widget (layout_dlg,
-		    button_new (9, 36, B_MINUS, "-", '-', 0, bminus_cback, 0, NULL));
+		    button_new (9, 36, B_MINUS, NARROW_BUTTON, "&-", bminus_cback, 0, NULL));
 	add_widget (layout_dlg,
-		    button_new (9, 33, B_PLUS, "+", '+', 0, bplus_cback, 0, NULL));
+		    button_new (9, 31, B_PLUS, NARROW_BUTTON, "&+", bplus_cback, 0, NULL));
     }
 #endif    
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text,                             check_options[i].hkey, check_options[i].hpos,  check_options[i].tkname
+#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].tkname
 
     for (i = 0; i < 6; i++){
-	check_options [i].widget = check_new (8 - i, 33, XTRACT(i));
+	check_options [i].widget = check_new (8 - i, 31, XTRACT(i));
 	add_widgetl (layout_dlg, check_options [i].widget, XV_WLAY_BELOWCLOSE);
     }
 #ifdef HAVE_XVIEW
-    add_widgetl (layout_dlg, label_new (2, 32, " Other options ", "oo"), 
+    add_widgetl (layout_dlg, label_new (2, 30, "Other options", "oo"),
         XV_WLAY_NEXTCOLUMN);
-    add_widgetl (layout_dlg, label_new (2, 6, " Panel split ", "ps"),
+    add_widgetl (layout_dlg, label_new (8, 5, "Highlight...", "hl"),
+        XV_WLAY_NEXTCOLUMN);
+    add_widgetl (layout_dlg, label_new (2, 5, "Panel split", "ps"),
         XV_WLAY_NEXTCOLUMN);
 #endif        
+    check_options [8].widget = check_new (10, 6, XTRACT(8));
+    add_widgetl (layout_dlg, check_options [8].widget, XV_WLAY_BELOWCLOSE);
+    check_options [7].widget = check_new (9, 6, XTRACT(7));
+    add_widgetl (layout_dlg, check_options [7].widget, XV_WLAY_BELOWCLOSE);
+
+    _filetype_mode = filetype_mode;
+    _permission_mode = permission_mode;
     _equal_split = equal_split;
     _menubar_visible = menubar_visible;
     _command_prompt = command_prompt;
@@ -391,11 +406,11 @@ static void init_layout (void)
     _message_visible = message_visible;
     _xterm_hintbar = xterm_hintbar;
 #ifndef HAVE_X
-    bright_widget = button_new(6, 18, B_2RIGHT, ">", '>', 0, b2right_cback, 0, ">");
+    bright_widget = button_new(6, 15, B_2RIGHT, NARROW_BUTTON, "&>", b2right_cback, 0, ">");
     add_widgetl (layout_dlg, bright_widget, XV_WLAY_RIGHTOF);
-    bleft_widget = button_new (6, 15, B_2LEFT, "<", '<', 0, b2left_cback, 0, "<");
+    bleft_widget = button_new (6, 9, B_2LEFT, NARROW_BUTTON, "&<", b2left_cback, 0, "<");
     add_widgetl (layout_dlg, bleft_widget, XV_WLAY_RIGHTOF);
-    check_options [6].widget = check_new (5, 8, XTRACT(6));
+    check_options [6].widget = check_new (5, 6, XTRACT(6));
 #endif
     old_first_panel_size = -1;
     old_horizontal_split = -1;
@@ -405,7 +420,7 @@ static void init_layout (void)
     _output_lines = output_lines;
 #ifndef HAVE_X
     add_widget (layout_dlg, check_options [6].widget);
-    radio_widget = radio_new (3, 8, 2, s_split_direction, 1, "r");
+    radio_widget = radio_new (3, 6, 2, s_split_direction, 1, "r");
     add_widget (layout_dlg, radio_widget);
     radio_widget->sel = horizontal_split;
 #endif
@@ -434,12 +449,14 @@ void layout_cmd (void)
 
     if (result == B_ENTER || result == B_EXIT){
 	for (i = 0; check_options [i].text; i++)
-	    *check_options [i].variable = check_options [i].widget->state & C_BOOL;
+	    if (check_options [i].widget)
+		*check_options [i].variable = check_options [i].widget->state & C_BOOL;
+#ifndef HAVE_X		
 	horizontal_split = radio_widget->sel;
 	first_panel_size = _first_panel_size;
 	output_lines = _output_lines;
-
 	layout_do_change = 1;
+#endif	
     }
     if (result == B_EXIT){
 	save_layout ();
@@ -474,9 +491,18 @@ int panel_event    (Gpm_Event *event, WPanel *panel);
 int menu_bar_event (Gpm_Event *event, void *);
 extern char *prompt;
 
+#ifndef HAVE_X
 #ifdef HAVE_SLANG
 void init_curses ()
 {
+    extern int force_ugly_line_drawing;
+    extern int SLtt_Has_Alt_Charset;
+    SLtt_get_terminfo ();
+#ifndef OS2_NT
+    if (force_ugly_line_drawing)
+	SLtt_Has_Alt_Charset = 0;
+#endif
+    SLsmg_init_smg ();
     do_enter_ca_mode ();
     init_colors ();
     keypad (stdscr, TRUE);
@@ -485,6 +511,7 @@ void init_curses ()
 #else
 void init_curses (void)
 {
+    initscr();
     if (!status_using_ncurses)
 	do_enter_ca_mode ();
     mc_raw_mode ();
@@ -493,19 +520,38 @@ void init_curses (void)
     nodelay (stdscr, FALSE);
     init_colors ();
 }
-#endif
+#endif /* ! HAVE_SLANG */
+void done_screen ()                                                      
+{                                                                               
+    if (!(quit & SUBSHELL_EXIT))                                                
+	clr_scr ();                                                             
+    reset_shell_mode ();                                                        
+    mc_noraw_mode ();                                                           
+    if (use_mouse_p)                                                            
+	shut_mouse ();                                                          
+    keypad (stdscr, FALSE);                                                     
+}                                                                               
+#else
+void init_curses ()
+{
+}
+void done_screen ()
+{
+}
+#endif /* HAVE_X */
 
 void panel_do_cols (int index)
 {
     if (get_display_type (index) == view_listing)
-	panel_update_format ((WPanel *) panels [index].widget);
+	set_panel_formats ((WPanel *) panels [index].widget);
     else {
 	panel_update_cols (panels [index].widget, frame_half);
     }
 }
 
 #ifdef HAVE_X
-void setup_panels (void)
+void
+setup_panels (void)
 {
     Widget *w = panels [0].widget;
 
@@ -519,7 +565,8 @@ void setup_panels (void)
     if (panels [1].type == view_listing){
 	x_panel_set_size (1);
     }
-    
+
+    load_hint ();
 #ifdef HAVE_XVIEW
     panel_do_cols (0);
     panel_do_cols (1);
@@ -580,8 +627,8 @@ void setup_panels (void)
     if (command_prompt) {
 	    widget_set_size (&cmdline->input.widget,
 			     LINES-1-keybar_visible, promptl,
-			     1, COLS-promptl);
-	    winput_set_origin (&cmdline->input, promptl, COLS-promptl);
+			     1, COLS-promptl-(keybar_visible ? 0 : 1));
+	    winput_set_origin (&cmdline->input, promptl, COLS-promptl-(keybar_visible ? 0 : 1));
 	    widget_set_size (&the_prompt->widget,
 			     LINES-1-keybar_visible, 0,
 			     1, promptl);
@@ -656,7 +703,6 @@ void change_screen_size (void)
      * more than one curses implementation does this); it is guaranteed to work
      * only with slang.
      */
-    initscr ();
     init_curses ();
 #endif
     setup_panels ();
