@@ -5,6 +5,7 @@
 
 void layout_cmd (void);
 void init_curses (void);
+void done_screen (void);
 void setup_panels (void);
 void destroy_panels (void);
 void move_resize_panel (void);
@@ -25,15 +26,24 @@ Widget *get_panel_widget (int index);
 void set_hintbar (char *str);
 
 extern int winch_flag;
-extern int horizontal_split;
 extern int equal_split;
 extern int first_panel_size;
 extern int output_lines;
 extern int command_prompt;
 extern int keybar_visible;
 extern int layout_do_change;
-extern int nice_rotating_dash;
 extern int output_start_y;
 extern int message_visible;
 extern int xterm_hintbar;
+
+#ifdef HAVE_GNOME
+#    define nice_rotating_dash 1
+#    define horizontal_split   0
+#    define xterm_hintbar      0
+#    define message_visible    1
+#else
+extern int horizontal_split;
+extern int nice_rotating_dash;
+#endif
+
 #endif /* __LAYOUT_H */
