@@ -40,7 +40,7 @@
 #include "key.h"
 #include "setup.h"
 #include "main.h"
-#define UX		3
+#define UX		4
 #define UY		3
 
 #define BX		25
@@ -52,12 +52,11 @@
 #define BUTTONS 	2
 
 struct {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
 } learn_but[BUTTONS] = {
-    { B_CANCEL, 0, 14, "[ Cancel ]",  'c', 2, },
-    { B_ENTER,  0,  0, "[[ Save ]]", 's', 3, },
+    { B_CANCEL, NORMAL_BUTTON, 0, 14, "&Cancel" },
+    { B_ENTER, DEFPUSH_BUTTON, 0,  0, "&Save" }
 };
 
 static Dlg_head *learn_dlg;
@@ -75,20 +74,20 @@ static int learnchanged;
 #ifndef HAVE_X
 static void learn_refresh (void)
 {
-    attrset (REVERSE_COLOR);
+    attrset (COLOR_NORMAL);
     dlg_erase (learn_dlg);
     
-    draw_box (learn_dlg, 1, 2, 21, 70);
+    draw_box (learn_dlg, 1, 2, 21, 73);
     
     attrset (COLOR_HOT_NORMAL);
-    dlg_move (learn_dlg, 1, (72 - 19) / 2);
+    dlg_move (learn_dlg, 1, (72 - 12) / 2);
     addstr (" Learn keys ");
 }
 #endif
 
 static int learn_button (int action, void *param)
 {
-    char *seq;
+    unsigned char *seq;
     Dlg_head *d = message (D_INSERT | 1, " Teach me a key ",
 "Please press the %s\n"
 "and then wait until this message disappears.\n\n"
@@ -98,8 +97,10 @@ static int learn_button (int action, void *param)
 "and wait as well.", 
         key_name_conv_tab [action - B_USER].longname);
     mc_refresh ();
-    if (learnkeys [action - B_USER].sequence != NULL)
-        free (learnkeys [action - B_USER].sequence);
+    if (learnkeys [action - B_USER].sequence != NULL) {
+	free (learnkeys [action - B_USER].sequence);
+	learnkeys [action - B_USER].sequence = NULL;
+    }
     seq = learn_key ();
 
     if (seq){
@@ -108,13 +109,16 @@ static int learn_button (int action, void *param)
 	 */
 	if (*seq && strcmp (seq, "\\e") && strcmp (seq, "\\e\\e")
 	    && strcmp (seq, "^m" ) 
-            && (seq [1] || (*seq < ' ' && *seq > '~'))){
+            && (seq [1] || (*seq < ' ' || *seq > '~'))){
 	    
 	    learnchanged = 1;
 	    learnkeys [action - B_USER].sequence = seq;
 	    seq = convert_controls (seq);
 	    define_sequence (key_name_conv_tab [action - B_USER].code, seq, 
 			     MCKEY_NOACTION);
+	} else {
+	    message (0, " Cannot accept this key ",
+		" You have entered \"%s\"", seq);
 	}
 	
     	free (seq);
@@ -168,14 +172,14 @@ static int learn_check_key (int c)
 	            learn_dlg->ret_value = B_CANCEL;
 	            if (learnchanged) {
 	                if (query_dialog (" Learn keys ", "\
-It seems that all your keys already\n\
-work fine. That's great.",
-	                    1, 2, " Save ", " Discard ") == 0)
+			    It seems that all your keys already\n\
+			    work fine. That's great.",
+	                    1, 2, "&Save", "&Discard") == 0)
 	                    learn_dlg->ret_value = B_ENTER;
 	            } else {
 	            	message (1, " Learn keys ", "\
-Great! You have a complete terminal database!\n\
-All your keys work well.");
+			Great! You have a complete terminal database!\n\
+			All your keys work well.");
 	            }
 	            learn_dlg->running = 0;
 	        }
@@ -227,12 +231,12 @@ static void init_learn (void)
 
     do_refresh ();
 
-    learn_dlg = create_dlg (0, 0, 23, 75, dialog_colors,
+    learn_dlg = create_dlg (0, 0, 23, 78, dialog_colors,
 			      learn_callback, "[Learn keys]", "Learn keys",
 			      DLG_CENTER);
     x_set_dialog_title (learn_dlg, "Learn keys");
 
-#define XTRACT(i) BY+learn_but[i].y, BX+learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].text, learn_but[i].hkey, learn_but[i].hpos, 0, 0, NULL
+#define XTRACT(i) BY+learn_but[i].y, BX+learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].flags, learn_but[i].text, 0, 0, NULL
 
     for (i = 0; i < BUTTONS; i++)
 	add_widget (learn_dlg, button_new (XTRACT (i)));
@@ -250,11 +254,11 @@ static void init_learn (void)
     for (i = j - 1, key = key_name_conv_tab + j - 1; i >= 0; i--, key--) {
     	learnkeys [i].ok = 0;
     	learnkeys [i].sequence = NULL;
-        sprintf (buffer, " %-18s", key->longname);
+        sprintf (buffer, "%-16s", key->longname);
 	add_widget (learn_dlg, learnkeys [i].button = (Widget *)
-	    button_new (y, x, B_USER + i, buffer, 0, -1, learn_button, 0, NULL));
+	    button_new (y, x, B_USER + i, NARROW_BUTTON, buffer, learn_button, 0, NULL));
 	add_widget (learn_dlg, learnkeys [i].label = (Widget *)
-	    label_new (y, x + 20, "", NULL));
+	    label_new (y, x + 19, "", NULL));
 	if (i % 13)
 	    y--;
 	else {
