@@ -55,6 +55,9 @@ extern int option_backspace_through_tabs;
 extern int option_fake_half_tabs;
 extern int option_save_mode;
 extern int option_backup_ext_int;
+extern int option_auto_para_formatting;
+extern int option_typewriter_wrap;
+extern int edit_confirm_save;
 #endif	/* ! USE_INTERNAL_EDIT */
 extern int fast_reload_w;
 extern int clear_before_exec;
@@ -115,6 +118,7 @@ typedef struct {
 } key_map;
 
 void update_panels (int force_update, char *current_file, char *other_file);
+void create_panels (void);
 void repaint_screen (void);
 void outrefresh_screen (void);
 void suspend_cmd (void);
@@ -168,6 +172,13 @@ extern Dlg_head   *midnight_dlg;
 extern WLabel      *process_status;
 #include "menu.h"
 extern WMenu      *the_menubar;
+
+extern Dlg_head *midnight_dlg;
 #endif
 
+#ifdef OS2_NT
+#    define MC_BASE ""
+#else
+#    define MC_BASE "/.mc/"
+#endif
 #endif
