@@ -27,7 +27,7 @@
 #endif
 #include <signal.h>	/* For kill() and SIGQUIT */
 #include <fcntl.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
 #    include <termios.h>
 #endif
 #include <malloc.h>
