@@ -59,10 +59,12 @@ enum {
 };
 
 /* When using Slang with color, we have all the indexes free but
-   those defined here (A_BOLD, A_UNDERLINE, A_REVERSE) */
+ * those defined here (A_BOLD, A_UNDERLINE, A_REVERSE, A_BOLD_REVERSE)
+ */
 #define A_BOLD      0x40
 #define A_UNDERLINE 0x40
 #define A_REVERSE   0x20
+#define A_BOLD_REVERSE 0x21
 
 #ifndef A_NORMAL
 #    define A_NORMAL    0x00
@@ -74,6 +76,8 @@ enum {
 #define TRUE 1
 #define FALSE 0
 
+void slang_set_raw_mode (void);
+
 #define doupdate()
 #define raw() slang_set_raw_mode()
 #define noraw()
@@ -101,11 +105,15 @@ int getch (void);
 void slang_keypad (int set);
 void slang_shell_mode (void);
 void slang_shutdown (void);
-void init_pair (int, char *, char *);
 int has_colors (void);
 /* Internal function prototypes */
 void load_terminfo_keys ();
 
+/* FIXME Clean up this; gnome has nothing to do here */
+#ifndef HAVE_GNOME
+void init_pair (int, char *, char *);
+#endif
+
 /* copied from slcurses.h (MC version 4.0.7) */
 #define move SLsmg_gotorc
 #define clreol SLsmg_erase_eol
@@ -134,9 +142,7 @@ void load_terminfo_keys ();
 #define inch SLsmg_char_at
 #define endwin SLsmg_reset_smg
 
-#ifndef OS2_NT
 #define SLsmg_draw_double_box(r,c,dr,dc) SLsmg_draw_box ((r), (c), (dr), (dc))
-#endif
 
 #if 0
 #   define one_vline() {acs (); addch (ACS_VLINE); noacs ();}
