@@ -42,22 +42,22 @@
 #include "panel.h"		/* Needed for the externs */
 #include "file.h"
 #include "layout.h"		/* For nice_rotating_dash */
-#define PX	5
+#define PX	4
 #define PY	2
 
-#define RX      5
+#define RX      4
 #define RY      11
 
-#define CX	5
+#define CX	4
 #define CY	2
 
-#define BX	15
+#define BX	14
 #define BY	16
 
-#define OX	34
+#define OX	33
 #define OY	2
 
-#define TX	36
+#define TX	35
 #define TY	11
 
 
@@ -74,28 +74,28 @@ static struct {
     int    *variable;
     void   (*toggle_function)(void);
     WCheck *widget;
-    int    hkey;
-    int    hpos;
     char   *tk;
 } check_options [] = {
-   {"safe deLete",       &know_not_what_am_i_doing, TOGGLE_VARIABLE,    0,'l', 7, "safe-del" },
-   {"cd follows linKs",  &cd_symlinks,       TOGGLE_VARIABLE,       0,'k',14, "cd-follow" },
-   {"advanced choWn",    &advanced_chfns,    TOGGLE_VARIABLE,       0,'w',12, "achown" },
-   {"lYnx-like motion",  &navigate_with_arrows,TOGGLE_VARIABLE,     0,'y', 1, "lynx" },
-   {"Rotating dash",     &nice_rotating_dash,TOGGLE_VARIABLE,       0,'r', 0, "rotating" },
-   {"Complete: show all",&show_all_if_ambiguous,TOGGLE_VARIABLE,    0,'c', 0, "completion" },
-   {"Use internal view", &use_internal_view, TOGGLE_VARIABLE,       0,'u', 0, "view-int" },
-   {"use internal edIt", &use_internal_edit, TOGGLE_VARIABLE,       0,'i',15, "edit-int" },
-   {"auto mEnus",        &auto_menu,         TOGGLE_VARIABLE,       0,'e', 6, "auto-menus" },
-   {"Auto save setup",   &auto_save_setup,   TOGGLE_VARIABLE,       0,'a', 0, "auto-save" },
-   {"shell Patterns",    &easy_patterns,     TOGGLE_VARIABLE,       0,'p', 6, "shell-patt" },
-   {"Verbose operation", &verbose,           TOGGLE_VARIABLE,       0,'v', 0, "verbose" },
-   {"Fast dir reload",   &fast_reload,       toggle_fast_reload,    0,'f', 0, "fast-reload" },
-   {"miX all files",     &mix_all_files,     toggle_mix_all_files,  0,'x', 2, "mix-files" },
-   {"Drop down menus",   &drop_menus,        TOGGLE_VARIABLE,       0,'d', 0, "drop-menus" },
-   {"Mark moves down",   &mark_moves_down,   TOGGLE_VARIABLE,       0,'m', 0, "mark-moves" },
-   {"show Hidden files", &show_dot_files,    toggle_show_hidden,    0,'h', 5, "show-hidden" },
-   {"show Backup files", &show_backups,      toggle_show_backup,    0,'b', 5, "show-backup" },
+   {"safe de&Lete",       &know_not_what_am_i_doing, TOGGLE_VARIABLE,0, "safe-del" },
+   {"cd follows lin&Ks",  &cd_symlinks,       TOGGLE_VARIABLE,       0, "cd-follow" },
+   {"advanced cho&Wn",    &advanced_chfns,    TOGGLE_VARIABLE,       0, "achown" },
+   {"l&Ynx-like motion",  &navigate_with_arrows,TOGGLE_VARIABLE,     0, "lynx" },
+#ifndef HAVE_GNOME
+   {"ro&Tating dash",     &nice_rotating_dash,TOGGLE_VARIABLE,       0, "rotating" },
+#endif
+   {"co&Mplete: show all",&show_all_if_ambiguous,TOGGLE_VARIABLE,    0, "completion" },
+   {"&Use internal view", &use_internal_view, TOGGLE_VARIABLE,       0, "view-int" },
+   {"use internal ed&It", &use_internal_edit, TOGGLE_VARIABLE,       0, "edit-int" },
+   {"auto m&Enus",        &auto_menu,         TOGGLE_VARIABLE,       0, "auto-menus" },
+   {"&Auto save setup",   &auto_save_setup,   TOGGLE_VARIABLE,       0, "auto-save" },
+   {"shell &Patterns",    &easy_patterns,     TOGGLE_VARIABLE,       0, "shell-patt" },
+   {"&Verbose operation", &verbose,           TOGGLE_VARIABLE,       0, "verbose" },
+   {"&Fast dir reload",   &fast_reload,       toggle_fast_reload,    0, "fast-reload" },
+   {"mi&X all files",     &mix_all_files,     toggle_mix_all_files,  0, "mix-files" },
+   {"&Drop down menus",   &drop_menus,        TOGGLE_VARIABLE,       0, "drop-menus" },
+   {"ma&Rk moves down",   &mark_moves_down,   TOGGLE_VARIABLE,       0, "mark-moves" },
+   {"show &Hidden files", &show_dot_files,    toggle_show_hidden,    0, "show-hidden" },
+   {"show &Backup files", &show_backups,      toggle_show_backup,    0, "show-backup" },
    { 0, 0, 0, 0 }
 };
 
@@ -110,13 +110,24 @@ static int configure_callback (struct Dlg_head *h, int Id, int Msg)
 {
     switch (Msg) {
     case DLG_DRAW:
-	attrset (REVERSE_COLOR);
+#ifndef HAVE_X    
+	attrset (COLOR_NORMAL);
 	dlg_erase (h);
-	
-	draw_box (h, 1, 2, 17, 62);
+	draw_box (h, 1, 1, 17, 62);
 	draw_box (h, PY, PX, 8, 27);
 	draw_box (h, OY, OX, 14, 27);
 	draw_box (h, RY, RX, 5, 27);
+
+	attrset (COLOR_HOT_NORMAL);
+	dlg_move (h, 1, (62 - 20)/2);
+	addstr (" Configure options ");
+	dlg_move (h, OY, OX+1);
+	addstr (" Other options ");
+	dlg_move (h, RY, RX+1);
+	addstr (" Pause after run... ");
+	dlg_move (h, PY, PX+1);
+	addstr (" Panel options ");
+#endif
 	break;
 
     case DLG_END:
@@ -130,24 +141,24 @@ static void init_configure (void)
 {
     int i;
 
-    conf_dlg = create_dlg (0, 0, 19, 66, dialog_colors,
+    conf_dlg = create_dlg (0, 0, 19, 64, dialog_colors,
 			   configure_callback, "[Options Menu]",
 			   "option", DLG_CENTER | DLG_GRID);
     x_set_dialog_title (conf_dlg, "Configure options");
 
     add_widgetl (conf_dlg,
-	button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0, "button-cancel"),
+	button_new (BY, BX+26, B_CANCEL, NORMAL_BUTTON, "&Cancel", 0, 0, "button-cancel"),
 	XV_WLAY_RIGHTOF);
 
     add_widgetl (conf_dlg,
-	button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0, "button-save"),
+	button_new (BY, BX+12, B_EXIT, NORMAL_BUTTON, "&Save", 0, 0, "button-save"),
 	XV_WLAY_RIGHTOF);
     
     add_widgetl (conf_dlg,
-        button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0, "button-ok"),
+        button_new (BY, BX, B_ENTER, DEFPUSH_BUTTON, "&Ok", 0, 0, "button-ok"),
         XV_WLAY_CENTERROW);
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].hkey, check_options[i].hpos, check_options [i].tk
+#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options [i].tk
 
     /* Add all the checkboxes */
     for (i = 0; i < 12; i++){
@@ -156,24 +167,24 @@ static void init_configure (void)
 	    XV_WLAY_BELOWCLOSE);
     }
 
-    add_widgetl (conf_dlg, label_new (OY, OX + 1, "Other options", "label-other"),
-        XV_WLAY_NEXTCOLUMN);
-
     pause_radio = radio_new (RY+1, RX+1, 3, pause_options, 1, "pause-radio");
     pause_radio->sel = pause_after_run;
     add_widgetl (conf_dlg, pause_radio, XV_WLAY_BELOWCLOSE);
 
-    add_widgetl (conf_dlg, label_new (RY, RX + 1, "Pause after run...", "label-pause"),
-        XV_WLAY_BELOWOF);
-
     for (i = 0; i < 6; i++){
 	check_options [i+12].widget = check_new (PY + (6-i), PX+2,
 						  XTRACT(i+12));
 	add_widgetl (conf_dlg, check_options [i+12].widget,
 	    XV_WLAY_BELOWCLOSE);
     }
-    add_widgetl (conf_dlg, label_new (PY, PX + 1, "Panel options", "label-panel"),
-        XV_WLAY_NEXTCOLUMN);
+#ifdef HAVE_XVIEW
+     add_widgetl (conf_dlg, label_new (OY, OX + 1, "Other options", "label-other"),
+         XV_WLAY_NEXTCOLUMN);
+     add_widgetl (conf_dlg, label_new (RY, RX + 1, "Pause after run...", "label-pause"),
+         XV_WLAY_BELOWOF);
+     add_widgetl (conf_dlg, label_new (PY, PX + 1, "Panel options", "label-panel"),
+         XV_WLAY_NEXTCOLUMN);
+#endif
 }
 
 
