@@ -3,6 +3,8 @@
 
 #include "dlg.h"
 
+#define LIST_TYPES	4
+
 enum list_types {
     list_full,			/* Name, size, perm/date */
     list_brief,			/* Name */
@@ -28,20 +30,33 @@ enum panel_display_enum {
 #define J_LEFT  0
 #define J_RIGHT 1
 
-#define NORMAL         0
-#define SELECTED       1
-#define MARKED         2
-#define MARKED_SELECTED        3
-#define STATUS         5
-
+#define NORMAL		0
+#define SELECTED	1
+#define MARKED		2
+#define MARKED_SELECTED	3
+#define STATUS		5
 
+/*
+ * This describes a format item.  The parse_display_format routine parses
+ * the user specified format and creates a linked list of format_e structures.
+ *
+ * parse_display_format computes the actual field allocations if
+ * the COMPUTE_FORMAT_ALLOCATIONs define is set.  MC frontends that are
+ * just interested in the parsed display format should not set this define.
+ */
 typedef struct format_e {
     struct format_e *next;
-    int  field_len;
-    char *(*string_fn)(file_entry *, int len);
-    int  just_mode;
-    char *title;
-    char *id;
+    int    requested_field_len;
+    int    field_len;
+    int    just_mode;
+    int    expand;
+    char   *(*string_fn)(file_entry *, int len);
+    char   *title;
+    char   *id;
+
+    /* first format_e has the number of items */
+    int    items;
+    int    use_in_gui;
 } format_e;
 
 typedef struct {
@@ -68,10 +83,9 @@ typedef struct {
 
     int      dirty;		/* Should we redisplay the panel? */
 
-
-    int	     user_mini_status;  /* Is mini_status_format used */
-    char     *user_format;      /* User format */
-    char     *mini_status_format;/* User format for mini status line */
+    int	     user_mini_status;  	/* Is user_status_format used */
+    char     *user_format;      	/* User format */
+    char     *user_status_format[LIST_TYPES];/* User format for status line */
 
     format_e *format;		/* Display format */
     format_e *status_format;    /* Mini status format */
@@ -97,14 +111,15 @@ typedef struct {
 /* This macro extracts the number of available lines in a panel */
 #define llines(p) (p->widget.lines-3 - (show_mini_info ? 2 : 0))
 
-WPanel *panel_new ();
+WPanel *panel_new (char *panel_name);
 void panel_set_size (WPanel *panel, int x1, int y1, int x2, int y2);
 void paint_paint (WPanel *panel);
 void panel_refresh (WPanel *panel);
 void Xtry_to_select (WPanel *panel, char *name);
 
 extern int torben_fj_mode;
-extern int highlight_mode;
+extern int permission_mode;
+extern int filetype_mode;
 extern int show_mini_info;
 extern int panel_scroll_pages;
 
@@ -149,17 +164,16 @@ extern WPanel *the_info_panel;
 void paint_frame (WPanel *panel);
 void panel_update_contents (WPanel *panel);
 void panel_update_cols (Widget *widget, int frame_size);
-void panel_update_format (WPanel *panel);
-format_e *parse_display_format (WPanel *panel, char *format, 
-				char **error, int isstatus);
+format_e *use_display_format (WPanel *panel, char *format, char **error, int isstatus);
 char *panel_format (WPanel *panel);
 char *mini_status_format (WPanel *panel);
-char *set_panel_format (WPanel *p, char *format);
+int set_panel_formats (WPanel *p);
 
 extern WPanel *current_panel;
-extern WPanel *get_other_panel ();
 
-#define other_panel get_other_panel(__LINE__)
+#define get_current_panel() ((WPanel *) get_panel_widget (get_current_index ()))
+#define get_other_panel() ((WPanel *) get_panel_widget (get_other_index ()))
+#define other_panel get_other_panel()
 
 extern WPanel *left_panel;
 extern WPanel *right_panel;
@@ -197,6 +211,8 @@ char *string_file_nuid (file_entry *fe, int len);
 
 void file_mark (WPanel *panel, int index, int val);
 void do_file_mark (WPanel *panel, int index, int val);
+int  file_compute_color (int attr, file_entry *fe);
+int  file_entry_color (file_entry *fe);
 
 /* NOTE: Have to be ifdefed for HAVE_X */
 void x_panel_select_item (WPanel *panel, int index, int val);
