@@ -33,8 +33,10 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <grp.h>
-#include <pwd.h>
+#ifndef OS2_NT
+#   include <grp.h>
+#   include <pwd.h>
+#endif
 #include "tty.h"
 #include "mad.h"
 #include "util.h"		/* Needed for the externs */
@@ -80,15 +82,14 @@ static char *panelize_section = "Panelize";
 static int last_listitem;
 
 struct {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
     char *tkname;
 } panelize_but[BUTTONS] = {
-    { B_CANCEL, 0, 53, "[ Cancel ]",  'c', 2, "c"},
-    { B_ADD,    0, 28, "[ Add new ]", 'a', 2, "a"},
-    { B_REMOVE, 0, 16, "[ Remove ]",  'r', 2, "r"},
-    { B_ENTER,  0,  0, "[[ Panelize ]]", 'p', 5, "p"},
+    { B_CANCEL, NORMAL_BUTTON, 0, 53, "&Cancel",  "c"},
+    { B_ADD, NORMAL_BUTTON,    0, 28, "&Add new", "a"},
+    { B_REMOVE, NORMAL_BUTTON, 0, 16, "&Remove",  "r"},
+    { B_ENTER, DEFPUSH_BUTTON, 0,  0, "&Panelize","p"},
 };
 
 /* Directory panelize */
@@ -101,7 +102,7 @@ static struct panelize{
 #ifndef HAVE_X
 static void panelize_refresh (void)
 {
-    attrset (REVERSE_COLOR);
+    attrset (COLOR_NORMAL);
     dlg_erase (panelize_dlg);
     
     draw_box (panelize_dlg, 1, 2, 20, 70);
@@ -162,7 +163,7 @@ static void init_panelize (void)
 			      DLG_CENTER|DLG_GRID);
     x_set_dialog_title (panelize_dlg, "External panelize");
 
-#define XTRACT(i) BY+panelize_but[i].y, BX+panelize_but[i].x, panelize_but[i].ret_cmd, panelize_but[i].text, panelize_but[i].hkey, panelize_but[i].hpos, 0, 0, panelize_but[i].tkname
+#define XTRACT(i) BY+panelize_but[i].y, BX+panelize_but[i].x, panelize_but[i].ret_cmd, panelize_but[i].flags, panelize_but[i].text, 0, 0, panelize_but[i].tkname
 
     for (i = 0; i < BUTTONS; i++)
 	add_widgetl (panelize_dlg, button_new (XTRACT (i)), (i == BUTTONS - 1) ?
