@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    Written by: 1995 Miguel de Icaza
-              1997 Timur Bakeyev
+               1997 Timur Bakeyev
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -60,10 +60,7 @@
 # include "drive.h"
 #endif
 
-#ifdef HAVE_TK
-#   include "tkmain.h"
-#   include "tkscreen.h"
-#endif
+#include "x.h"
 
 /* "$Id: screen.c,v 1.19 1995/02/21 19:07:10 miguel Exp $" */
 #define ELEMENTS(arr) ( sizeof(arr) / sizeof((arr)[0]) )
@@ -80,8 +77,12 @@ int torben_fj_mode = 0;
 /* If true, up/down keys scroll the pane listing by pages */
 int panel_scroll_pages = 1;
 
-/* If 1, we use permission hilighting, if 2 - then add per file type hilighting */
-int highlight_mode = 0;
+/* If 1, we use permission hilighting */
+int permission_mode = 0;
+
+/* If 1 - then add per file type hilighting */
+int filetype_mode = 1;
+
 
 /* This gives abilitiy to determine colored user priveleges */
 extern user_in_groups *current_user_gid;
@@ -162,9 +163,9 @@ add_permission_string (char *dest, int width, file_entry *fe, int attr, int colo
 }
 
 int
-normal_color (file_entry *fe)
+file_entry_color (file_entry *fe)
 {
-    if (highlight_mode == 2){
+    if (filetype_mode){
         if (S_ISDIR (fe->buf.st_mode))
             return (DIRECTORY_COLOR);
         else if (S_ISLNK (fe->buf.st_mode)) {
@@ -373,28 +374,29 @@ static struct {
     int  expands;
     int  default_just;
     char *title;
+    int  use_in_gui;
     char *(*string_fn)(file_entry *, int);
     sortfn *sort_routine;
 } formats [] = {
-{ "name",  12, 1, J_LEFT,  "Name",       string_file_name,       (sortfn *) sort_name },
-{ "size",  7,  0, J_RIGHT, "Size",       string_file_size,       (sortfn *) sort_size },
-{ "type",  1,  0, J_LEFT,  "",           string_file_type,       (sortfn *) sort_type },
-{ "mtime", 12, 0, J_RIGHT, "MTime",      string_file_mtime,      (sortfn *) sort_time },
-{ "bsize", 7,  0, J_RIGHT, "Size",      string_file_size_brief, (sortfn *) sort_size },
-{ "perm",  10, 0, J_LEFT,  "Permission", string_file_permission, NULL },
-{ "mode",  6,  0, J_RIGHT, "Perm",       string_file_perm_octal, NULL },
-{ "|",     1,  0, J_RIGHT, "|",          0,                      NULL },
-{ "nlink", 2,  0, J_RIGHT, "Nl",         string_file_nlinks,     (sortfn *) sort_links },
-{ "ngid",  5,  0, J_RIGHT, "GID",        string_file_ngid,       (sortfn *) sort_ngid },
-{ "nuid",  5,  0, J_RIGHT, "UID",        string_file_nuid,       (sortfn *) sort_nuid },
-{ "owner", 8,  0, J_LEFT,  "Owner",      string_file_owner,      (sortfn *) sort_owner },
-{ "group", 8,  0, J_LEFT,  "Group",      string_file_group,      (sortfn *) sort_group },
-{ "atime", 12, 0, J_RIGHT, "ATime",      string_file_atime,      (sortfn *) sort_atime },
-{ "ctime", 12, 0, J_RIGHT, "CTime",      string_file_ctime,      (sortfn *) sort_ctime },
-{ "space", 1,  0, J_RIGHT, " ",          string_space,           NULL },
-{ "dot",   1,  0, J_RIGHT, " ",          string_dot,             NULL },
-{ "mark",  1,  0, J_RIGHT, " ",          string_marked,          NULL },
-{ "inode", 5,  0, J_RIGHT, "Inode",      string_inode,           (sortfn *) sort_inode },
+{ "name",  12, 1, J_LEFT,  "Name",       1, string_file_name,       (sortfn *) sort_name },
+{ "size",  7,  0, J_RIGHT, "Size",       1, string_file_size,       (sortfn *) sort_size },
+{ "type",  1,  0, J_LEFT,  "",           1, string_file_type,       (sortfn *) sort_type },
+{ "mtime", 12, 0, J_RIGHT, "MTime",      1, string_file_mtime,      (sortfn *) sort_time },
+{ "bsize", 7,  0, J_RIGHT, "Size",       1, string_file_size_brief, (sortfn *) sort_size },
+{ "perm",  10, 0, J_LEFT,  "Permission", 1, string_file_permission, NULL },
+{ "mode",  6,  0, J_RIGHT, "Perm",       1, string_file_perm_octal, NULL },
+{ "|",     1,  0, J_RIGHT, "|",          0, 0,                      NULL },
+{ "nlink", 2,  0, J_RIGHT, "Nl",         1, string_file_nlinks,     (sortfn *) sort_links },
+{ "ngid",  5,  0, J_RIGHT, "GID",        1, string_file_ngid,       (sortfn *) sort_ngid },
+{ "nuid",  5,  0, J_RIGHT, "UID",        1, string_file_nuid,       (sortfn *) sort_nuid },
+{ "owner", 8,  0, J_LEFT,  "Owner",      1, string_file_owner,      (sortfn *) sort_owner },
+{ "group", 8,  0, J_LEFT,  "Group",      1, string_file_group,      (sortfn *) sort_group },
+{ "atime", 12, 0, J_RIGHT, "ATime",      1, string_file_atime,      (sortfn *) sort_atime },
+{ "ctime", 12, 0, J_RIGHT, "CTime",      1, string_file_ctime,      (sortfn *) sort_ctime },
+{ "space", 1,  0, J_RIGHT, " ",          0, string_space,           NULL },
+{ "dot",   1,  0, J_RIGHT, " ",          0, string_dot,             NULL },
+{ "mark",  1,  0, J_RIGHT, " ",          1, string_marked,          NULL },
+{ "inode", 5,  0, J_RIGHT, "Inode",      1, string_inode,           (sortfn *) sort_inode },
 };
 
 static char *
@@ -424,11 +426,36 @@ to_buffer (char *dest, int just_mode, int len, char *txt)
     return dest;
 }
 
+int
+file_compute_color (int attr, file_entry *fe)
+{
+    int color;
+    
+    switch (attr){
+    case SELECTED:
+	color = SELECTED_COLOR;
+	break;
+    case MARKED:
+	color = MARKED_COLOR;
+	break;
+    case MARKED_SELECTED:
+	color = MARKED_SELECTED_COLOR;
+	break;
+    case STATUS:
+	color = NORMAL_COLOR;
+	break;
+    case NORMAL:
+    default:
+	color = file_entry_color(fe);
+    }
+    return color;
+}
+
 /* Formats the file number file_index of panel in the buffer dest */
 void
 format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int isstatus)
 {
-    int      i, color, length, empty_line;
+    int      color, length, empty_line;
     char     *txt;
     char     *old_pos;
     char     *cdest = dest;
@@ -440,20 +467,7 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
     home       = (isstatus) ? panel->status_format : panel->format;   
     fe         = &panel->dir.list [file_index];
 
-    switch (attr){
-    case SELECTED:
-    case MARKED:
-    case MARKED_SELECTED:
-	color = sel_mark_color [attr];
-	break;
-    case STATUS:
-	color = NORMAL_COLOR;
-	break;
-    case NORMAL:
-    default:
-	color = normal_color(fe);
-    }
-
+    color = file_compute_color (attr, fe);
     for (format = home; format; format = format->next){
 	if (format->string_fn){
 	    if (empty_line)
@@ -472,9 +486,9 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
 	    
             attrset (color);
  
-            if (highlight_mode && !strcmp(format->id, "perm"))
+            if (permission_mode && !strcmp(format->id, "perm"))
                 add_permission_string (old_pos, format->field_len, fe, attr, color, 0);
-            else if (highlight_mode && !strcmp(format->id, "mode"))
+            else if (permission_mode && !strcmp(format->id, "mode"))
                 add_permission_string (old_pos, format->field_len, fe, attr, color, 1);
             else
 		addstr (old_pos);
@@ -613,7 +627,7 @@ display_mini_info (WPanel *panel)
 #endif
 }
 
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 void
 paint_dir (WPanel *panel)
 {
@@ -797,15 +811,7 @@ Xtry_to_select (WPanel *panel, char *name)
 	return;
 }
 
-#ifdef HAVE_TK
-void panel_update_cols (Widget *widget, int frame_size)
-{
-    /* X version gets this information from a different source */
-}
-#else
-/* Sets the cols variable in the widget structure according to the
- * settings selected by the user
- */
+#ifndef PORT_HAS_PANEL_UPDATE_COLS
 void
 panel_update_cols (Widget *widget, int frame_size)
 {
@@ -849,15 +855,19 @@ panel_save_name (WPanel *panel)
 static void
 panel_destroy (WPanel *p)
 {
+    int i;
+    
     char *name = panel_save_name (p);
 
     panel_save_setup (p, name);
     clean_dir (&p->dir, p->count);
 
-    free (p->user_format);
-    free (p->mini_status_format);
     delete_format (p->format);
     delete_format (p->status_format);
+    
+    free (p->user_format);
+    for (i = 0; i < LIST_TYPES; i++)
+	free (p->user_status_format [i]);
     free (p->dir.list);
     free (p->panel_name);
     free (name);
@@ -870,27 +880,6 @@ panel_format_modified (WPanel *panel)
     x_reset_sort_labels (panel);
 }
 
-void
-panel_update_format (WPanel *panel)
-{
-    char *err;
-    
-    panel->format = parse_display_format (panel, panel_format (panel),
-					  &err, 0);
-    if (err)
-	free (err);
-    
-    if (show_mini_info){
-        panel->status_format = parse_display_format (panel, mini_status_format (panel),
-						     &err, 1);
-	if (err)
-	    free (err);
-    }
-    
-    panel_format_modified (panel);
-    
-    panel_update_cols (&(panel->widget), panel->frame_size);
-}
 
 /* Panel creation */
 /* The parameter specifies the name of the panel for setup retieving */
@@ -899,7 +888,7 @@ panel_new (char *panel_name)
 {
     WPanel *panel;
     char *section;
-    char *err;
+    int i, err;
 
     panel = xmalloc (sizeof (WPanel), "panel_new");
 
@@ -926,14 +915,18 @@ panel_new (char *panel_name)
     panel->reverse     = 0;
     panel->dirty       = 1;
     panel->searching   = 0;
-    panel->dirs_marked         = 0;
-    panel->is_panelized = 0;
+    panel->dirs_marked = 0;
+    panel->is_panelized  = 0;
     panel->has_dir_sizes = 0;
+    panel->format	 = 0;
+    panel->status_format = 0;    
     panel->format_modified = 1;
     
     panel->panel_name = strdup (panel_name);
     panel->user_format = strdup (DEFAULT_USER_FORMAT);
-    panel->mini_status_format = strdup (DEFAULT_USER_FORMAT);
+    
+    for(i = 0; i < LIST_TYPES; i++)
+	panel->user_status_format [i] = strdup (DEFAULT_USER_FORMAT);
     
     panel->search_buffer [0] = 0;
     panel->frame_size = frame_half;
@@ -946,19 +939,20 @@ panel_new (char *panel_name)
     free (section);
 
     /* Load format strings */
-    panel->format = parse_display_format (panel, panel_format (panel), &err, 0);
-    if (err) free (err);
+    err = set_panel_formats (panel);
+    if (err){
+	if (err & 0x01){
+	    free (panel->user_format);
+	    panel->user_format = strdup (DEFAULT_USER_FORMAT);
+	}
+	if (err & 0x02){
+	    free (panel->user_status_format [panel->list_type]);
+	    panel->user_status_format [panel->list_type] = strdup (DEFAULT_USER_FORMAT);
+	}
+	set_panel_formats (panel);
+    }
     
-    if (show_mini_info){
-        panel->status_format = parse_display_format (panel, mini_status_format (panel), &err, 1);
-        if (err)
-	    free (err);
-    } else
-        panel->status_format = 0;
-    
-
     /* Load the default format */
-    
     panel->count = do_load_dir (&panel->dir, panel->sort_type,
 				panel->reverse, panel->case_sensitive, panel->filter);
     return panel;
@@ -1010,44 +1004,7 @@ panel_reload (WPanel *panel)
 	}
 }
 
-#ifdef HAVE_TK
-void
-paint_frame (WPanel *panel)
-{
-    char     *txt, buffer [30], new_buffer [80];
-    int      header_len, spaces, extra;
-    int      i = 0, j;
-    format_e *format;
-    
-    show_dir (panel);
-    x_sort_label_start (panel);
-
-    for (format = panel->format; format; format = format->next){
-	txt = format->title;
-
-	header_len = strlen (txt);
-	if (header_len > format->field_len){
-	    strcpy (buffer, txt);
-	    txt = buffer;
-	    txt [format->field_len] = 0;
-	    header_len = strlen (txt);
-	}
-	
-	spaces = (format->field_len - header_len) / 2;
-	extra  = (format->field_len - header_len) % 2;
-	sprintf (new_buffer, "%*s%-s%*s", spaces, "",
-		 txt, spaces+extra, "");
-
-	for (j = 0; j < ELEMENTS(formats); j++){
-	    if (strcmp (format->id, formats [j].id))
-		continue;
-	    x_add_sort_label (panel, i, new_buffer, format->id, formats [j].sort_routine);
-	}
-    }
-    panel->format_modified = 0;
-}
-
-#else
+#ifndef PORT_HAS_PAINT_FRAME
 void
 paint_frame (WPanel *panel)
 {
@@ -1158,41 +1115,25 @@ parse_panel_size (WPanel *panel, char *format, int isstatus)
 */
 
 format_e *
-parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
+parse_display_format (WPanel *panel, char *format, char **error, int isstatus, int *res_total_cols)
 {
-#define MAX_EXPAND 4
-    
-    format_e *darr, *old, *home = 0;   /* The formats we return */
-    int  expand_top = 0;               /* Max used element in expand */
-    int  usable_columns;               /* Usable columns in the panel */
-    int  total_cols = 0;               /* Used columns by the format */
-    char *expand_list [MAX_EXPAND];    /* Expand at most 4 fields. */
-    int  i, j;
-    int  set_justify;                  /* flag: set justification mode? */
-    int  justify = 0;                  /* Which mode. */
-    int  cols_save;
+    format_e *darr, *old, *home = 0;   	/* The formats we return */
+    int  total_cols = 0;		/* Used columns by the format */
+    int  set_justify;                  	/* flag: set justification mode? */
+    int  justify = 0;                  	/* Which mode. */
+    int  items = 0;			/* Number of items in the format */
+    int  i;
 
     *error = 0;
     
-    if (!format)
-	format = DEFAULT_USER_FORMAT;
-    
-    /* This makes sure that the panel and mini status full/half mode
-       setting is equal */
+    /*
+     * This makes sure that the panel and mini status full/half mode
+     * setting is equal
+     */
     format = parse_panel_size (panel, format, isstatus);
 
-    /* Status needn't to be split */
-    usable_columns = ((panel->widget.cols-2)/((isstatus)
-					      ? 1
-					      : (panel->split+1))) - (!isstatus && panel->split);
-    
-    /* Clean expand list */
-    for (i = 0; i < MAX_EXPAND; i++)
-        expand_list [i] = '\0';
-    
     while (*format){
 	int found = 0;
-	int expands = 1;
 
         darr = xmalloc (sizeof (format_e), "parse_display_format");
 	
@@ -1221,12 +1162,16 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 
 	    format += klen;
 
-            darr->field_len = formats [i].min_size;
-            darr->string_fn = formats [i].string_fn;
-            darr->title     = formats [i].title;
-            darr->id        = formats [i].id;
-
+	    if (formats [i].use_in_gui)
+	    	items++;
 
+	    darr->use_in_gui          = formats [i].use_in_gui;
+            darr->requested_field_len = formats [i].min_size;
+            darr->string_fn           = formats [i].string_fn;
+            darr->title               = formats [i].title;
+            darr->id                  = formats [i].id;
+	    darr->expand              = formats [i].expands;
+	    
 	    if (set_justify)
 		darr->just_mode = justify;
 	    else
@@ -1243,29 +1188,21 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 		/* If the size was specified, we don't want
 		 * auto-expansion by default
 		 */
-		expands = 0;
+		darr->expand = 0;
 		format++;
 		req_length = atoi (format);
-#if 0		
-		if (req_length > darr->field_len)
-#endif
-		    darr->field_len = req_length;
+		darr->requested_field_len = req_length;
 
 		format = skip_numbers (format);
 
 		/* Now, if they insist on expansion */
 		if (*format == '+'){
-		    expands = 1;
+		    darr->expand = 1;
 		    format++;
 		}
 
 	    }
 
-	    if (expands) {
-		/* Take note if it's expandable */
-		if (formats [i].expands && expand_top < MAX_EXPAND)
-		    expand_list [expand_top++] = formats [i].id;
-	    }
 	    break;
 	}
 	if (!found){
@@ -1279,7 +1216,48 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 	    format [pos] = old_char;
 	    return 0;
 	}
-	total_cols += darr->field_len;
+	total_cols += darr->requested_field_len;
+    }
+    
+    *res_total_cols = total_cols;
+    home->items = items;
+    return home;
+}
+
+format_e *
+use_display_format (WPanel *panel, char *format, char **error, int isstatus)
+{
+#define MAX_EXPAND 4
+    int  expand_top = 0;               /* Max used element in expand */
+    int  usable_columns;               /* Usable columns in the panel */
+    int  total_cols;
+    char *expand_list [MAX_EXPAND];    /* Expand at most 4 fields. */
+    int  i;
+    format_e *darr, *home;
+    
+    if (!format)
+	format = DEFAULT_USER_FORMAT;
+    
+    home = parse_display_format (panel, format, error, isstatus, &total_cols);
+
+    if (*error)
+	return 0;
+	    
+    /* Status needn't to be split */
+    usable_columns = ((panel->widget.cols-2)/((isstatus)
+					      ? 1
+					      : (panel->split+1))) - (!isstatus && panel->split);
+    
+    /* Clean expand list */
+    for (i = 0; i < MAX_EXPAND; i++)
+        expand_list [i] = '\0';
+    
+    
+    /* Look for the expandable fields and set field_len based on the requested field len */
+    for (darr = home; darr && expand_top < MAX_EXPAND; darr = darr->next){
+	darr->field_len = darr->requested_field_len;
+	if (darr->expand)
+	    expand_list [expand_top++] = darr->id;
     }
     
     /* If we used more columns than the available columns, adjust that */
@@ -1306,49 +1284,59 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
     if ((usable_columns > total_cols) && expand_top){
 	int spaces = (usable_columns - total_cols) / expand_top;
 	int extra  = (usable_columns - total_cols) % expand_top;
-	
-        for (i = 0; darr && i < expand_top; i++){
-	    for (darr = home; darr; darr = darr->next){
-		if (!strcmp(darr->id, expand_list [i])){
-		    darr->field_len += ( spaces + ((i == 0) ? extra : 0 ));
-		    break;
-                }
-            }
-        }
+
+	for (i = 0, darr = home; darr && (i < expand_top); darr = darr->next)
+		if (darr->expand){
+			darr->field_len += (spaces + ((i == 0) ? extra : 0));
+			i++;
+		}
     }
     return home;
 }
 
-/* Switches the panel to the mode specified in the format */
-char *
-set_panel_format (WPanel *p, char *format)
+/* Switches the panel to the mode specified in the format 	    */
+/* Seting up both format and status string. Return: 0 - on success; */
+/* 1 - format error; 2 - status error; 3 - errors in both formats.  */
+int
+set_panel_formats (WPanel *p)
 {
-    char *err;
-
-    if (p->format)
-	delete_format (p->format);
+    format_e *form;
+    char *err;    
+    int retcode = 0;
+    
+    form = use_display_format (p, panel_format (p), &err, 0);
+	
+    if (err){
+        free (err);
+        retcode = 1;
+    }
+    else {
+        if (p->format)
+    	    delete_format (p->format);
+		
+	p->format = form;
+    }
     
-    p->format = parse_display_format (p, format, &err, 0);
-
-    if (err)
-	return err;
-
     if (show_mini_info){
-	if (p->status_format)
-	    delete_format (p->status_format);
+    
+	form = use_display_format (p, mini_status_format (p), &err, 1);
 	
-	p->status_format = parse_display_format (p, mini_status_format (p), &err, 1);
-	
-	if (err)
-	    return err;
+	if (err){
+	    free (err);
+	    retcode += 2;
+	}
+	else {
+	    if (p->status_format)
+		delete_format (p->status_format);
+		
+	    p->status_format = form;
+	}
     }
     
     panel_format_modified (p);
     panel_update_cols (&(p->widget), p->frame_size);
-    paint_frame (p);
-    paint_panel (p);
     
-    return 0;
+    return retcode;
 }
 
 /* Given the panel->view_type returns the format string to be parsed */
@@ -1377,7 +1365,7 @@ char *
 mini_status_format (WPanel *panel)
 {
     if (panel->user_mini_status)
-       return panel->mini_status_format;
+       return panel->user_status_format [panel->list_type];
 
     switch (panel->list_type){
 
@@ -1385,7 +1373,7 @@ mini_status_format (WPanel *panel)
        return "full perm,space,nlink,space,owner,space,group,space,size,space,mtime,space,name";
 
     case list_brief:
-       return "half type,name";
+       return "half type,name,space,bsize,space,perm,space";
 
     case list_full:
        return "half type,name";
@@ -1906,12 +1894,14 @@ do_enter (WPanel *panel)
 	    if (vfs_current_is_local ()) 
 #endif	    
 	    {
-	        char *tmp = copy_strings (".", PATH_SEP_STR, selection (panel)->fname, 0);
+	        char *tmp = name_quote (selection (panel)->fname, 0);
+	        char *cmd = copy_strings (".", PATH_SEP_STR, tmp, 0);
 	        if (!confirm_execute || (query_dialog (" The Midnight Commander ",
 	                                    " Do you really want to execute? ",
-	                                         0, 2, " Yes ", " No ") == 0))
-	            execute (tmp);
+	                                         0, 2, "&Yes", "&No") == 0))
+	            execute (cmd);
 	        free (tmp);
+	        free (cmd);
 	    } 
 #ifdef USE_VFS	    	    
 	    else if (vfs_current_is_extfs ()) {
@@ -2013,7 +2003,6 @@ static key_map panel_keymap [] = {
 
     /* To quickly move in the panel */
     { ALT('g'),   goto_top_file },
-    { ALT('h'),   goto_middle_file },
     { ALT('r'),   goto_middle_file }, /* M-r like emacs */
     { ALT('j'),   goto_bottom_file },
 
@@ -2060,6 +2049,9 @@ panel_key (WPanel *panel, int key)
 	    return 1;
 	}
     }
+    if (torben_fj_mode && key == ALT('h'))
+	goto_middle_file (panel);
+
     /* We do not want to take a key press if nothing can be done with it */
     /* The command line widget may do something more usefull */
     if (key == KEY_LEFT)
@@ -2200,18 +2192,6 @@ mark_if_marking (WPanel *panel, Gpm_Event *event)
     return 0;
 }
 
-WPanel *
-get_other_panel (int line)
-{
-    if (get_other_type () != view_listing){
-	fprintf (stderr, "Fatal: used opanel with view_type != listing\n\r");
-	fprintf (stderr, "Report this error to mc-devel@nuclecu.unam.mx\n\r");
-	fprintf (stderr, "Error line number: %d\n\r", line);
-	exit (1);
-    }
-    return (WPanel *) get_panel_widget (get_other_index ());
-}
-
 void
 file_mark (WPanel *panel, int index, int val)
 {
@@ -2219,8 +2199,7 @@ file_mark (WPanel *panel, int index, int val)
     x_panel_select_item (panel, index, val);
 }
 
-#ifdef HAVE_TK
-
+#ifdef PORT_WANTS_GET_SORT_FN
 sortfn *
 get_sort_fn (char *name)
 {
