@@ -24,7 +24,7 @@ extern int startup_right_mode;
 #    define PROFILE_NAME     "mc.ini"
 #    define HOTLIST_FILENAME "mc.hot"
 #else
-#    define PROFILE_NAME     ".mc.ini"
-#    define HOTLIST_FILENAME ".mc.hot"
+#    define PROFILE_NAME     ".mc/ini"
+#    define HOTLIST_FILENAME ".mc/hotlist"
 #endif
 #endif
