@@ -394,21 +394,21 @@ void init_subshell (void)
 	switch (subshell_type)
 	{
 	    case BASH:
-		init_file = ".mc.bashrc";
+		init_file = ".mc/bashrc";
 		if (access (init_file, R_OK) == -1)
-		    init_file += 3;   /* Skip the ".mc" part ;-) */
+		    init_file = ".bashrc";
 
 		/* Make MC's special commands not show up in bash's history */
 		putenv ("HISTCONTROL=ignorespace");
 
 		/* Allow alternative readline settings for MC */
-		if (access (".mc.inputrc", R_OK) == 0)
-		    putenv ("INPUTRC=.mc.inputrc");
+		if (access (".mc/inputrc", R_OK) == 0)
+		    putenv ("INPUTRC=.mc/inputrc");
 
 		break;
 
 	    case TCSH:
-		init_file = ".mc.tcshrc";
+		init_file = ".mc/tcshrc";
 		if (access (init_file, R_OK) == -1)
 		    init_file += 3;
 		break;
@@ -694,7 +694,7 @@ int exit_subshell (void)
     
     if (subshell_state != INACTIVE && subshell_alive)
 	quit = !query_dialog (" Warning ", " The shell is still active."
-			      " Quit anyway? ", 0, 2, " Yes ", " No ");
+			      " Quit anyway? ", 0, 2, "&Yes", "&No");
 
 #if AIX_TCSH_CODE_BELOW_IS_IT_FIXED
     /* New Test code */
@@ -720,7 +720,7 @@ int exit_subshell (void)
 	    quit = TRUE;
 	else
 	    quit = !query_dialog (" Warning ", " There are stopped jobs."
-				  " Quit anyway? ", 0, 2, " Yes ", " No ");
+				  " Quit anyway? ", 0, 2, "&Yes", "&No");
 
 	synchronize ();
 	subshell_state = RUNNING_COMMAND;
@@ -741,34 +741,6 @@ int exit_subshell (void)
 
 /* }}} */
 
-/* {{{ quote_directory */
-void quote_directory (int fd, char *string)
-{
-    int q;
-
-    if (!string)
-	return;
-    
-    while (*string){
-	q = 0;
-	switch (*string){
-	case '"':  q = 1; break;
-	case '\\': q = 1; break;
-	case '\'': q = 1; break;
-	case ' ': q = 1; break;
-	case '!': q = 1; break;
-	}
-	if (q){
-	    char q = '\\';
-
-	    write (fd, &q, 1);
-	}
-	write (fd, string, 1);
-	string++;
-    }
-}
-
-/* }}} */
 /* {{{ do_subshell_chdir */
 /* If it actually changed the directory it returns true */
 void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
@@ -787,17 +759,22 @@ void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
     
     temp = strdup (directory);
 	
-    /* The initial space keeps this out of the command history (in bash) */
+    /* The initial space keeps this out of the command history (in bash
+       because we set "HISTCONTROL=ignorespace") */
     write (subshell_pty, " cd ", 4);
-    quote_directory (subshell_pty, temp);
+    if (*directory) {
+	temp = name_quote (directory, 0);
+	write (subshell_pty, temp, strlen (temp));    
+        free (temp);
+    } else
+	write (subshell_pty, "/", 1);
     write (subshell_pty, "\n", 1);
-    free (temp);
     
     subshell_state = RUNNING_COMMAND;
     feed_subshell (QUIETLY, FALSE);
     
     if (subshell_alive && strcmp (subshell_cwd, cpanel->cwd) && strcmp (cpanel->cwd, "."))
-	fprintf (stderr, "\rWarning: couldn't change to %s.\n\r", cpanel->cwd);
+	fprintf (stderr, "Warning: Couldn't change to %s.\n", cpanel->cwd);
 
     if (reset_prompt)
 	prompt_pos = 0;
