@@ -26,14 +26,14 @@ void do_enter_ca_mode (void)
 {
     if (!xterm_flag)
 	return;
-    fprintf (stderr, /* ESC_STR ")0" */ ESC_STR "7" ESC_STR "[?47h");
-    fflush (stderr);
+    fprintf (stdout, /* ESC_STR ")0" */ ESC_STR "7" ESC_STR "[?47h");
+    fflush (stdout);
 }
 
 void do_exit_ca_mode (void)
 {
     if (!xterm_flag)
 	return;
-    fprintf (stderr, ESC_STR "[?47l" ESC_STR "8" ESC_STR "[m");
-    fflush (stderr);
+    fprintf (stdout, ESC_STR "[?47l" ESC_STR "8" ESC_STR "[m");
+    fflush (stdout);
 }
