@@ -179,14 +179,20 @@ char *strip_ext(char *ss)
     return ss;
 }
 
-char *expand_format (char c)
+char *expand_format (char c, int quote)
 {
     WPanel *panel;
+    char *(*quote_func)(const char *, int);
+    
+    if (quote)
+	quote_func = name_quote;
+    else
+	quote_func = fake_name_quote;
 
     if (c == '%')
 	return strdup ("%");
     
-    if (c >= 'a')
+    if (islower (c))
 	panel = current_panel;
     else {
 	if (get_other_type () == view_listing){
@@ -197,17 +203,17 @@ char *expand_format (char c)
     if (!panel)
 	panel = cpanel;
 
-    c |= 0x20;
+    c = tolower (c);
     
     switch (c){
     case 'f': 
-    case 'p': return strdup (panel->dir.list [panel->selected].fname);
+    case 'p': return (*quote_func) (panel->dir.list [panel->selected].fname, 0);
     case 'b':
-	return strip_ext(strdup (panel->dir.list [panel->selected].fname));
-    case 'd': return strdup (panel->cwd);
+	return strip_ext((*quote_func) (panel->dir.list [panel->selected].fname, 0));
+    case 'd': return (*quote_func) (panel->cwd, 0);
     case 's':
 	if (!panel->marked)
-	    return strdup (panel->dir.list [panel->selected].fname);
+	    return (*quote_func) (panel->dir.list [panel->selected].fname, 0);
 
 	/* Fall through */
 
@@ -215,17 +221,18 @@ char *expand_format (char c)
     case 'u':
     {
 	int length = 2, i;
-	char *block;
+	char *block, *tmp;
 	
 	for (i = 0; i < panel->count; i++)
 	    if (panel->dir.list [i].f.marked)
 		length += strlen (panel->dir.list [i].fname) + 1;
 
-	block = xmalloc (length, "expand_format");
+	block = xmalloc (length*2+1, "expand_format");
 	*block = 0;
 	for (i = 0; i < panel->count; i++)
 	    if (panel->dir.list [i].f.marked){
-		strcat (block, panel->dir.list [i].fname);
+		strcat (block, tmp = (*quote_func) (panel->dir.list [i].fname, 0));
+		free (tmp);
 		strcat (block, " ");
 		if (c == 'u')
 		    do_file_mark (panel, i, 0);
@@ -514,6 +521,7 @@ void execute_menu_command (char *s)
 	if (parameter_found){
 	    if (*commands == '}'){
 		char *parameter;
+		char *tmp;
 		parameter_found = 0;
 		parameter = input_dialog (" Parameter ", prompt, "");
 		if (!parameter || !*parameter){
@@ -522,8 +530,9 @@ void execute_menu_command (char *s)
 		    unlink (file_name);
 		    return;
 		}
-		fputs (parameter, cmd_file);
+		fputs (tmp = name_quote (parameter, 0), cmd_file);
 		free (parameter);
+		free (tmp);
 	    } else {
 		int len = strlen (prompt);
 
@@ -538,7 +547,7 @@ void execute_menu_command (char *s)
 	    if (*commands == '{')
 		parameter_found = 1;
 	    else{
-		char *text = expand_format (*commands);
+		char *text = expand_format (*commands, 1);
 		fputs (text, cmd_file);
 		free (text);
 	    }
@@ -555,6 +564,39 @@ void execute_menu_command (char *s)
     unlink (file_name);
 }
 
+/* 
+**	Check owner of the menu file. Using menu file is allowed, if
+**	owner of the menu is root or the actual user. In either case
+**	file should not be group and word-writable.
+**	
+**	Q. Should we apply this routine to system and home menu (and .ext files)?
+*/
+static int
+menu_file_own(char* path)
+{
+	struct stat st;
+
+	if (stat (path, &st) == 0
+		&& (!st.st_uid || (st.st_uid == geteuid ()))
+		&& ((st.st_mode & (S_IWGRP | S_IWOTH)) == 0)
+	) {
+		return 1;
+	}
+	else
+	{
+		extern int verbose;
+		if (verbose)
+		{
+			message (0, " Warning -- ignoring file ",
+				"File %s is not owned by root or you or is world writable.\n"
+				"Using it may compromise your security",
+				path
+			);
+		}
+		return 0;
+	}
+}
+
 void user_menu_cmd (void)
 {
     char *menu, *p;
@@ -568,10 +610,14 @@ void user_menu_cmd (void)
 	return;
     }
     
-    menu = concat_dir_and_file (home_dir, MC_MENU);
-    if (!exist_file (menu)){
+    menu = strdup (MC_LOCAL_MENU);
+    if (!exist_file (menu) || !menu_file_own (menu)){
 	free (menu);
-	menu = copy_strings (mc_home, "mc.menu", 0);
+        menu = concat_dir_and_file (home_dir, MC_HOME_MENU);
+	if (!exist_file (menu)){
+	    free (menu);
+	    menu = concat_dir_and_file (mc_home, MC_GLOBAL_MENU);
+	}
     }
 
     if ((data = load_file (menu)) == NULL){
