@@ -46,8 +46,10 @@
 #	include <sys/timeb.h>	/* alex: for struct timeb, used in time.h */
 #endif /* SCO_FLAVOR */
 #include <time.h>
-#include <pwd.h>
-#include <grp.h>
+#ifndef OS2_NT
+#   include <pwd.h>
+#   include <grp.h>
+#endif
 #include <string.h>
 #include <ctype.h>
 #ifdef HAVE_SYS_SELECT_H
@@ -164,6 +166,56 @@ char *trim (char *s, char *d, int len)
     return d;
 }
 
+char *
+name_quote (const char *s, int quote_percent)
+{
+    char *ret, *d;
+    
+    d = ret = xmalloc (strlen (s)*2 + 2 + 1, "quote_name");
+    if (*s == '-') {
+        *d++ = '.';
+        *d++ = '/';
+    }
+
+    for (; *s; s++, d++) {
+	switch (*s)
+	{
+	    case '%':
+		if (quote_percent)
+		    *d++ = '%';
+		break;
+	    case '\'':
+	    case '\\':
+	    case '\r':
+	    case '\n':
+	    case '\t':
+	    case '$':
+	    case '?':
+	    case '*':
+	    case '(':
+	    case ')':
+	    case '[':
+	    case ']':
+	    case '"':
+	    case '!':
+	    case '&':
+	    case '#':
+	    case '`':
+	    case ' ':
+		*d++ = '\\';
+	}
+	*d = *s;
+    }
+    *d = '\0';
+    return ret;
+}
+
+char *
+fake_name_quote (const char *s, int quote_percent)
+{
+    return strdup (s);
+}
+
 /* If passed an empty txt (this usually means that there is an error)
  * in the upper layers, we return "/"
  */
@@ -286,7 +338,7 @@ strip_password (char *path)
 {
     char *at, *inner_colon, *dir;
     
-    if (dir = strchr (path, PATH_SEP))
+    if ((dir = strchr (path, PATH_SEP)) != NULL)
 	*dir = '\0';
     /* search for any possible user */
     at = strchr (path, '@');
@@ -520,7 +572,7 @@ int get_int (char *file, char *key, int def)
 
 int set_int (char *file, char *key, int value)
 {
-    char buffer [6];
+    char buffer [30];
 
     sprintf (buffer,  "%d", value);
     return WritePrivateProfileString (app_text, key, buffer, file);
@@ -823,7 +875,6 @@ int strcasecmp (const char *s, const char *d)
 }
 #endif /* HAVE_STRCASECMP */
 
-#ifndef HAVE_VFS
 /* getwd is better than getcwd, the later uses a popen ("pwd"); */
 char *get_current_wd (char *buffer, int size)
 {
@@ -836,7 +887,6 @@ char *get_current_wd (char *buffer, int size)
 #endif
     return p;
 }
-#endif
 
 #define CHECK(x) if (x == -1) return 0;
 
@@ -856,16 +906,18 @@ long get_small_endian_long (int fd)
 /* This function returns 0 if the file is not in gunzip format  */
 /* or how much memory must be allocated to load the gziped file */
 /* Warning: this function moves the current file pointer */
-long int is_gunzipable (int fd)
+long int is_gunzipable (int fd, int *type)
 {
-     unsigned char magic [4];
-
+    unsigned char magic [4];
+	
+    *type = ISGUNZIPABLE_GUNZIP;
+	
     /* Read the magic signature */
     CHECK (mc_read (fd, &magic [0], 1));
     CHECK (mc_read (fd, &magic [1], 1));
     CHECK (mc_read (fd, &magic [2], 1));
     CHECK (mc_read (fd, &magic [3], 1));
-    
+	
     /* GZIP_MAGIC and OLD_GZIP_MAGIC */
     if (magic [0] == 037 && (magic [1] == 0213 || magic [1] == 0236)){
 	/* Read the uncompressed size of the file */
@@ -873,34 +925,90 @@ long int is_gunzipable (int fd)
 	return get_small_endian_long (fd);
     }
 
-     /* PKZIP_MAGIC */
-     if (magic [0] == 0120 && magic [1] == 0113 && magic [2] == 003 &&
-	 magic [3] == 004){
-	 /* Read compression type */
-	 mc_lseek (fd, 8, SEEK_SET);
-	 CHECK (mc_read (fd, &magic [0], 1));
-	 CHECK (mc_read (fd, &magic [1], 1));
-	 /* Gzip can handle only deflated (8) or stored (0) files */
-	 if ((magic [0] != 8 && magic [0] != 0) || magic [1] != 0)
+    /* PKZIP_MAGIC */
+    if (magic [0] == 0120 && magic [1] == 0113 && magic [2] == 003 && magic [3] == 004){
+	/* Read compression type */
+	mc_lseek (fd, 8, SEEK_SET);
+	CHECK (mc_read (fd, &magic [0], 1));
+	CHECK (mc_read (fd, &magic [1], 1));
+	
+	/* Gzip can handle only deflated (8) or stored (0) files */
+	if ((magic [0] != 8 && magic [0] != 0) || magic [1] != 0)
 	     return 0;
-	 /* Read the uncompressed size of the first file in the archive */
-	 mc_lseek (fd, 22, SEEK_SET);
-	 return get_small_endian_long (fd);
-     }
+        /* Read the uncompressed size of the first file in the archive */
+	mc_lseek (fd, 22, SEEK_SET);
+	return get_small_endian_long (fd);
+    }
 
-     /* PACK_MAGIC and LZH_MAGIC and compress magic */
-     if (magic [0] == 037 &&
-	 (magic [1] ==  036 || magic [1] == 0240 || magic [1] == 0235)){
-	 
+    /* PACK_MAGIC and LZH_MAGIC and compress magic */
+    if (magic [0] == 037 && (magic [1] ==  036 || magic [1] == 0240 || magic [1] == 0235)){
 	 /* In case the file is packed, sco lzhed or compress_magic, the */
 	 /* program guesses that the uncompressed size is (at most) four */
 	 /* times the length of the compressed size, if the compression  */
 	 /* ratio is more than 4:1 the end of the file is not displayed  */
 	 return 4*mc_lseek (fd, 0, SEEK_END);
     }
+
+    /* BZIP and BZIP2 files */
+    if ((magic[0] == 'B') && (magic[1] == 'Z') &&
+	(magic [3] >= '1') && (magic [3] <= '9')){
+            switch (magic[2]) {
+                case '0':
+                    *type = ISGUNZIPABLE_BZIP;
+                    return 5*mc_lseek (fd, 0, SEEK_END);
+                case 'h': 
+	            *type = ISGUNZIPABLE_BZIP2;
+	            return 5*mc_lseek (fd, 0, SEEK_END);
+            }
+    }
     return 0;
 }
 
+char *
+decompress_command (int type)
+{
+	switch (type){
+	case ISGUNZIPABLE_GUNZIP:
+		return "gzip -cdf";
+		
+	case ISGUNZIPABLE_BZIP:
+		return "bzip -d";
+		
+	case ISGUNZIPABLE_BZIP2:
+		return "bzip2 -d";
+	}
+	/* Should never reach this place */
+	fprintf (stderr, "Fatal: decompress_command called with an unknown argument\n");
+	return 0;
+}
+
+void
+decompress_command_and_arg (int type, char **cmd, char **flags)
+{
+	switch (type){
+	case ISGUNZIPABLE_GUNZIP:
+		*cmd   = "gzip";
+		*flags = "-cdf";
+		return;
+
+	case ISGUNZIPABLE_BZIP:
+		*cmd   = "bzip";
+		*flags = "-d";
+		return;
+
+		
+	case ISGUNZIPABLE_BZIP2:
+		*cmd   = "bzip2";
+		*flags = "-d";
+		return;
+	}
+	*cmd   = 0;
+	*flags = 0;
+	
+	/* Should never reach this place */
+	fprintf (stderr, "Fatal: decompress_command called with an unknown argument\n");
+}
+
 /* Hooks */
 void add_hook (Hook **hook_list, void (*hook_fn)(void *), void *data)
 {
