@@ -36,7 +36,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdarg.h>
-#include <sys/wait.h>		/* my_system */
+#ifdef HAVE_SYS_WAIT_H
+#    include <sys/wait.h>	/* my_system */
+#endif
 #include <errno.h>		/* my_system */
 #include <time.h>
 #include <pwd.h>
@@ -80,20 +82,20 @@ void init_groups (void)
     struct group *grp;
     user_in_groups *cug, *pug;
 
-    current_user_uid = 0;      /* is this root.wheel */
-    current_user_gid = 0;      /* by default ? :-))  */
-
     pwd = getpwuid (current_user_uid=getuid ());
 
+    current_user_gid = (pug = xmalloc (sizeof (user_in_groups), "init_groups"));
+    current_user_gid->gid = getgid (); current_user_gid->next = 0;
+
+    if (pwd == 0)
+       return;
+    
     setgrent ();
     while (grp = getgrent ())
        for (i = 0; grp->gr_mem[i]; i++)
            if (!strcmp (pwd->pw_name,grp->gr_mem[i]))
                {
                cug = xmalloc (sizeof (user_in_groups), "init_groups");
-               if (!current_user_gid)
-                   current_user_gid = pug = cug;
-
                cug->gid  = grp->gr_gid;
                pug->next = cug;
                cug->next = 0;
@@ -794,7 +796,7 @@ int gettimeofday (struct timeval *tp, void *tzp)
 int
 errno_dir_not_empty (int err)
 {
-    if (err == ENOTEMPTY || errno == EEXIST)
+    if (err == ENOTEMPTY || err == EEXIST)
 	return 1;
     return 0;
 }
