@@ -6,6 +6,7 @@
                1994, 1995 Janne Kukonlehto
                1995 Jakub Jelinek
                1996 Joseph M. Hinkle
+	       1997 Norbert Warmuth
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +40,9 @@
 #   include <sys/mman.h>
 #endif
 #include <fcntl.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#endif
 #include <ctype.h>	/* For toupper() */
 #include <stdlib.h>	/* atoi() */
 #include <malloc.h>
@@ -114,9 +117,7 @@ int mouse_move_pages_viewer = 1;
 int have_fast_cpu = 0;
 
 int default_hex_mode = 0;
-#ifdef HEX_EDIT
 int default_hexedit_mode = 0;
-#endif /* HEX_EDIT */
 int default_magic_flag = 1;
 int default_nroff_flag = 0;
 int altered_hex_mode = 0;
@@ -271,7 +272,6 @@ get_byte (WView *view, int byte_index)
     }
 }
 
-#ifdef HEX_EDIT
 static void
 enqueue_change (struct hexedit_change_node **head, struct hexedit_change_node *node)
 {
@@ -398,25 +398,30 @@ static int
 view_ok_to_quit (WView *view)
 {
     int r;
-
+    char *text;
+    
     if (!view->change_list)
 	return 1;
     
     query_set_sel (1);
-    r = query_dialog (" Save changes ", view->filename, 2 | WITH_HOTKEYS, 3, "y Yes", "n No", "c Cancel");
-    if (r == 2)
-	return 0;
-
-    if (r == 1){
+    text = copy_strings ("File: \n\n    ", view->filename,
+			 "\n\nhas been modified, do you want to save the changes?\n", NULL);
+	    
+    r = query_dialog (" Save changes ", text, 2, 3, "&Yes", "&No", "&Cancel");
+    free (text);
+    
+    switch (r) {
+    case 0:
+	save_edit_changes (view);
+	return 1;
+    case 1:
 	free_change_list (view);
 	return 1;
+    default:
+	return 0;
     }
-    save_edit_changes (view);
-    return 1;
 }
 
-#endif /* HEX_EDIT */
-
 static char *
 set_view_init_error (WView *view, char *msg)
 {
@@ -474,10 +479,7 @@ init_growing_view (WView *view, char *name, char *filename)
 static char *load_view_file (WView *view, char *filename)
 {
     char *cmd;
-
-#ifndef HAVE_MMAP
-    int count;
-#endif
+    int  type;
 
     if ((view->file = mc_open (filename, O_RDONLY)) < 0){
 	set_view_init_error (view, 0);
@@ -512,15 +514,20 @@ static char *load_view_file (WView *view, char *filename)
     }
     
     /* First, try to open a compressed file */
-    if (!view->viewer_magic_flag && (is_gunzipable (view->file)) != 0){
+    if (!view->viewer_magic_flag && (is_gunzipable (view->file, &type)) != 0){
 	close_view_file (view);
 	if (vfs_file_is_local (filename)) {
-	    cmd = copy_strings ("gzip -dc ", "'", filename, "'", 0);
+	    char *tmp = name_quote (filename, 0);
+	    cmd = copy_strings (decompress_command (type), " ", tmp, 0);
+	    free (tmp);
 	}
 	else {
+	    char *tmp;
 	    if ((view->localcopy = mc_getlocalcopy (filename)) == 0)
 		return set_view_init_error (view, " Can not fetch local copy " );
-	    cmd = copy_strings ("gzip -dc ", "'", view->localcopy, "'", 0);
+	    tmp = name_quote (view->localcopy, 0);
+	    cmd = copy_strings (decompress_command (type), " ", tmp, 0);
+	    free (tmp);
 	}
         return init_growing_view (view, cmd, filename);
     }
@@ -549,6 +556,7 @@ static char *load_view_file (WView *view, char *filename)
 	*/
 	view->data = (unsigned char*) xmalloc (view->s.st_size, "load_view_file");
     if (view->data == (unsigned char*) 0
+		|| mc_lseek(view->file,0,0) != 0
 		|| mc_read(view->file, view->data, view->s.st_size) != view->s.st_size
 	) {
 		if (view->data != (unsigned char*)0)
@@ -633,13 +641,11 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
 	get_byte (view, 0);
 	move_forward (view, start_line - 1);
     }
-#ifdef HEX_EDIT
     view->edit_cursor = view->first;
     view->file_dirty =  0;
     view->nib_shift = 0;
     view->view_side = view_side_left;
     view->change_list = NULL;
-#endif /* HEX_EDIT */
 
     return 0;
 }
@@ -737,7 +743,9 @@ view_percent (WView *view, int p, int w)
 {
     fprintf (stderr, "Missing tk view_percent\n");
 }
-#else
+
+#else /* HAVE_TK */
+
 static void
 view_percent (WView *view, int p, int w)
 {
@@ -773,11 +781,9 @@ view_status (WView *view)
 					view->command ? view->command:"", i));
     	if (w > 30){
     	    widget_move (view, view->have_frame, 24);
-#ifdef HEX_EDIT
             if (view->hex_mode)
                 printw ("Offset 0x%08x", view->edit_cursor);
             else
-#endif /* HEX_EDIT */
     	    printw ("Col %d", -view->start_col);
     	}
     	if (w > 60){
@@ -790,16 +796,14 @@ view_status (WView *view)
 		addstr ("  [grow]");
 	}
         if (w - i > 4)
-#ifdef HEX_EDIT
             if (view->hex_mode)
                 view_percent (view, view->edit_cursor - view->first, w);
             else
-#endif /* HEX_EDIT */
 	    view_percent (view, view->start_display - view->first, w);
     }
     attrset (SELECTED_COLOR);
 }
-#endif
+#endif HAVE_TK
 
 #ifdef HAVE_TK
 
@@ -930,7 +934,6 @@ void
 view_gotoyx (WView *view, int row, int col)
 {
     if (row != view->current_line){
-	fprintf (stderr, "%d\n", row);
 	flush_line (view);
     }
     view->current_line = row;
@@ -1008,9 +1011,7 @@ display (WView *view)
     long from;
     int c;
     int boldflag = 0;
-#ifdef HEX_EDIT
-     struct hexedit_change_node *curr = view->change_list;
-#endif /* HEX_EDIT */
+    struct hexedit_change_node *curr = view->change_list;
  
     height = view->widget.lines - frame_shift;
     width = view->widget.cols - frame_shift;
@@ -1054,7 +1055,6 @@ display (WView *view)
 	    height -= 2;    
     }
     
-#ifdef HEX_EDIT
     /* Find the first displayable changed byte */
     while (curr) {
         if (curr->offset < from)
@@ -1062,7 +1062,6 @@ display (WView *view)
         else
             break;
     }
-#endif
     if (view->hex_mode){
         char hex_buff[10];   /* A temporary buffer for sprintf and mvwaddstr */
         int bytes;	     /* Number of bytes already printed on the line */
@@ -1083,7 +1082,6 @@ display (WView *view)
             hex_buff[2] = 0;
             for (bytes = 0; bytes < view->bytes_per_line
 		 && from < view->last_byte; bytes++, from++){
-#ifdef HEX_EDIT
                 /* Display and mark changed bytes */
                 if (curr && from == curr->offset) {
                     c = curr->value;
@@ -1091,7 +1089,6 @@ display (WView *view)
                     boldflag = 3;
                     set_color (7);
                 } else
-#endif /* HEX_EDIT */
                 c = (unsigned char) get_byte (view, from);
 		
 	    	if (view->found_len && from >= view->search_start
@@ -1099,7 +1096,6 @@ display (WView *view)
 	    	    boldflag = 1;
 		    set_color (BOLD_COLOR);
 	    	}
-#ifdef HEX_EDIT
                 /* Display the navigation cursor */
                 if (from == view->edit_cursor) {
 		    if (view->view_side == view_side_left){
@@ -1109,7 +1105,6 @@ display (WView *view)
                     boldflag = 2;
                     set_color (view->view_side == view_side_left ? 15 : 31);
                 }
-#endif /* HEX_EDIT */
 
                 /* Print a hex number (sprintf is too slow) */
                 hex_buff [0] = hex_char [(c >> 4)];
@@ -1117,22 +1112,14 @@ display (WView *view)
 		view_gotoyx (view, row, col);
                 add_string (view, hex_buff);
                 col += 3;
-#ifdef HEX_EDIT
                 /* Turn off the cursor or changed byte highlighting here */
                 if (boldflag > 1)
                     set_color (DEF_COLOR);
-#endif /* HEX_EDIT */
                 if ((bytes & 3) == 3 && bytes + 1 < view->bytes_per_line){
-#ifdef HEX_EDIT
                     /* Turn off the search highlighting */
                     if (boldflag == 1 &&
                             from == view->search_start + view->found_len - 1)
                         set_color (DEF_COLOR);
-#else /* HEX_EDIT */
-                
-                    if (boldflag && from == view->search_start + view->found_len - 1)
-                    	set_color (DEF_COLOR);
-#endif /* HEX_EDIT */
 		    
                     /* Hex numbers are printed in the groups of four */
                     /* Groups are separated by a vline */
@@ -1155,7 +1142,6 @@ display (WView *view)
 		
                 if (!is_printable (c))
                     c = '.';
-#ifdef HEX_EDIT
                 switch (boldflag) {
                     default:
                         break;
@@ -1174,7 +1160,6 @@ display (WView *view)
 			view->cursor_row = row;
 		    }
 		}
-#endif /* HEX_EDIT */
 		add_character (view, c);
 		
                 if (boldflag){
@@ -1322,17 +1307,10 @@ move_forward2 (WView *view, long current, int lines, long upto)
     int  line;
     int  col = 0;
 
-#ifdef HEX_EDIT
     if (!upto && !view->hex_mode && view->last == view->last_byte)
         return current;
-#else /* HEX_EDIT */
- 
-    if (!upto && view->last == view->last_byte)
-    	return current;
-#endif /* HEX_EDIT */
 
     if (view->hex_mode){
-#ifdef HEX_EDIT
         p = current + lines * view->bytes_per_line;
         p = (p >= view->last_byte) ? current : p;
         if (lines == 1) {
@@ -1349,9 +1327,6 @@ move_forward2 (WView *view, long current, int lines, long upto)
                 p : view->edit_cursor;
         }
         return p;
-#else /* HEX_EDIT */
-        return (p = current + lines * view->bytes_per_line) >= view->last_byte ? current : p;
-#endif /* HEX_EDIT */
     } else {
     	if (upto){
     	    lines = -1;
@@ -1417,17 +1392,11 @@ move_backward2 (WView *view, long current, int lines)
     long p, q;
     int line;
 
-#ifdef HEX_EDIT
     if (!view->hex_mode && current == view->first)
         return current;
-#else /* HEX_EDIT */
-    if (current == view->first)
-	return current;
-#endif /* HEX_EDIT */
     
     if (view->hex_mode){
 	p = current - lines * view->bytes_per_line;
-#ifdef HEX_EDIT
         p = (p < view->first) ? view->first : p;
         if (lines == 1) {
             q = view->edit_cursor - view->bytes_per_line;
@@ -1439,10 +1408,6 @@ move_backward2 (WView *view, long current, int lines)
                 p : view->edit_cursor;
         }
         return p;
-#else /* HEX_EDIT */
-
-        return (p < view->first) ? view->first : p;
-#endif /* HEX_EDIT */
     } else {
     	if (current == view->last_byte
 	    && get_byte (view, current - 1) != '\n')
@@ -1477,9 +1442,7 @@ move_backward (WView *view, int i)
     view->search_start = view->start_display =
 	move_backward2 (view, view->start_display, i);
     view->found_len = 0;
-#ifdef HEX_EDIT
     view->last = view->first + ((LINES-2) * view->bytes_per_line);
-#endif /* HEX_EDIT */
     view->dirty++;
 }
 
@@ -1523,9 +1486,7 @@ move_forward (WView *view, int i)
     	view->start_display = view->bottom_first;
     view->search_start = view->start_display;
     view->found_len = 0;
-#ifdef HEX_EDIT
     view->last = view->first + ((LINES-2) * view->bytes_per_line);
-#endif /* HEX_EDIT */
     view->dirty++;
 }
 
@@ -1535,15 +1496,9 @@ move_to_top (WView *view)
 {
     view->search_start = view->start_display = view->first;
     view->found_len = 0;
-#ifdef HEX_EDIT
     view->last = view->first + ((LINES-2) * view->bytes_per_line);
-#endif /* HEX_EDIT */
-    
-#ifdef HEX_EDIT
     view->nib_shift = 0;
     view->edit_cursor = view->start_display; 
-#endif /* HEX_EDIT */
-    
     view->dirty++;
 }
 
@@ -1552,11 +1507,9 @@ move_to_bottom (WView *view)
 {
     view->search_start = view->start_display = get_bottom_first (view, 0, 1);
     view->found_len = 0;
-#ifdef HEX_EDIT
     view->last = view->first + ((LINES-2) * view->bytes_per_line);
     view->edit_cursor = (view->edit_cursor < view->start_display) ?
         view->start_display : view->edit_cursor;
-#endif /* HEX_EDIT */
     view->dirty++;
 }
 
@@ -1564,17 +1517,10 @@ move_to_bottom (WView *view)
 static void
 move_right (WView *view)
 {
-#ifndef HEX_EDIT
-    if (wrap_mode)
-	return;
-    
-#else /* HEX_EDIT */
     if (wrap_mode && !view->hex_mode)
         return;
     if (view->hex_mode) {
-#ifdef HEX_EDIT
         view->last = view->first + ((LINES-2) * view->bytes_per_line);
-#endif /* HEX_EDIT */
 
         if (view->hex_mode && view->view_side == view_side_left) {
             view->nib_shift = 1 - view->nib_shift;
@@ -1588,7 +1534,6 @@ move_right (WView *view)
            move_forward(view, 1);
         }
     } else
-#endif /* HEX_EDIT */
     if (--view->start_col > 0)
 	view->start_col = 0;
     view->dirty++;
@@ -1597,11 +1542,6 @@ move_right (WView *view)
 static void
 move_left (WView *view)
 {
-#ifndef HEX_EDIT
-    if (wrap_mode)
-	return;
-    
-#else /* HEX_EDIT */
     if (wrap_mode && !view->hex_mode)
         return;
     if (view->hex_mode) {
@@ -1617,7 +1557,6 @@ move_left (WView *view)
            move_backward(view, 1);
         }
     } else
-#endif /* HEX_EDIT */
     if (++view->start_col > 0)
 	view->start_col = 0;
     view->dirty++;
@@ -1969,11 +1908,9 @@ hex_search (WView *view, char *text)
     
     view->search_start = pos + 1;
     view->found_len = block_len;
-#ifdef HEX_EDIT
     /* Set the edit cursor to the search position, left nibble */
     view->edit_cursor = view->search_start;
     view->nib_shift = 0;
-#endif /* HEX_EDIT */
     
     /* Adjust the file offset */
     view->start_display = (pos & (~(view->bytes_per_line-1)));
@@ -2058,7 +1995,6 @@ static void help_cmd (void)
 /* Both views */
 void toggle_wrap_mode (WView *view)
 {
-#ifdef HEX_EDIT
     if (view->hex_mode) {
         if (view->growing_buffer != 0) {
             return;
@@ -2072,11 +2008,7 @@ void toggle_wrap_mode (WView *view)
         view_labels (view);
         view->dirty++;
         view_update (view);
-#else /* HEX_EDIT */
-    if (view->hex_mode){
-    	get_bottom_first (view, 1, 0);
-#endif /* HEX_EDIT */
-	return;
+		return;
     } 
     wrap_mode = 1 - wrap_mode;
     get_bottom_first (view, 1, 0);
@@ -2119,14 +2051,12 @@ toggle_hex_mode (WView *view)
     view_update (view);
 }
 
-#ifdef HEX_EDIT
 /* Both views */
 void
 toggle_hexedit_mode(WView *view)
 {
     view->hexedit_mode = 1 - view->hexedit_mode;
 }
-#endif /* HEX_EDIT */
 
 /* Both views */
 void
@@ -2159,17 +2089,12 @@ regexp_search (WView *view, int direction)
     char *regexp = "";
     static char *old = 0;
 
-#ifdef HEX_EDIT
     /* This is really an F6 key handler */
     if (view->hex_mode){
         /* Save it without a confirmation prompt */
         save_edit_changes(view);
 	return;
     }
-#else /* HEX_EDIT */ 
-    if (view->hex_mode)
-	return;
-#endif /* HEX_EDIT */ 
     
     regexp = old ? old : regexp;
     regexp = input_dialog (" Search ", " Enter regexp:", regexp);
@@ -2258,18 +2183,11 @@ change_nroff (WView *view)
     view_update (view);
 }
 
-int
-view_save_changes (WView *view)
-{
-}
-
 /* Real view only */
 static void
 quit_cmd (WView *view)
 {
-#ifdef HEX_EDIT
     if (view_ok_to_quit (view))
-#endif /* HEX_EDIT */
 	view->widget.parent->running = 0;
 }
 
@@ -2286,16 +2204,11 @@ view_labels (WView *view)
     my_define (h, 5, "Line", goto_line, view);
     my_define (h, 6, view->hex_mode ? "Save" : "RxSrch", regexp_search_cmd, view);
 
-#ifdef HEX_EDIT
     my_define (h, 2, view->hex_mode ? view->hexedit_mode ?
                      view->view_side == view_side_left ? "EdText" : "EdHex" :
                      view->growing_buffer ? "" : "Edit" :
                      wrap_mode ? "UnWrap" : "Wrap",
                      toggle_wrap_mode, view);
-#else /* HEX_EDIT */
-    my_define (h, 2, view->hex_mode ? "" : wrap_mode ? "UnWrap" : "Wrap",
-	       toggle_wrap_mode, view);
-#endif /* HEX_EDIT */
    
     my_define (h, 7, view->hex_mode ? "HxSrch" : "Search",
 	       normal_search_cmd, view);
@@ -2352,7 +2265,6 @@ view_handle_key (WView *view, int c)
 
     set_monitor (view, off);
     
-#ifdef HEX_EDIT
     if (view->hex_mode) {
         switch (c) {
         case 0x09:		/* Tab key */
@@ -2400,7 +2312,6 @@ view_handle_key (WView *view, int c)
 	    }
 	}
     }
-#endif /* HEX_EDIT */
     
     if (check_left_right_keys (view, c))
 	return 1;
@@ -2422,6 +2333,7 @@ view_handle_key (WView *view, int c)
 	/* Continue search */
     case XCTRL('s'):
     case 'n':
+    case KEY_F(17):
 	if (view->last_search){
 	    (*view->last_search)(view, view->search_exp);
 	} else {
@@ -2630,7 +2542,7 @@ view_adjust_size (Dlg_head *h)
     WButtonBar *bar;
 
     /* Look up the viewer and the buttonbar, we assume only two widgets here */
-    view = (WView *) find_widget_type (h, view_callback);
+    view = (WView *) find_widget_type (h, (callback_fn) view_callback);
     bar  = (WButtonBar *) view->widget.parent->current->next->widget;
     widget_set_size (&view->widget, 0, 0, LINES-1, COLS);
     widget_set_size (&bar->widget, LINES-1, 0, 1, COLS);
@@ -2683,6 +2595,7 @@ view_hook (void *v)
 {
     WView *view = (WView *) v;
     WPanel *panel;
+    struct stat s;
 
     /* If the user is busy typing, wait until he finishes to update the
        screen */
@@ -2700,9 +2613,16 @@ view_hook (void *v)
 	panel = other_panel;
     else
 	return;
-	
-    if (!S_ISREG (panel->dir.list [panel->selected].buf.st_mode))
-	return;
+
+    if (S_ISLNK (panel->dir.list [panel->selected].buf.st_mode)){
+        if (mc_stat (panel->dir.list [panel->selected].fname, &s) != 0)
+	    return;
+	if (!S_ISREG (s.st_mode))
+	    return;
+    } else {
+        if (!S_ISREG (panel->dir.list [panel->selected].buf.st_mode))
+	    return;
+    }
     
     view_init (view, 0, panel->dir.list [panel->selected].fname, 0);
     display (view);
@@ -2779,9 +2699,7 @@ view_new (int y, int x, int cols, int lines, int is_panel)
     view->start_col = 0;
     view->dirty = 0;
     view->hex_mode = default_hex_mode;
-#ifdef HEX_EDIT
     view->hexedit_mode = default_hexedit_mode;
-#endif /* HEX_EDIT */
     view->viewer_magic_flag = default_magic_flag;
     view->viewer_nroff_flag = default_nroff_flag;
     view->view_quit = 0;
@@ -2807,11 +2725,9 @@ view_new (int y, int x, int cols, int lines, int is_panel)
 
 /* }}} */
 /* {{{ Emacs local variables */
-
 /*
    Cause emacs to enter folding mode for this file:
    Local variables:
    end:
    */
-
 /* }}} */
