@@ -1,8 +1,6 @@
 #ifndef __VIEW_H
 #define __VIEW_H
 
-#define  HEX_EDIT               /* Trial of HexEdit for v3.5.10 */
-
 #ifdef WANT_WIDGETS
 /* The growing buffers data types */
 typedef struct {
@@ -41,7 +39,6 @@ typedef struct {
 				/* For the case of WINCH we should reset it to -1 */
     long start_display;		/* First char displayed */
     int  start_col;		/* First displayed column, negative */
-#ifdef HEX_EDIT
     int  edit_cursor;           /* HexEdit cursor position in file */
     char hexedit_mode;          /* Hexidecimal editing mode flag */ 
     char nib_shift;             /* A flag for inserting nibbles into bytes */
@@ -51,7 +48,6 @@ typedef struct {
     int  cursor_col;		/* Cursor column */
     int  cursor_row;		/* Cursor row */
     struct hexedit_change_node *change_list;   /* Linked list of changes */
-#endif /* HEX_EDIT */
 
     int dirty;			/* Number of skipped updates */
 
@@ -138,8 +134,6 @@ extern int altered_nroff_flag;
 
 void view_adjust_size ();
 
-#ifdef HEX_EDIT
-
 /* A node for building a change list on change_list */
 struct hexedit_change_node {
    struct hexedit_change_node *next;
@@ -147,6 +141,4 @@ struct hexedit_change_node {
    unsigned char              value;
 };
 
-#endif /* HEX_EDIT */
-
 #endif	/* __VIEW_H */
