@@ -20,7 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>	/* For free() */
 #include <string.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
 #    include <termios.h>
 #endif
 #include "mad.h"
@@ -244,7 +244,7 @@ key_code_name_t key_name_conv_tab [] = {
     { KEY_PPAGE,     "pgup", "Page Up key" },
     { KEY_IC,        "insert", "Insert key" },
     { KEY_DC,        "delete", "Delete key" },
-    { ALT('\t'),     "complete", "Completion (M-tab)" },
+    { ALT('\t'),     "complete", "Completion/M-tab" },
     { KEY_KP_ADD,    "kpplus", "+ on keypad" },
     { KEY_KP_SUBTRACT,"kpminus", "- on keypad" },
     { KEY_KP_MULTIPLY,"kpasterix", "* on keypad" },
