@@ -1,4 +1,3 @@
-
 /* {{{  */
 
 /* {{{ Copyright Notice */
@@ -50,10 +49,7 @@
 #include "key.h"	/* For mi_getch() */
 #include "dialog.h"	/* For do_refresh() and my_wputs() */
 #include "complete.h"   /* INPUT_COMPLETE_CD */
-
-#if HAVE_TK
-#   include "tkmain.h"
-#endif
+#include "x.h"
 
 /* }}} */
 
@@ -72,14 +68,14 @@ dialog_repaint (struct Dlg_head *h, int back, int title_fore)
     attrset (title_fore);
     if (h->title){
 	dlg_move (h, 1, (h->cols-strlen (h->title))/2);
-	printw ("%s", h->title);
+	addstr (h->title);
     }
 }
 #endif
 void
 common_dialog_repaint (struct Dlg_head *h)
 {
-    dialog_repaint (h, REVERSE_COLOR, COLOR_HOT_NORMAL);
+    dialog_repaint (h, COLOR_NORMAL, COLOR_HOT_NORMAL);
 }
 
 int
@@ -113,12 +109,21 @@ static int listbox_callback (Dlg_head *h, int id, int msg)
 
 Listbox *create_listbox_window (int cols, int lines, char *title, char *help)
 {
-    int xpos, ypos;
+    int xpos, ypos, len;
     Listbox  *listbox = xmalloc (sizeof (Listbox), "create_listbox_window");
+    char* cancel_string = "&Cancel";
 
     /* Adjust sizes */
-    lines = (lines > LINES-6) ? lines = LINES - 6 : lines;
-    cols = cols > COLS-6 ? COLS - 6 : cols;
+    lines = (lines > LINES-6) ? LINES - 6 : lines;
+
+	if (title && (cols < (len = strlen(title) + 2)))
+		cols = len;
+
+	/* no &, but 4 spaces around button for brackets and such */
+	if (cols < (len = strlen(cancel_string) + 3))
+		cols = len;
+	
+    cols = cols > COLS-6 ? COLS-6 : cols;
 
     /* I'm not sure if this -2 is safe, should test it */
     xpos = (COLS-cols)/2;
@@ -132,7 +137,8 @@ Listbox *create_listbox_window (int cols, int lines, char *title, char *help)
     listbox->list = listbox_new (2, 2, cols, lines, listbox_finish, 0, "li");
 
     add_widget (listbox->dlg,
-	button_new (lines+3, (cols/2)-5, B_CANCEL, "[ Cancel ]",'c',2,0, 0, "c"));
+		button_new (lines+3, (cols/2 + 2) - len/2, 
+		B_CANCEL, NORMAL_BUTTON, cancel_string, 0, 0, "c"));
     add_widget (listbox->dlg, listbox->list);
 #ifndef HAVE_X
     listbox_refresh(listbox->dlg);
@@ -178,8 +184,9 @@ static int query_callback (struct Dlg_head *h, int Id, int Msg)
 	attrset (NORMALC);
 	dlg_erase (h);
 	draw_box (h, 1, 1, h->lines-2, h->cols-2);
+	attrset (HOT_NORMALC);
 	dlg_move (h, 1, (h->cols-strlen (info->header))/2);
-	printw ("%s", info->header);
+	addstr (info->header);
 	break;
 #endif
     }
@@ -209,13 +216,15 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 #endif
     
     /* set dialog colors */
+    query_colors [0] = (flags & D_ERROR) ? ERROR_COLOR :  Q_UNSELECTED_COLOR;
     query_colors [1] = (flags & D_ERROR) ? REVERSE_COLOR : Q_SELECTED_COLOR;
-    query_colors [0] = (flags & D_ERROR) ? ERROR_COLOR : Q_UNSELECTED_COLOR;
+    query_colors [2] = (flags & D_ERROR) ? ERROR_COLOR : COLOR_HOT_NORMAL;
+    query_colors [3] = (flags & D_ERROR) ? COLOR_HOT_NORMAL :  COLOR_HOT_FOCUS;
     
     if (count > 0){
 	va_start (ap, count);
 	for (i = 0; i < count; i++)
-	    win_len += strlen (va_arg (ap, char *)) + 2;
+	    win_len += strlen (va_arg (ap, char *)) + 6;
 	va_end (ap);
     }
 
@@ -241,48 +250,29 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 
     if (count > 0){
 
-	cols = (cols-win_len-2)/2 + 2;
+	cols = (cols-win_len-2)/2+2;
 	va_start (ap, count);
 	for (i = 0; i < count; i++){
 	    cur_name = va_arg (ap, char *);
-	    xpos = strlen (cur_name)+2;
+	    xpos = strlen (cur_name)+6;
 #ifndef HAVE_XVIEW
-    if (flags & WITH_HOTKEYS)
-	add_widget (query_dlg, button_new
-		    (lines-3, cols, B_USER+i, cur_name+1,
-		     cur_name [0], -1, 0, 0, NULL));
-    else {
-	char *s = cur_name;
-
-	while (*s == ' ')
-	    s++;
-
-	add_widget (query_dlg, button_new
-		    (lines-3, cols, B_USER+i, cur_name,
-		     *s, -1, 0, 0, NULL));
-    }
+	    add_widget (query_dlg, button_new
+			(lines-3, cols, B_USER+i, NORMAL_BUTTON, cur_name,
+			 0, 0, NULL));
 #else			 
 	    buttonnames [i] = cur_name;
 #endif			     
 	    cols += xpos;
 	    if (i == sel_pos)
-	        query_dlg->initfocus = query_dlg->current;
+		query_dlg->initfocus = query_dlg->current;
 	}
 	va_end (ap);
 
 #ifdef HAVE_XVIEW
-        for (i = count - 1; i >= 0; i--) {
-	    if (flags & WITH_HOTKEYS)
-		add_widgetl (query_dlg, button_new
-			    (0, 0, B_USER+i, buttonnames [i] + 1,
-			     *(buttonnames [i]), -1, 0, 0, NULL),
-			     i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
-	    else
-		add_widgetl (query_dlg, button_new
-			    (0, 0, B_USER+i, buttonnames [i],
-			     *(buttonnames [i]), -1, 0, 0, NULL), 
-			     i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
-	}  
+	for (i = count - 1; i >= 0; i--)
+	    add_widgetl (query_dlg, button_new
+			 (0, 0, B_USER+i, NORMAL_BUTTON, buttonnames [i], 0, 0, NULL), 
+			 i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
 #endif	
 	add_widget (query_dlg, label_new (2, 3, text, NULL));
 	
@@ -299,7 +289,7 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	destroy_dlg (query_dlg);
     } else {
 #ifdef HAVE_X
-	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, "OK", 0, -1, 0, 0, NULL),
+	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, NORMAL_BUTTON, "&Ok", 0, 0, NULL),
 	    XV_WLAY_CENTERROW);
 
 	add_widget (query_dlg, label_new (2, 3, text, NULL));
@@ -313,7 +303,7 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	}
 #else
 	add_widget (query_dlg, label_new (2, 3, text, NULL));
-	add_widget (query_dlg, button_new(0, 0, 0, "", 0, -1, 0, 0, NULL));
+	add_widget (query_dlg, button_new(0, 0, 0, HIDDEN_BUTTON, "-", 0, 0, NULL));
 #endif /* HAVE_X */
 	last_query_dlg = query_dlg;
     }
@@ -402,10 +392,11 @@ Chooser *new_chooser (int lines, int cols, char *help, int flags)
     
     if (button_lines){
 	add_widget (c->dialog, button_new (lines-button_lines+1,
-				    20, B_ENTER, "[ Remove ]", 'r', 2,
-				    remove_callback, c, "button-remove"));
+				20, B_ENTER, DEFPUSH_BUTTON, "&Remove",
+				remove_callback, c, "button-remove"));
 	add_widget (c->dialog, button_new (lines-button_lines+1,
-				    4, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "button-cancel"));
+				    4, B_CANCEL, NORMAL_BUTTON, "&Cancel",
+				    0, 0, "button-cancel"));
     }		    
     add_widget (c->dialog, c->listbox);
     return c;
@@ -431,13 +422,13 @@ static int quick_callback (struct Dlg_head *h, int id, int Msg)
     switch (Msg){
 #ifndef HAVE_X    
     case DLG_DRAW:
-	attrset (REVERSE_COLOR);
+	attrset (COLOR_NORMAL);
 	dlg_erase (h);
 	draw_box (h, 1, 1, h->lines-2, h->cols-2);
 	
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 2);
-	printw (h->data);
+	dlg_move (h, 1,((h->cols-strlen (h->data))/2));
+	addstr (h->data);
 	break;
 
     case DLG_KEY:
@@ -485,19 +476,18 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
 	
 	switch (qw->widget_type){
 	case quick_checkbox:
-	    widget = check_new (ypos, xpos, *qw->result, qw->text,
-				qw->hotkey, qw->hotkey_pos, qw->tkname);
+	    widget = check_new (ypos, xpos, *qw->result, qw->text, qw->tkname);
 	    break;
 
 	case quick_radio:
-	    r = radio_new (ypos, xpos, qw->hotkey, qw->str_result, 1, qw->tkname);
+	    r = radio_new (ypos, xpos, qw->hotkey_pos, qw->str_result, 1, qw->tkname);
 	    r->pos = r->sel = qw->value;
 	    widget = r;
 	    break;
 	    
 	case quick_button:
-	    widget = button_new (ypos, xpos, qw->value, qw->text,
-				qw->hotkey, qw->hotkey_pos, 0, 0, qw->tkname);
+	    widget = button_new (ypos, xpos, qw->value, (qw->value==B_ENTER) ? DEFPUSH_BUTTON : NORMAL_BUTTON,
+	    qw->text, 0, 0, qw->tkname);
 	    break;
 
 	    /* We use the hotkey pos as the field length */
@@ -566,24 +556,32 @@ char *real_input_dialog_help (char *header, char *text, char *help, char *def_te
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 6, 10, 1, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
+    { quick_button, 6, 10, 1, 0, "&Cancel", 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 3, 10, 1, 0, "[ Ok ]", 'o', 2, B_ENTER, 0, 0,
+    { quick_button, 3, 10, 1, 0, "&Ok", 0, B_ENTER, 0, 0,
 	  XV_WLAY_DONTCARE, "button-ok" },
-    { quick_input,  4, 80, 0, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input" },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
+    { quick_input,  4, 80, 0, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, 0 },
+    { quick_label,  3, 80, 2, 0, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
     int len;
     int i;
     int lines;
     char *my_str;
-    
+    char tk_name[64] = "inp";
+
+/* we need a unique name for tkname because widget.c:history_tool()
+   needs a unique name for each dialog - using the header is ideal */
+    strncpy (tk_name + 3, header, 60);
+    tk_name[63] = '\0';
+    quick_widgets[2].tkname = tk_name;
+
     len = max (strlen (header), msglen (text, &lines)) + 4;
     len = max (len, 64);
 
     if (strncmp (text, "Password", 8) == 0){
 	quick_widgets [INPUT_INDEX].value = 1;
+	tk_name[3]=0;
     } else {
 	quick_widgets [INPUT_INDEX].value = 0;
     }
@@ -621,14 +619,14 @@ int input_dialog_help_2 (char *header, char *text1, char *text2, char *help, cha
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 6, 10, 4, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
+    { quick_button, 6, 10, 4, 0, "Cancel", 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 3, 10, 4, 0, "[ Ok ]", 'o', 2, B_ENTER, 0, 0,
+    { quick_button, 3, 10, 4, 0, "Ok", 0, B_ENTER, 0, 0,
 	  XV_WLAY_DONTCARE, "button-ok" },
-    { quick_input,  4, 80, 4, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
-    { quick_label,  3, 80, 3, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
-    { quick_input,  4, 80, 3, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-lbl" },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-lbl" },
+    { quick_input,  4, 80, 4, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
+    { quick_label,  3, 80, 3, 0, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
+    { quick_input,  4, 80, 3, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-lbl" },
+    { quick_label,  3, 80, 2, 0, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-lbl" },
     { 0 } };
     
     int len;
