@@ -747,6 +747,6 @@ if {$use_separate_gui_files} {
     source $LIBDIR/gui.tcl
 }
 
-if {!$mc_running} {
+if {$mc_running == 0} {
     test_gui 
 }
