@@ -27,7 +27,9 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #include <errno.h>
 #ifdef SCO_FLAVOR
 #include <sys/timeb.h>	/* alex: for struct timeb definition */
@@ -191,7 +193,7 @@ static FILE *open_extfs_archive (int fstype, char *name, struct extfs_archive **
     struct stat mystat;
     struct extfs_archive *current_archive;
     struct extfs_entry *root_entry;
-    char *local_name = NULL;
+    char *local_name = NULL, *tmp = 0;
     const int uses_archive = extfs_need_archive [fstype];
     
     if (uses_archive == EFS_NEED_FILE){
@@ -202,10 +204,14 @@ static FILE *open_extfs_archive (int fstype, char *name, struct extfs_archive **
 	    if (local_name == NULL)
 		return NULL;
 	}
+        tmp = name_quote (name, 0);
     }
+
     
     cmd = copy_strings (LIBDIR "extfs/", extfs_prefixes [fstype], 
-                        " list ", local_name ? local_name : name, 0);
+                        " list ", local_name ? local_name : tmp, 0);
+    if (tmp)
+	free (tmp);
     result = popen (cmd, "r");
     free (cmd);
     if (result == NULL) {
@@ -296,6 +302,7 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
                     message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
                     free (buffer);
+		    pclose (extfsd);
                     return -1;
                 }
 		entry = (struct extfs_entry *) xmalloc (sizeof (struct extfs_entry), "Extfs: extfs_entry");
@@ -315,6 +322,7 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
 	                message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
 			free (buffer);
+			pclose (extfsd);
 	                return -1;
 	            } else {
 			entry->inode = pent->inode;
@@ -344,8 +352,10 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
 		    if (current_link_name != NULL && S_ISLNK (hstat.st_mode)) {
 	    		inode->linkname = current_link_name;
 	    		current_link_name = NULL;
-		    } else
+		    } else {
+			inode->mode &= ~S_IFLNK;
 	    		inode->linkname = NULL;
+		    }
 		    if (S_ISDIR (hstat.st_mode))
 	    		make_dot_doubledot (entry);
 	    	}
@@ -491,6 +501,38 @@ static char *extfs_get_path (char *inname, struct extfs_archive **archive, int i
     return local;
 }
 
+/* Returns allocated path (without leading slash) inside the archive  */
+static char *extfs_get_path_from_entry (struct extfs_entry *entry)
+{
+    struct list {
+	struct list *next;
+	char *name;
+    } *head, *p;
+    char *localpath;
+    size_t len;
+    
+    for (len = 0, head = 0; entry->dir; entry = entry->dir) {
+	p = xmalloc (sizeof  (struct list), "Extfs: list");
+        p->next = head;
+	p->name = entry->name;
+	head = p;
+	len += strlen (entry->name) + 1;
+    }
+    
+    localpath = xmalloc (len, "Extfs: localpath");
+    *localpath = '\0';
+    for ( ; head; ) {
+	strcat (localpath, head->name);
+	if (head->next)
+	    strcat (localpath, "/");
+	p = head;
+	head = head->next;
+	free (p);
+    }
+    return (localpath);
+}
+
+
 struct extfs_loop_protect {
     struct extfs_entry *entry;
     struct extfs_loop_protect *next;
@@ -656,14 +698,17 @@ static char *get_archive_name (struct extfs_archive *archive)
 void extfs_run (char *file)
 {
     struct extfs_archive *archive;
-    char *p, *q, *cmd;
+    char *p, *q, *cmd, *archive_name;
 
     if ((p = extfs_get_path (file, &archive, 0, 0)) == NULL)
 	return;
-    q = (*p == '/') ? p + 1 : p;
+    q = name_quote (((*p == '/') ? p + 1 : p), 0);
     
+    archive_name = name_quote (get_archive_name(archive), 0);
     cmd = copy_strings (LIBDIR "extfs/", extfs_prefixes [archive->fstype], 
-                        " run ", get_archive_name(archive), " ", q, 0);
+                        " run ", archive_name, " ", q, 0);
+    free (archive_name);
+    free (q);
     shell_execute(cmd, 0);
     free(cmd);
     free(p);
@@ -681,27 +726,31 @@ static void *extfs_open (char *file, int flags, int mode)
 	return NULL;
     q = (*p == '/') ? p + 1 : p;
     entry = extfs_find_entry (archive->root_entry, q, 0);
+    free (p);
     if (entry == NULL) {
-        free (p);
     	return NULL;
     }
     if ((entry = extfs_resolve_symlinks (entry)) == NULL) {
-        free (p);
 	return NULL;
     }
     if (S_ISDIR (entry->inode->mode)) {
     	extfserrno = EISDIR;
-    	free (p);
     	return NULL;
     }
     if (entry->inode->local_filename == NULL) {
-        char *cmd;
+        char *cmd, *archive_name;
         
         entry->inode->local_filename = strdup (tmpnam (NULL));
+	p = extfs_get_path_from_entry (entry);
+	q = name_quote (p, 0);
+	free (p);
+	archive_name = name_quote (get_archive_name (archive), 0);
         cmd = copy_strings (LIBDIR "extfs/", extfs_prefixes [archive->fstype], 
                             " copyout ", 
-                            get_archive_name (archive), 
+                            archive_name, 
                             " ", q, " ", entry->inode->local_filename, 0);
+	free (q);
+	free (archive_name);
 #ifndef SCO_FLAVOR
         if (system (cmd)) {
             free (entry->inode->local_filename);
@@ -721,7 +770,6 @@ static void *extfs_open (char *file, int flags, int mode)
 #endif /* SCO_FLAVOR */
         free (cmd);
     }
-    free (p);
     
     local_handle = open (entry->inode->local_filename, flags, mode);
     if (local_handle == -1) {
@@ -759,17 +807,23 @@ static int extfs_close (void *data)
     /* Commit the file if it has changed */
     if (file->has_changed){
 	struct extfs_archive *archive;
-	char   *archive_name;
+	char   *archive_name, *file_name;
 	char   *cmd;
+	char   *p;
 	
 	archive = file->archive;
-	archive_name = get_archive_name (archive);
+	archive_name = name_quote (get_archive_name (archive), 0);
+	p = extfs_get_path_from_entry (file->entry);
+	file_name = name_quote (p, 0);
+	free (p);
 	
 	cmd = copy_strings (LIBDIR "extfs/",
 			    extfs_prefixes [archive->fstype],
 			    " copyin ", archive_name, " ",
-			    file->entry->name, " ",
+			    file_name, " ",
 			    file->entry->inode->local_filename, 0);
+	free (archive_name);
+	free (file_name);
 	if (system (cmd))
 	    errno_code = EIO;
 	free (cmd);
