@@ -3,7 +3,8 @@
    
    Written by: 1995 Ching Hui
                1995 Jakub Jelinek
-               1995 Miguel de Icaza
+               1995, 1996, 1997 Miguel de Icaza
+	       1997 Norbert Warmuth
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,10 +89,10 @@
 
 #define UPLOAD_ZERO_LENGTH_FILE
 
-/* If defined, to get the proper information on symlinks, the program will
- * first CD and then LS instead of LS the directory directly
+/* If defined, to get the proper information on files and symlinks, the 
+ * program will first CD and then LS instead of LS the directory directly
  */
-#undef CD_THEN_LS_DOT
+#define CD_THEN_LS_DOT
 
 void print_vfs_message(char *, ...);
 
@@ -142,6 +143,7 @@ extern char *last_current_dir;
 char reply_str [80];
 
 static char *ftpfs_get_current_directory(struct ftpfs_connection *bucket);
+static int __ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path);
 
 /* Extract the hostname and username from the path */
 /* path is in the form: [user@]hostname:port/remote-dir, e.g.:
@@ -324,16 +326,16 @@ static int command (struct ftpfs_connection *bucket, int wait_reply,
     got_sigpipe = 0;
     enable_interrupt_key();
     status = write(sock, buf, strlen(buf));
-    disable_interrupt_key();
-    
     if (status < 0){
-	disable_interrupt_key();
 	code = 421;
 	if (errno == EPIPE){
 	    got_sigpipe = 1;
 	}
+	disable_interrupt_key();
 	return TRANSIENT;
     }
+    disable_interrupt_key();
+    
     if (wait_reply)
 	return getreply (sock, (wait_reply & WANT_STRING) ? reply_str : NULL, sizeof (reply_str)-1);
     return COMPLETE;
@@ -809,6 +811,7 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     qproxy(bucket)= 0;
     qupdir(bucket)= 0;
     qdcache(bucket)=0;
+    bucket->__inode_counter = 0;
     bucket->lock = 0;
     bucket->use_proxy = ftpfs_check_proxy (host);
     bucket->password = 0;
@@ -1221,9 +1224,17 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
     struct ftpentry *fe;
     
     print_vfs_message("Resolving symlink...");
-    
+
+#ifdef CD_THEN_LS_DOT
+    if (__ftpfs_chdir(bucket, dir->remote_path) != COMPLETE) {
+	print_vfs_message("ftpfs: CWD failed.");
+	return;
+    }
+    sock = open_data_connection (bucket, "LIST -lLa", ".", 0);
+#else    
     sock = open_data_connection (bucket, "LIST -lLa", 
 				 dir->remote_path, 0);
+#endif				 
     if (sock == -1) {
 	print_vfs_message("ftpfs: couldn't resolve symlink");
 	return;
@@ -1256,6 +1267,7 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
 		    if (fe->l_stat == NULL)
 			goto done;
 		    *fe->l_stat = s;
+                    (*fe->l_stat).st_ino = bucket->__inode_counter++;
 		    break;
 		}
 		if (r < 0)
@@ -1295,7 +1307,6 @@ get_line_interruptible (char *buffer, int size, int fd)
     return 0;
 }
 
-#if 0
 /* Return true if path is the same directoy as the one we are on now */
 static int
 ftpfs_same_dir (char *path, struct ftpfs_connection *bucket)
@@ -1306,7 +1317,6 @@ ftpfs_same_dir (char *path, struct ftpfs_connection *bucket)
 	return 1;
     return 0;
 }
-#endif
 
 static struct ftpfs_dir *
 retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
@@ -1344,7 +1354,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     has_symlinks = 0;
     print_vfs_message("ftpfs: Reading FTP directory...");
 #ifdef CD_THEN_LS_DOT
-    if (command(bucket, WAIT_REPLY, "CWD %s", remote_path) != COMPLETE) {
+    if (__ftpfs_chdir(bucket, remote_path) != COMPLETE) {
 	ftpfserrno = ENOENT;
 	print_vfs_message("ftpfs: CWD failed.");
 	return NULL;
@@ -1424,6 +1434,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 	    fe->local_filename = fe->remote_filename = NULL;
 	    fe->l_stat = NULL;
 	    fe->bucket = bucket;
+            (fe->s).st_ino = bucket->__inode_counter++;
 	    if (S_ISLNK(fe->s.st_mode))
 		has_symlinks = 1;
 	    
@@ -2327,30 +2338,40 @@ static int ftpfs_rename (char *path1, char *path2)
     return -1;
 }
 
+static int
+__ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path)
+{
+    int r;
+
+    if (ftpfs_same_dir (remote_path, bucket))
+	return COMPLETE;
+
+    r = command (bucket, WAIT_REPLY, "CWD %s", remote_path);
+    if (r != COMPLETE) {
+	ftpfserrno = EIO;
+    } else {
+	if (qcdir(bucket))
+	    free(qcdir(bucket));
+	qcdir(bucket) = strdup (remote_path);
+    }
+    return r;
+}
+
 static int ftpfs_chdir (char *path)
 {
     char *remote_path;
     struct ftpfs_connection *bucket;
-    int r = COMPLETE;
+    int r;
 
  retry:
     if (!(remote_path = ftpfs_get_path(&bucket, path)))
 	return -1;
     got_sigpipe = 0;
-    r = command (bucket, WAIT_REPLY, "CWD %s", remote_path);
-    if (got_sigpipe){
+    r = __ftpfs_chdir (bucket, remote_path);
+    free (remote_path);
+    if (got_sigpipe)
 	goto retry;
-    }
     
-    if (r != COMPLETE) {
-	ftpfserrno = EIO;
-	free(remote_path);
-    } else {
-	if (qcdir(bucket))
-	    free(qcdir(bucket));
-	qcdir(bucket) = remote_path;
-    }
-
     vfs_add_noncurrent_stamps (&ftpfs_vfs_ops, (vfsid) bucket, NULL);
     return r == COMPLETE ? 0 : -1;
 }
