@@ -35,38 +35,38 @@
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #include <fcntl.h>
 #include <string.h>
 #ifdef HAVE_LIMITS_H
-#include <limits.h>
+#    include <limits.h>
 #endif
 #ifndef NGROUPS_MAX
-#include <sys/param.h>
-#ifdef NGROUPS
-#define NGROUPS_MAX NGROUPS
-#endif
+#    include <sys/param.h>
+#    ifdef NGROUPS
+#        define NGROUPS_MAX NGROUPS
+#    endif
 #endif
 #ifdef HAVE_GRP_H
-#include <grp.h>
+#    include <grp.h>
 #endif
 #ifdef HAVE_SHADOW_H
-#include <shadow.h>
+#    include <shadow.h>
 #else
-#ifdef HAVE_SHADOW_SHADOW_H
-#include <shadow/shadow.h>
-#endif
+#    ifdef HAVE_SHADOW_SHADOW_H
+#        include <shadow/shadow.h>
+#    endif
 #endif
 #ifdef HAVE_CRYPT_H
-#include <crypt.h>
+#    include <crypt.h>
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <signal.h>
 #ifdef SCO_FLAVOR
-#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#     include <sys/timeb.h>	/* alex: for struct timeb definition */
 #endif /* SCO_FLAVOR */
 #include <time.h>
 #include <utime.h>
@@ -76,21 +76,21 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
-#if 0
-#include <sys/byteorder.h>
-#endif
 #ifdef HAVE_PMAP_SET
-#include <rpc/rpc.h>
-#include <rpc/pmap_prot.h>
-#ifdef HAVE_RPC_PMAP_CLNT_H
-#include <rpc/pmap_clnt.h>
-#endif
+#    include <rpc/rpc.h>
+#    include <rpc/pmap_prot.h>
+#    ifdef HAVE_RPC_PMAP_CLNT_H
+#        include <rpc/pmap_clnt.h>
+#    endif
 #endif
 
 /* Authentication include files */
 #include <pwd.h>
 #ifdef HAVE_PAM
 #    include <security/pam_misc.h>
+#    ifndef PAM_ESTABLISH_CRED
+#        define PAM_ESTABLISH_CRED PAM_CRED_ESTABLISH
+#    endif
 #endif
 
 #include "../src/fs.h"
@@ -674,32 +674,30 @@ mc_pam_conversation (int messages, const struct pam_message **msg,
 	return PAM_CONV_ERR;
     *resp = r;
     
-    for (status = PAM_SUCCESS; messages--; msg++){
+    for (status = PAM_SUCCESS; messages--; msg++, r++){
 	switch ((*msg)->msg_style){
 
 	case PAM_PROMPT_ECHO_ON: 
 	    r->resp = strdup (up->username);
 	    r->resp_retcode  = PAM_SUCCESS;
-	    r++;
 	    break;
 
 	case PAM_PROMPT_ECHO_OFF:
 	    r->resp = strdup (up->password);
 	    r->resp_retcode = PAM_SUCCESS;
-	    r++;
 	    break;
 
 	case PAM_ERROR_MSG:
-	    status = PAM_CONV_ERR;
+	    r->resp = NULL;
+	    r->resp_retcode = PAM_SUCCESS;
 	    break;
 
 	case PAM_TEXT_INFO:
+	    r->resp = NULL;
+	    r->resp_retcode = PAM_SUCCESS;
 	    break;
-
 	}
     }
-    if (status != PAM_SUCCESS)
-	*resp = NULL;
     return status;
 }
 
@@ -713,7 +711,6 @@ mc_pam_auth (char *username, char *password)
     pam_handle_t *pamh;
     struct user_pass up;
     int status;
-    int i = 0;
     
     up.username = username;
     up.password = password;
@@ -725,7 +722,7 @@ mc_pam_auth (char *username, char *password)
 	goto failed_pam;
     if ((status = pam_acct_mgmt (pamh, 0)) != PAM_SUCCESS)
 	goto failed_pam;
-    if ((status = pam_setcred (pamh, PAM_CRED_ESTABLISH)) != PAM_SUCCESS)
+    if ((status = pam_setcred (pamh, PAM_ESTABLISH_CRED)) != PAM_SUCCESS)
 	goto failed_pam;
     pam_end (pamh, status);
     return 0;
