@@ -27,7 +27,9 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #include <errno.h>
 #ifdef SCO_FLAVOR
 #include <sys/timeb.h>	/* alex: for struct timeb definition */
@@ -243,31 +245,55 @@ static INLINE int gzip_limit_ok (int size)
  * Returns: 0 on failure
  */
 static INLINE int load_compressed_tar (struct tarfs_archive *current_archive,
-					int size, int fd)
+					int size, int fd, int type)
 {	
     int pipehandle, i;
     long l, l2;
     union record *ur;
     pid_t p;
+    char  *cmd, *cmd_flags;
+    void *tmp;
     
     l2 = 0;
     current_archive->is_gzipped = targz_growing;
     size = (size + RECORDSIZE - 1) / RECORDSIZE * RECORDSIZE;
-    current_archive->block_first =
-	(union record *) xmalloc (size, "Tar.gz growing buffers");
     current_archive->count_first = size / RECORDSIZE;
     current_archive->count_blocks = 0;
     current_archive->block_ptr = NULL;
     current_archive->root_entry = NULL;
-    
-    pipehandle = mc_doublepopen (fd, -1, &p, "gzip", "gzip", "-cdf", NULL);
+
+    decompress_command_and_arg (type, &cmd, &cmd_flags);
+    pipehandle = mc_doublepopen (fd, -1, &p, cmd, cmd, cmd_flags, NULL);
     if (pipehandle == -1)
     {
-	free (current_archive->block_first);
 	free (current_archive);
 	mc_close (fd);
 	return 0;
     }
+
+    /* On some systems it's better to allocate the memory for the tar-file
+       this way:
+       - allocate some memory we don't need (size 100k).
+       - allocate the memory for the uncompressed tar-file (the size of 
+         this memory block can be really big - five times the size of the 
+         compressed tar-file). 
+       - free the 100k from the first step
+
+       Without the extra steps a few malloc/free implementations can't give 
+       back memory to the operating system when the memory for the tar-file is
+       freed after the vfs-timeout.
+     */
+    tmp = malloc (100*1024);
+    current_archive->block_first = (union record *) malloc (size);
+    if (tmp)
+       free (tmp); /* make a hole in size of 100k */
+    if (0 == current_archive->block_first) {
+    	mc_doublepclose (pipehandle, p);
+	free (current_archive);
+	mc_close (fd);
+	return 0;
+    }
+
     ur = current_archive->block_first;
     l = 0;
     while ((i = read (pipehandle, (char *) ur, RECORDSIZE)) == RECORDSIZE) {
@@ -310,7 +336,7 @@ static INLINE int load_compressed_tar (struct tarfs_archive *current_archive,
 
 /* Returns a file handle of the opened local tar file or -1 on error */
 static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
-					int size, int fd)
+					int size, int fd, int type)
 {
     FILE *f;
     char *command;
@@ -323,8 +349,9 @@ static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
     
     /* Some security is sometimes neccessary :) */        
     command = copy_strings ("touch ", current_archive->tmpname,
-			    " ; chmod 0600 ", current_archive->tmpname,
-                            " ; gzip -cdf 2>/dev/null >", current_archive->tmpname, NULL);
+			    " ; chmod 0600 ", current_archive->tmpname, " ; ",
+			    decompress_command (type),
+			    "2>/dev/null >", current_archive->tmpname, NULL);
     
     if ((f = popen (command, "w")) == NULL) {
 	mc_close (fd);
@@ -366,7 +393,7 @@ static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
 static int open_tar_archive (char *name, struct tarfs_archive **pparc)
 {
     static dev_t __tar_no = 0;
-    int result;
+    int result, type;
     long size;
     mode_t mode;
     struct tarfs_archive *current_archive;
@@ -386,16 +413,16 @@ static int open_tar_archive (char *name, struct tarfs_archive **pparc)
     current_archive->next = first_archive;
     current_archive->fd_usage = 0;
     current_archive->fd = -1;
-    size = is_gunzipable (result);
+    size = is_gunzipable (result, &type);
     mc_lseek (result, 0, SEEK_SET);
 
     /* Find out the method to handle this tar file */
     if (size > 0 && gzip_limit_ok (size)) {
-	if (load_compressed_tar (current_archive, size, result) == 0)
+	if (load_compressed_tar (current_archive, size, result, type) == 0)
 	    return -1;
 	result = 0;
     } else if (size > 0) {
-	result = uncompress_tar_file (current_archive, size, result);
+	result = uncompress_tar_file (current_archive, size, result, type);
 	if (result == -1)
 	    return -1;
     } else {
@@ -556,7 +583,10 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	    bcopy (data, bp, written);
 	    bp += written;
 	}
-	bp [hstat.st_size - 1] = 0;	/* just to make sure */
+#if 0
+	if (hstat.st_size > 1)
+	    bp [hstat.st_size - 1] = 0;	/* just to make sure */
+#endif
 	goto recurse;
     } else {
 	struct tarfs_entry *entry, *pent;
@@ -772,6 +802,7 @@ char *tarfs_analysis (char *name, char **archive, int is_dir)
                   {4, ".tpz"},
 		  {6, ".tar.z"},
 		  {7, ".tar.bz"},
+		  {8, ".tar.bz2"},
                   {6, ".tar.Z"} };
     char *p, *local;
     unsigned int i;
