@@ -30,6 +30,7 @@
 #include <xview/cursor.h>
 #include <xview/defaults.h>
 #include <xview/dragdrop.h>
+#include <X11/xpm.h>
 
 #include <stdio.h>
 
@@ -50,6 +51,7 @@ int fdXConnection;
 int displayWidth;
 int displayHeight;
 int defaultDepth;
+int screen_no;
 Server_image iconRegular [2];
 Server_image iconDirectory [2];
 Cms mccms;
@@ -86,7 +88,7 @@ extern int containers_no;
 extern int is_right;
 extern Dlg_head *midnight_dlg;
 
-void get_panel_color (Cms cms, int i)
+void get_panel_color (Xv_Singlecolor *colors, int i)
 {
     static char *defcolors [] = {
         "black", "gray75", /* normal */
@@ -100,13 +102,16 @@ void get_panel_color (Cms cms, int i)
     char buffer [40], buffer2 [40];
     Xv_singlecolor col;
     int red, green, blue;
+    XColor xcolor, scolor;
     
     sprintf (buffer, "mxc.panelcolor.%s", names [i]);
     sprintf (buffer2, "Mxc.PanelColor.%s", names [i]);
     p = defaults_get_string (buffer, buffer2, "");
-    if (p != NULL)
+    if (p != NULL) {
         while (*p == ' ' || *p == '\t')
             p++;
+    }
+    
     if (p != NULL && *p) {
         if (sscanf (p, "%i %i %i", &red, &green, &blue) == 3) {
             col.red = (u_char) red;
@@ -120,24 +125,38 @@ void get_panel_color (Cms cms, int i)
     } else {
         p = defcolors [i];
     }
-    if (p == NULL)
-        xv_set (cms,
-            CMS_INDEX, CMS_CONTROL_COLORS + i,
-            CMS_COLOR_COUNT, 1,
-            CMS_COLORS, &col,
-            NULL);
-    else
-        xv_set (cms,
-            CMS_INDEX, CMS_CONTROL_COLORS + i,
-            CMS_COLOR_COUNT, 1,
-            CMS_NAMED_COLORS, 
-                p, NULL,
-            NULL);
+    
+    if (p == NULL) {
+      colors->red = col.red;
+      colors->green = col.green;
+      colors->blue = col.blue;
+    }
+    else {
+       XLookupColor (dpy, DefaultColormap(dpy, screen_no), p,
+		     &xcolor, &scolor);
+       colors->red = (unsigned char)(xcolor.red >> 8);
+       colors->green = (unsigned char)(xcolor.green >> 8);
+       colors->blue = (unsigned char)(xcolor.blue >> 8);
+    }
 }
 
+#include "Directory.xpm"
+#include "Regular.xpm"
+
 int xtoolkit_init(int *argc, char **argv)
 {
-    int screen_no, i;
+    int i;
+    char *mycolors[] = {
+      "LawnGreen", "gold", "brown", "sienna",
+      "salmon", "coral", "violet", "orchid",
+      "cornsilk", "azure", "misty rose", "snow",
+      "black", "red4", "green4", "yellow4",
+      "blue4", "magenta4", "cyan4", "gray75",
+      "gray50", "red", "green", "yellow",
+      "blue", "magenta", "cyan", "white", "black" };
+    Xv_singlecolor my_xv_colors[29];
+    XColor exact_c, screen_c;
+    Pixmap pixmap, shapemask;
 
     xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 
     	XV_ERROR_PROC, xv_error_handler,
@@ -155,29 +174,29 @@ int xtoolkit_init(int *argc, char **argv)
     defaultDepth = DefaultDepth (dpy, screen_no);
     fdXConnection = XConnectionNumber (dpy);
     xv_dispatch_a_bit ();
+
+    /* set up the array of Xv_singlecolors */
+    for (i = 0; i < 29; ++i) {
+        XLookupColor (dpy, DefaultColormap(dpy, screen_no), mycolors[i],
+                      &exact_c, &screen_c);
+        my_xv_colors[i].red = (unsigned char)(exact_c.red >> 8);
+        my_xv_colors[i].green = (unsigned char)(exact_c.green >> 8);
+        my_xv_colors[i].blue = (unsigned char)(exact_c.blue >> 8);
+    }
+
+    defaults_load_db (APP_DEFAULTS);
+    for (i = 0; i < 8; i++)
+      get_panel_color (&my_xv_colors[i], i);
+       
     mccms = (Cms) xv_create (XV_NULL, CMS,
         CMS_CONTROL_CMS, TRUE,
-	CMS_SIZE, 33,
+	CMS_SIZE, 29 + CMS_CONTROL_COLORS,
 	CMS_NAME, "mccms",
-        CMS_TYPE, XV_DYNAMIC_CMS,
-	CMS_NAMED_COLORS,
-	    "LawnGreen", "gold", "brown", "sienna",
-	    "salmon", "coral", "violet", "orchid",
-	    "cornsilk", "azure", "misty rose", "snow",
- 	    "black", "red4", "green4", "yellow4",
-	    "blue4", "magenta4", "cyan4", "gray75",
-	    "gray50", "red", "green", "yellow",
-	    "blue", "magenta", "cyan", "white",
-	    "black",
-	    NULL,
+	CMS_COLORS, my_xv_colors,
 	NULL);
 	
-     defaults_load_db (APP_DEFAULTS);
-     for (i = 0; i < 8; i++)
-     	get_panel_color (mccms, i);
-
+	/* this seems never to be used ?? -- BD */	
      cmsicon = (Cms) xv_create(XV_NULL, CMS,
- 			CMS_TYPE, XV_DYNAMIC_CMS,
  			CMS_SIZE, 18,
  			CMS_COLORS, icon_colors,
  			CMS_NAME, "iconcms",
@@ -201,32 +220,41 @@ int xtoolkit_init(int *argc, char **argv)
 	    SERVER_IMAGE_X_BITS, dropcursor_bits,
 	    NULL),
 	NULL);
+
+    XpmCreatePixmapFromData (dpy, (Drawable)xv_get(mcframe, XV_XID),
+			     Regular_xpm, &pixmap, &shapemask, NULL);
+
     iconRegular [0] = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,
         XV_WIDTH, 16,
 	XV_HEIGHT, 16,
         SERVER_IMAGE_COLORMAP, "mccms",
-	SERVER_IMAGE_DEPTH, 8,
-	SERVER_IMAGE_BITS, Regular_bits,
+	  SERVER_IMAGE_DEPTH, defaultDepth,
+	  SERVER_IMAGE_PIXMAP, pixmap,
 	NULL);
 
     iconRegular [1] = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,
 	XV_WIDTH, 16,
 	XV_HEIGHT, 16,
-	SERVER_IMAGE_X_BITS, Regular_mask,
+	  SERVER_IMAGE_PIXMAP, shapemask,
 	NULL);
 
+    XpmCreatePixmapFromData (dpy, (Drawable)xv_get(mcframe, XV_XID),
+			     Directory_xpm, &pixmap, &shapemask, NULL);
+
      iconDirectory [0] = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,
  	XV_WIDTH, 16,
  	XV_HEIGHT, 16,
  	SERVER_IMAGE_COLORMAP, "mccms",
- 	SERVER_IMAGE_DEPTH, 8,
- 	SERVER_IMAGE_X_BITS, Directory_bits,
+ 	SERVER_IMAGE_DEPTH, defaultDepth,
+	SERVER_IMAGE_PIXMAP, pixmap,
      	NULL);
+    
      iconDirectory [1] = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,
      	XV_WIDTH, 16,
      	XV_HEIGHT, 16,
-     	SERVER_IMAGE_X_BITS, Directory_mask,
+        SERVER_IMAGE_PIXMAP, shapemask,
      	NULL);
+     
     xv_set (splitcursor,
         XV_INCREMENT_REF_COUNT,
 	NULL);
@@ -671,6 +699,7 @@ static Notify_value x_container_item_interpos (Xv_opaque item, Event *event,
 }
 
 extern Menu MenuBar []; /* from main.c */
+#include "panel_icon.xpm"
 
 widget_data x_create_panel_container (int which)
 {
@@ -679,6 +708,10 @@ widget_data x_create_panel_container (int which)
     Canvas leftcanvas, rightcanvas;
     Canvas division;
     int panelsplit = 0;
+    Pixmap pixmap, mask;
+    
+    XpmCreatePixmapFromData (dpy, (Drawable)xv_get(mcframe, XV_XID),
+			    panel_icon_xpm, &pixmap, &mask, NULL);
     
     panelframe = (Frame) xv_create (mcframe, FRAME,
     	XV_WIDTH, displayWidth * 2 / (3 * containers_no),
@@ -693,8 +726,8 @@ widget_data x_create_panel_container (int which)
 	        XV_WIDTH, 64,
 	        XV_HEIGHT, 64,
 	        SERVER_IMAGE_COLORMAP, "iconcms",
-	        SERVER_IMAGE_DEPTH, 8,
-	        SERVER_IMAGE_BITS, icon_bits,
+	        SERVER_IMAGE_DEPTH, defaultDepth,
+	        SERVER_IMAGE_PIXMAP, pixmap,
 	        NULL),
 	    NULL),	
         NULL);
@@ -1035,3 +1068,42 @@ int move (int y, int x)
 {
     return 0;
 }
+
+void
+x_init_dlg (Dlg_head *h)
+{
+	if (h != midnight_dlg)
+		xvdlg_show(h, h->wdata);
+	else {
+		int i;
+		for (i = 0; i < containers_no; i++)
+			x_panel_container_show (containers [i]);
+	}
+}
+
+void
+x_destroy_dlg (Dlg_head *h)
+{
+}
+
+void
+x_focus_widget (Widget_Item *p)
+{
+}
+
+void
+edition_post_exec (void)
+{
+}
+
+void
+edition_pre_exec  (void)
+{
+    if (clear_before_exec)
+	clr_scr ();
+    else {
+	if (!(console_flag || xterm_flag))
+	    printf ("\n\n");
+    }
+
+}
