@@ -43,6 +43,7 @@
 #include "xvmain.h"
 #include "xvscreen.h"
 #include "util.h"
+#include "ext.h" /* regex_command */
 
 void do_enter (WPanel *panel);
 
@@ -242,7 +243,7 @@ struct copy_move_in_panel {
     WPanel *from;
 };
 
-static copymove_in_panel (struct copy_move_in_panel *cmip)
+static void copymove_in_panel (struct copy_move_in_panel *cmip)
 {
     WPanel *panel = current_panel;
     
@@ -265,12 +266,12 @@ struct user_drop {
     char *drops;
 };
 
-static user_drop (struct user_drop *usdr)
+static void user_drop (struct user_drop *usdr)
 {
     WPanel *panel = current_panel;
     
     xv_set_current_panel (usdr->panel);
-    regex_command (usdr->filename, "Drop", usdr->drops);
+    regex_command (usdr->filename, "Drop", usdr->drops, NULL);
     free (usdr->drops);
     free (usdr);
     xv_set_current_panel (panel);
@@ -306,7 +307,7 @@ static void start_drag (Xv_window paint, Event *event, Canvas canvas)
     Selection_item sel_item;
     WPanel *panel;
     char *p, *q;
-    int i, copy = (event_shift_is_down (event) || event_ctrl_is_down (event));
+    int i = (event_shift_is_down (event) || event_ctrl_is_down (event));
     int trailslash;
     Xv_Server server = XV_SERVER_FROM_WINDOW (paint);
     
@@ -569,7 +570,7 @@ static void xv_panel_repaint_item (WPanel *panel, int idx, Xv_Window paint_windo
     file_entry *fe = & panel->dir.list [idx];
     Drawable xid = (Drawable) xv_get (paint_window, XV_XID);
     unsigned long fg, bg, *xc;
-    GC gc = (GC) (panel->gc);
+    GC gc = (GC) (panel->gc); /* where does this get set? -- BD */
     Rect rect;
     char *cache = fe->cache;
     XPoint xp [3];
@@ -832,11 +833,10 @@ static char *xv_filedepmenu_free = NULL;
 static void xv_filedepmenu_cmd (char *todo)
 {
     char *p;
-    int  movedir;
     
     p = strchr (todo, ' ');
     *p = 0;
-    regex_command (p + 1, todo, NULL, &movedir);
+    regex_command (p + 1, todo, NULL, NULL);
     free (todo);
 }
 
